#------------------------------------------------- {COPYRIGHT-TOP-RM}
# Licensed Materials - Property of IBM
#
# "Restricted Materials of IBM"
#
# 5724-C51
#
#  Copyright IBM Corp. 1999,2008 All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#------------------------------------------------- {COPYRIGHT-END-RM}
# FILE:  archive/scripts/absconfig.ksh, tbsm_base, tbsm_base31e
# SID:   1.8.8.2
# Date Modified: 08/03/03 05:42:06

# see Usage()

Usage()
{

  cat <<USAGE_DOC

Usage: $0 ( -o | -i ) -S<db_server> -U<db_user> -P<db_password>

  This script is used to output or input the configuration file for automatically creating 
    business system views.

Mandatory Argument
  
  -o<filename>                Output configuration file from database
  -i<filename>                Input configuration file into database
  -S<db_server>               Specifies server name where database resides.  Default=[$db_server].
  -U<db_user>                 Specifies database user name.  Default=[$db_user].
  -P<db_password>             Specifies password for database user.  Default=[$db_password].

Optional Argument

  -t                          Override backup of ABS configuration tables before loading new config file.
                              This can be useful to speed up ABS configuration processing by truncating 
                              tables holding ABS configuration information.  If -t option is specified, 
                              previously loaded configuration will not be restored in case ABSConfig.ksh 
                              script fails to load a new configuration.
  -c<codepage>                Specify the codepage to interpret <filename>.  Default=[$sCodepage].
                              OEM and ACP are acceptable values and will default to the system values.

                                

Examples:

  The following will output the current automatic business system configuration from the 
    database:

  $0 -ocurrentConfiguation.txt 

  The following will input a new automatic business system configuration into the 
    database:
 
  $0 -inewConfiguration.txt

NOTE: MAKE SURE THAT THE FOLLOWING JOBS ARE STOPPED IN THE SCHEDULER BEFORE LOADING ( -i )
  ABS Creation Process
  ABS Discovery Process

USAGE_DOC

# 
# Optional Argument
#   -f<version>                 Override the default version of the file specification (1.0, 1.1)
#   -d<verbose value>           @verbose flag value - use '1' for timing info and '2' for lots of detail
#

}

error_display()
{
  nCountError=$(egrep 'GTM[[:upper:]]{2}[[:digit:]]{4}E' "$1" | wc -l )
  nCountError=$((${nCountError} + $(egrep 'Error = \[Microsoft\]\[ODBC SQL Server Driver\]' "$1" | wc -l ) ))
  nCountError=$((${nCountError} + $(egrep '^Msg[[:space:]]*[[:digit:]]*,[[:space:]]*Level[[:space:]]*[[:digit:]]*,[[:space:]]*State[[:space:]]*[[:digit:]]*' "$1" | wc -l ) ))
  nCountError=$((${nCountError} + $(egrep 'Login failed for user' "$1" | wc -l ) ))
  nCountError=$((${nCountError} + $(egrep '\[DBNETLIB\]SQL Server does not exist or access denied.' "$1" | wc -l ) ))


  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    egrep '^GTM[[:upper:]]{2}[[:digit:]]{4}E' "$1" >&2
    egrep 'Error = \[Microsoft\]\[ODBC SQL Server Driver\]' "$1" >&2
    egrep '^Msg[[:space:]]*[[:digit:]]*,[[:space:]]*Level[[:space:]]*[[:digit:]]*,[[:space:]]*State[[:space:]]*[[:digit:]]*' "$1" >&2
    egrep 'Login failed for user' "$1" >&2
    egrep '\[DBNETLIB\]SQL Server does not exist or access denied.'  "$1" >&2
    return $nCountError
  fi

  return 0
}

check_sql_output()
{

  # $1 filename to check
  # Check for:Msg 170, Level 15, State 1, Procedure , Line 1
  nCountError=$(egrep '^Msg[[:space:]]*[[:digit:]]*,[[:space:]]*Level[[:space:]]*[[:digit:]]*,[[:space:]]*State[[:space:]]*[[:digit:]]*' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    #egrep '^Msg[[:space:]]*[[:digit:]]*,[[:space:]]*Level[[:space:]]*[[:digit:]]*,[[:space:]]*State[[:space:]]*[[:digit:]]*' "$1" >&2
    return $nCountError
  fi

  # Check for:[Microsoft][ODBC SQL Server Driver]
  nCountError=$(egrep 'Error = \[Microsoft\]\[ODBC SQL Server Driver\]' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    #egrep '\[Microsoft\]\[ODBC SQL Server Driver\]' "$1" >&2
    return $nCountError
  fi

  # Check for:GTMXX0000E ...
  nCountError=$(egrep 'GTM[[:upper:]]{2}[[:digit:]]{4}E' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    #egrep '^GTM[[:upper:]]{2}[[:digit:]]{4}E' "$1" >&2
    return $nCountError
  fi

  # Check for: Login failed for user 'nnnn'. 
  nCountError=$(egrep 'Login failed for user' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    egrep 'Login failed for user' "$1" >&2
    return $nCountError
  fi

  # Check for: [DBNETLIB]SQL Server does not exist or access denied. 
  nCountError=$(egrep '\[DBNETLIB\]SQL Server does not exist or access denied.' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    egrep '\[DBNETLIB\]SQL Server does not exist or access denied.'  "$1" >&2
    return $nCountError
  fi



  return 0
}

check_bcp_output()
{

  # $1 filename to check
  # Check for:SQLState = 08001, NativeError = 6
  # allow SQLState = 01000
  nCountError=$(sed 's/01000,//' "$1" | egrep 'SQLState[[:space:]]*=[[:space:]]*[[:digit:]]*,[[:space:]]*NativeError[[:space:]]*=[[:space:]]*[[:digit:]]*'  | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_bcp_output: Error in bcp output" >&2
    egrep 'SQLState[[:space:]]*=[[:space:]]*[[:digit:]]*,[[:space:]]*NativeError[[:space:]]*=[[:space:]]*[[:digit:]]*' "$1" >&2
    return $nCountError
  fi

  # Check for:Error = [Microsoft][ODBC SQL Server Driver][Named Pipes]Specified SQL server not found.
  # Warning = [Microsoft][ODBC SQL Server Driver][Named Pipes]ConnectionOpen (CreateFile()).

  nCountError=$(egrep 'Error = \[Microsoft\]\[ODBC SQL Server Driver\]' "$1" | wc -l )

  if [ $nCountError -ne 0 ]; then;
    # Errors have occurred
    echo "check_sql_output: Error in sql output" >&2
    egrep '\[Microsoft\]\[ODBC SQL Server Driver\]' "$1" >&2
    return $nCountError
  fi

}

get_sVersionDefault()
{
  echo "SELECT 'sVersionConfigFile ' + ISNULL( propertyvalue, '1.0' ) FROM SystemConfiguration WHERE groupname = 'ABS' AND propertyname = 'sVersionConfigFile'" > "${sFilenameSqlIn}"
  echo "" >> "${sFilenameLog}"

  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi

  sVersionDefault=$(awk '/sVersionConfigFile/ { print $2 }' "${sFilenameSqlOut}")
  return 0
}

get_sVersionIn()
{
  sVersionIn=$(awk 'BEGIN { sVersion="1.0" }
END { print sVersion }
/^[Hh]eader/ { nLineFound = 1 ; next }
/^[Vv]ersion/ { if (1==nLineFound) { nLineFound = 2 ; next } else { nLineFound = 0 } }
/^1\.1/ { if ( 2==nLineFound ) { sVersion="1.1" ; nLineFound = 0 ; next } }
/^1\.2/ { if ( 2==nLineFound ) { sVersion="1.2" ; nLineFound = 0 ; next } }
/^1\.3/ { if ( 2==nLineFound ) { sVersion="1.3" ; nLineFound = 0 ; next } }
/^1\.4/ { if ( 2==nLineFound ) { sVersion="1.4" ; nLineFound = 0 ; next } }
// { next }'  "${sFilenameIn}")

}

bcp_out_1_0()
{
  echo "" >> "${sFilenameLog}"
  echo "bcp out 1:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp1" queryout absConfigOut1.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 1:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 2:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp2" queryout absConfigOut2.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 2:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 3:" >> "${sFilenameLog}"
  $sSqlCall -Q"EXECUTE alobOutputConfigBcp3Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "bcp out 3:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp3" queryout absConfigOut3.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 3:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 4:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp4" queryout absConfigOut4.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 4:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 5:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp5" queryout absConfigOut5.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "$sFilenameLob"
  echo "check bcp out 5:" >> "$sFilenameLob"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "$sFilenameLob"
  echo "Creating output file" >> "${sFilenameLog}"

  echo "PatternList" > "${sFilenameOut}"
  echo "ListName\tOperand" >> "${sFilenameOut}"
  cat  absConfigOut5.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "Pattern" >> "${sFilenameOut}"
  echo "Pattern\tClass\tAttribute\tWhen\tOperator\tOperand1\tOperand2" >> "${sFilenameOut}"
  cat  absConfigOut1.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPattern" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tPatternRelated" >> "${sFilenameOut}"
  cat  absConfigOut2.txt >> "${sFilenameOut}"
  
  echo "" >> "${sFilenameOut}"
  echo "Path" >> "${sFilenameOut}"
  echo "Path\tLevel\tName" >> "${sFilenameOut}"
  cat  absConfigOut3.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPath" >> "${sFilenameOut}"
  echo "Criteria\tPath\tLevel\tPattern\tVariable\tValue" >> "${sFilenameOut}"
  cat absConfigOut4.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameLog}"
  echo "Move old bcp out files" >> "${sFilenameLog}"

  sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
  sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
  mv -v absConfigOut1.txt "${sFilenameOut}/Logs/$0.${sTimestamp}.bcp_out1.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut2.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out2.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut3.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out3.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut4.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out4.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut5.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out5.log" 1>> "${sFilenameLog}"
  
} # close of bcp_out_1_0



bcp_out_1_1()
{


  echo "" >> "${sFilenameLog}"
  echo "bcp out 1:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp1" queryout absConfigOut1.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 1:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 2:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp2" queryout absConfigOut2.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 2:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 7:" >> "${sFilenameLog}"
  $sSqlCall -Q"EXECUTE alobOutputConfigBcp7Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "bcp out 7:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp7" queryout absConfigOut7.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 7:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 4:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp4" queryout absConfigOut4.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 4:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 5:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp5" queryout absConfigOut5.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 5:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 6:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp6" queryout absConfigOut6.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 6:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "Creating output file" >> "${sFilenameLog}"

  echo "Header" > "${sFilenameOut}"
  echo "Version" >> "${sFilenameOut}"
  cat  absConfigOut6.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "PatternList" >> "${sFilenameOut}"
  echo "ListName\tOperand" >> "${sFilenameOut}"
  cat  absConfigOut5.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "Pattern" >> "${sFilenameOut}"
  echo "Pattern\tClass\tAttribute\tWhen\tOperator\tOperand1\tOperand2" >> "${sFilenameOut}"
  cat  absConfigOut1.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPattern" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tPatternRelated" >> "${sFilenameOut}"
  cat  absConfigOut2.txt >> "${sFilenameOut}"
  
  echo "" >> "${sFilenameOut}"
  echo "Path" >> "${sFilenameOut}"
  echo "Path\tLevel\tName\tDescription" >> "${sFilenameOut}"
  cat  absConfigOut7.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPath" >> "${sFilenameOut}"
  echo "Criteria\tPath\tLevel\tPattern\tVariable\tValue" >> "${sFilenameOut}"
  cat absConfigOut4.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameLog}"
  echo "Move old bcp out files" >> "${sFilenameLog}"

  sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
  sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
  mv -v absConfigOut1.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out1.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut2.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out2.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut4.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out4.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut5.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out5.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut6.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out6.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut7.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out7.log" 1>> "${sFilenameLog}"
  
} # close of bcp_out_1_1

bcp_out_1_2()
{


  echo "" >> "${sFilenameLog}"
  echo "bcp out 1:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp1" queryout absConfigOut1.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 1:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 2:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp2" queryout absConfigOut2.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 2:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 8:" >> "${sFilenameLog}"
  $sSqlCall -Q"EXECUTE alobOutputConfigBcp8Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "bcp out 8:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp8" queryout absConfigOut8.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 8:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 9:" >> "${sFilenameLog}"
  $sSqlCall -Q"SET CONCAT_NULL_YIELDS_NULL OFF EXECUTE alobOutputConfigBcp9Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 9:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp9" queryout absConfigOut9.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 9:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 5:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp5" queryout absConfigOut5.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 5:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 19:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp19" queryout absConfigOut19.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 19:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 10:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp10" queryout absConfigOut10.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 10:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 11:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp11" queryout absConfigOut11.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 11:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 12:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp12" queryout absConfigOut12.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 12:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 13:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp13" queryout absConfigOut13.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 13:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 14:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp14" queryout absConfigOut14.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 14:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 15:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp15" queryout absConfigOut15.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 15:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 16:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp16" queryout absConfigOut16.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 16:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 17:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp17" queryout absConfigOut17.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 17:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 18:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp18" queryout absConfigOut18.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 18:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 20:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp20" queryout absConfigOut20.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 20:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 20:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp21" queryout absConfigOut21.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 21:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "Creating output file" >> "${sFilenameLog}"

  echo "Header" > "${sFilenameOut}"
  echo "Version" >> "${sFilenameOut}"
  cat  absConfigOut19.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "PatternList" >> "${sFilenameOut}"
  echo "ListName\tOperand" >> "${sFilenameOut}"
  cat  absConfigOut5.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "Pattern" >> "${sFilenameOut}"
  echo "Pattern\tClass\tAttribute\tWhen\tOperator\tOperand1\tOperand2" >> "${sFilenameOut}"
  cat  absConfigOut1.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPattern" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tPatternRelated" >> "${sFilenameOut}"
  cat  absConfigOut2.txt >> "${sFilenameOut}"
  
  echo "" >> "${sFilenameOut}"
  echo "Path" >> "${sFilenameOut}"
  echo "Path\tLevel\tName\tDescription\tFunctionGroup" >> "${sFilenameOut}"
  cat  absConfigOut8.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPath" >> "${sFilenameOut}"
  echo "Criteria\tPath\tLevel\tPattern\tVariable\tValue\tFunctionGroup" >> "${sFilenameOut}"
  cat absConfigOut9.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToFunctionGroup" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tFunctionGroup" >> "${sFilenameOut}"
  cat absConfigOut10.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "FunctionGroup" >> "${sFilenameOut}"
  echo "FunctionGroup\tFunctionName\tInstance" >> "${sFilenameOut}"
  cat absConfigOut11.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "FunctionGroupRelevance" >> "${sFilenameOut}"
  echo "FunctionGroup\tRelevance" >> "${sFilenameOut}"
  cat absConfigOut12.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetThresholds" >> "${sFilenameOut}"
  echo "Instance\tEventType\tAlertState\tPriority\tValue" >> "${sFilenameOut}"
  cat absConfigOut13.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetSendStateChangeEvent" >> "${sFilenameOut}"
  echo "Instance\tValue" >> "${sFilenameOut}"
  cat absConfigOut14.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "DefineAndUsePBTs" >> "${sFilenameOut}"
  echo "Instance\tPBTName\tRelevance\tEnabled\tMinAlertState\tMaxAlertState\tMinPercentage\tMaxPercentage\tEventType\tEventName\tEventText\tEventAlertState\tEventPriority\tEventFlags\tEventState" >> "${sFilenameOut}"
  cat absConfigOut15.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetWeights" >> "${sFilenameOut}"
  echo "Instance\tNewWeight" >> "${sFilenameOut}"
  cat absConfigOut16.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "DefineServicesAndLinkToEvUsers" >> "${sFilenameOut}"
  echo "Instance\tDisplayName\tServiceID\tBusinessRole\tRedImpact\tYellowImpact\tFlags\tUserIDGroup" >> "${sFilenameOut}"
  cat absConfigOut17.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "UserIDGroup" >> "${sFilenameOut}"
  echo "Instance\tUserID" >> "${sFilenameOut}"
  cat absConfigOut18.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "AutoCopyEVService" >> "${sFilenameOut}"
  echo "Instance\tValue" >> "${sFilenameOut}"
  cat absConfigOut20.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetBSPriority" >> "${sFilenameOut}"
  echo "Instance\tNewPriority" >> "${sFilenameOut}"
  cat absConfigOut21.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"



  echo "" >> "${sFilenameLog}"
  echo "Move old bcp out files" >> "${sFilenameLog}"

  sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
  sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
  mv -v absConfigOut1.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out1.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut2.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out2.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut9.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out9.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut5.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out5.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut19.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out19.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut8.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out8.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut10.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out10.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut11.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out11.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut12.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out12.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut13.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out13.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut14.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out14.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut15.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out15.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut16.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out16.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut17.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out17.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut18.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out18.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut20.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out20.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut21.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out21.log" 1>> "${sFilenameLog}"

  
} # close of bcp_out_1_2


bcp_out_1_3()
{


  echo "" >> "${sFilenameLog}"
  echo "bcp out 1:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp1" queryout absConfigOut1.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 1:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 2:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp2" queryout absConfigOut2.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 2:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 8:" >> "${sFilenameLog}"
  $sSqlCall -Q"EXECUTE alobOutputConfigBcp28Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi


  echo "" >> "${sFilenameLog}"
  echo "bcp out 8:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp28" queryout absConfigOut8.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 8:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "sql prepare 9:" >> "${sFilenameLog}"
  $sSqlCall -Q"SET CONCAT_NULL_YIELDS_NULL OFF EXECUTE alobOutputConfigBcp9Detail" -o"${sFilenameSqlOut}"
  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 9:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp9" queryout absConfigOut9.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 9:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 5:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp5" queryout absConfigOut5.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 5:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 19:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp29" queryout absConfigOut29.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 19:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 10:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp10" queryout absConfigOut10.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 10:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 11:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp11" queryout absConfigOut11.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 11:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 12:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp12" queryout absConfigOut12.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 12:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 13:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp13" queryout absConfigOut13.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 13:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 14:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp14" queryout absConfigOut14.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 14:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 15:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp15" queryout absConfigOut15.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 15:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 16:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp16" queryout absConfigOut16.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 16:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 17:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp17" queryout absConfigOut17.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 17:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 18:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp18" queryout absConfigOut18.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 18:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 20:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp20" queryout absConfigOut20.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 20:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 21:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp21" queryout absConfigOut21.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 21:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 22:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp22" queryout absConfigOut22.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 22:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 23:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp23" queryout absConfigOut23.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 23:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 24:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp24" queryout absConfigOut24.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 24:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 25:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp25" queryout absConfigOut25.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 25:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 26:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp26" queryout absConfigOut26.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 26:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 27:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp27" queryout absConfigOut27.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 27:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi

  echo "" >> "${sFilenameLog}"
  echo "bcp out 30:" >> "${sFilenameLog}"
  bcp "EXECUTE Object..alobOutputConfigBcp30" queryout absConfigOut30.txt $sBcpParameter 2>&1 >> "${sFilenameLog}"
  echo "" >> "${sFilenameLog}"
  echo "check bcp out 30:" >> "${sFilenameLog}"
  check_bcp_output "${sFilenameLog}" 2>> "${sFilenameLog}"
  if [ $? -ne 0 ];then
    return 1
  fi



  echo "" >> "${sFilenameLog}"
  echo "Creating output file" >> "${sFilenameLog}"

  echo "Header" > "${sFilenameOut}"
  echo "Version" >> "${sFilenameOut}"
  # Echo the output version retrieved from SystemConfiguration or the input config file 
  # cat  absConfigOut29.txt >> "${sFilenameOut}"
  echo "$sVersionOut" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "PatternList" >> "${sFilenameOut}"
  echo "ListName\tOperand" >> "${sFilenameOut}"
  cat  absConfigOut5.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "Pattern" >> "${sFilenameOut}"
  if [ "$sVersionOut" = "1.4" ]; then
    echo "Pattern\tClass\tAttribute\tWhen\tOperator\tOperand1\tOperand2\tPatternUsage" >> "${sFilenameOut}"
  else  
    echo "Pattern\tClass\tAttribute\tWhen\tOperator\tOperand1\tOperand2" >> "${sFilenameOut}"
  fi  
  cat  absConfigOut1.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPattern" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tPatternRelated" >> "${sFilenameOut}"
  cat  absConfigOut2.txt >> "${sFilenameOut}"
  
  echo "" >> "${sFilenameOut}"
  echo "Path" >> "${sFilenameOut}"
  echo "Path\tLevel\tName\tDescription\tFunctionGroup\tBusinessSystemFolder" >> "${sFilenameOut}"
  cat  absConfigOut8.txt >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToPath" >> "${sFilenameOut}"
  echo "Criteria\tPath\tLevel\tPattern\tVariable\tValue\tFunctionGroup" >> "${sFilenameOut}"
  cat absConfigOut9.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "CriteriaToFunctionGroup" >> "${sFilenameOut}"
  echo "Criteria\tPattern\tFunctionGroup" >> "${sFilenameOut}"
  cat absConfigOut10.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "FunctionGroup" >> "${sFilenameOut}"
  echo "FunctionGroup\tFunctionName\tInstance" >> "${sFilenameOut}"
  cat absConfigOut11.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "FunctionGroupRelevance" >> "${sFilenameOut}"
  echo "FunctionGroup\tRelevance" >> "${sFilenameOut}"
  cat absConfigOut12.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetThresholds" >> "${sFilenameOut}"
  echo "Instance\tEventType\tAlertState\tPriority\tValue" >> "${sFilenameOut}"
  cat absConfigOut13.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetSendStateChangeEvent" >> "${sFilenameOut}"
  echo "Instance\tValue" >> "${sFilenameOut}"
  cat absConfigOut14.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "DefineAndUsePBTs" >> "${sFilenameOut}"
  echo "Instance\tPBTName\tRelevance\tEnabled\tMinAlertState\tMaxAlertState\tMinPercentage\tMaxPercentage\tEventType\tEventName\tEventText\tEventAlertState\tEventPriority\tEventFlags\tEventState" >> "${sFilenameOut}"
  cat absConfigOut15.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetWeights" >> "${sFilenameOut}"
  echo "Instance\tNewWeight" >> "${sFilenameOut}"
  cat absConfigOut16.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "DefineServicesAndLinkToEvUsers" >> "${sFilenameOut}"
  echo "Instance\tDisplayName\tServiceID\tBusinessRole\tRedImpact\tYellowImpact\tFlags\tUserIDGroup" >> "${sFilenameOut}"
  cat absConfigOut17.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "UserIDGroup" >> "${sFilenameOut}"
  echo "Instance\tUserID" >> "${sFilenameOut}"
  cat absConfigOut18.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "AutoCopyEVService" >> "${sFilenameOut}"
  echo "Instance\tValue" >> "${sFilenameOut}"
  cat absConfigOut20.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SetBSPriority" >> "${sFilenameOut}"
  echo "Instance\tNewPriority" >> "${sFilenameOut}"
  cat absConfigOut21.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "BuildBusinessSystem" >> "${sFilenameOut}"
  echo "Instance\tSeedResources\tLinkTypes\tEndLevel\tExclusions\tLeafNodes\tPassThroughNodes\tCustomProc" >> "${sFilenameOut}"
  cat absConfigOut22.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "SeedResources" >> "${sFilenameOut}"
  echo "Instance\tCriteria\tPattern" >> "${sFilenameOut}"
  cat absConfigOut23.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "LinkTypes" >> "${sFilenameOut}"
  echo "Instance\tOriginClasses\tSourceClasses\tLinkType\tDestinationClasses" >> "${sFilenameOut}"
  cat absConfigOut24.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "ClassList" >> "${sFilenameOut}"
  echo "Instance\tClass" >> "${sFilenameOut}"
  cat absConfigOut25.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "Exclusions" >> "${sFilenameOut}"
  echo "Instance\tClass\tObjectID\tInstanceID" >> "${sFilenameOut}"
  cat absConfigOut26.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "LeafNodes" >> "${sFilenameOut}"
  echo "Instance\tClass\tObjectID\tInstanceID" >> "${sFilenameOut}"
  cat absConfigOut27.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"

  echo "" >> "${sFilenameOut}"
  echo "PassThroughNodes" >> "${sFilenameOut}"
  echo "Instance\tClass\tObjectID\tInstanceID" >> "${sFilenameOut}"
  cat absConfigOut30.txt >> "${sFilenameOut}"
  echo "" >> "${sFilenameOut}"




  echo "" >> "${sFilenameLog}"
  echo "Move old bcp out files" >> "${sFilenameLog}"

  sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
  sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
  mv -v absConfigOut1.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out1.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut2.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out2.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut9.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out9.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut5.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out5.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut8.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out8.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut10.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out10.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut11.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out11.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut12.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out12.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut13.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out13.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut14.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out14.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut15.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out15.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut16.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out16.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut17.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out17.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut18.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out18.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut20.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out20.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut21.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out21.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut22.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out22.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut23.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out23.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut24.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out24.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut25.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out25.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut26.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out26.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut27.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out27.log" 1>> "${sFilenameLog}"
#  mv -v absConfigOut29.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out28.log" 1>> "${sFilenameLog}"
  mv -v absConfigOut30.txt "${sPathInstall}/Logs/$0.${sTimestamp}.bcp_out30.log" 1>> "${sFilenameLog}"

  
} # close of bcp_out_1_3

bcp_in_1_0()
{
awk -F"\x09" -v OFS="\x09" '

# /archive/scripts/absConfig.awk


BEGIN {

  nCountTypeRecord = 5

  # iStateRecord = 1 # Pattern Data
  # iStateRecord = 2 # CriteriaToPattern Data
  # iStateRecord = 3 # Path Data
  # iStateRecord = 4 # CriteriaToPath Data
  # iStateRecord = 5 # PatternList Data

  # Initialize the record count for each of the record types
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    nCountRecord[ iStateRecord ] = 0
  }

  # Establish the number of fields per record type
  nCountField[ 1 ] = 7
  nCountField[ 2 ] = 3
  nCountField[ 3 ] = 3
  nCountField[ 4 ] = 6
  nCountField[ 5 ] = 2

  # Establish the filenames to receive bcp data
  if ( "" == FILENAME_PREFIX_OUTPUT ) {
    FILENAME_PREFIX_OUTPUT = "./absConfig.ksh."
  }
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    sFilenameBcp[ iStateRecord ] = FILENAME_PREFIX_OUTPUT iStateRecord ".bcp"
  }

  # Initialize the files to receive bcp data
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    system( "rm -f " sFilenameBcp[ iStateRecord ] )
    close(  "rm -f " sFilenameBcp[ iStateRecord ] )
    system( "touch " sFilenameBcp[ iStateRecord ] )
    close(  "touch " sFilenameBcp[ iStateRecord ] )
  }

  # Initialize the state of the current record
  iStateRecord = 0 
  
}

END {

  # Output the bcp files
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    for( iRecord = 1; iRecord <= nCountRecord[ iStateRecord ]; iRecord++ ) {
      print sRecord[ iStateRecord, iRecord ] > sFilenameBcp[ iStateRecord ]
    }
  } 
}


# Ignorable lines 

# Target: Remark lines
# #
/^[[:space:]]*#/ {

  next
}

# Target: Blank lines
/^[[:space:]]*$/ {

  next
}

# Target: Column headers
# Pattern       Class   Attribute       When	Operator        Operand1   Operand2
( ( 1 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]/ ) { next }

# idCriteria      idPattern       idPatternRelated
( ( 2 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# sNamePathLob    idLevel sName
( ( 3 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]/ ) { next }

# idCriteria      sNamePathLob    idPattern       idLevel
( ( 4 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# ListName	Operand
( ( 5 == iStateRecord ) && /^[[:space:]]*[Ll][Ii][Ss][Tt][Nn][Aa][Mm][Ee][[:space:]]/ ) { next }


# Header lines

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 1
  next
}

# Target: iStateRecord
# CriteriaToPattern
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 2
  next
}

# Target: iStateRecord
# Path
/^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 3
  next
}

# Target: iStateRecord
# CriteriaToPath
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 4
  next
}

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 5
  next
}



# Data Records

# Target: 
// { 
  # HACK
  # print iStateRecord, $0
}

# Target: sRecord
( 0 != iStateRecord ) {

  # HACK
  # print "In the data record for iStateRecord = " iStateRecord

  # Break $0 into an array and then reform into bcp output
  delete sField
  split( $0, sField, FS )

  sRecordBcp = ""
  for( iField = 1; iField <= nCountField[ iStateRecord ] ; iField++ ) {
    sRecordBcp = sRecordBcp OFS sField[ iField ]
  } 
  sub( OFS, "", sRecordBcp )

  # Store bcp record for later output ( see END )
  sRecord[ iStateRecord, ++nCountRecord[ iStateRecord ] ] = sRecordBcp

}


' "${sFilenameIn}"


  # Ok at this point we need to copy the bcp files to the database server for ease of loading:

  cp -v ./absConfig.ksh.[12345].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.[12345].bcp 1>> "${sFilenameLog}"

  echo "" >> "${sFilenameLog}"
  echo "bcp_in creating sql in script:" >> "${sFilenameLog}"

  rm -f "${sFilenameSqlIn}"
  touch "${sFilenameSqlIn}"

  cat <<SQL_SCRIPT > "${sFilenameSqlIn}"


Use Object
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadWrapperAbsConfig' AND type = 'P' )
  DROP PROCEDURE alobLoadWrapperAbsConfig
GO
   
${sTruncate}

CREATE PROCEDURE alobLoadWrapperAbsConfig
  
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF

  TRUNCATE TABLE alobFilePattern
  TRUNCATE TABLE alobFileCriteriaToPattern
  TRUNCATE TABLE alobFilePath
  TRUNCATE TABLE alobFileCriteriaToPath
  TRUNCATE TABLE alobFilePatternList
  TRUNCATE TABLE alobFilePath7
  TRUNCATE TABLE alobFilePath8
  TRUNCATE TABLE alobFileCriteriaToPath9
  TRUNCATE TABLE alobFileDefineAndUsePBTs
  TRUNCATE TABLE alobFileDefineServicesAndLinkToEvUsers
  TRUNCATE TABLE alobFileSetSendStateChangeEvent
  TRUNCATE TABLE alobFileSetWeights
  TRUNCATE TABLE alobFileSetThresholds
  TRUNCATE TABLE alobFileFunctionGroup
  TRUNCATE TABLE alobFileCriteriaToFunctionGroup
  TRUNCATE TABLE alobFileUserIDGroup
  TRUNCATE TABLE alobFileFunctionGroupPrecedence


  BEGIN TRANSACTION absConfig                   

  DECLARE @nError INTEGER

  DELETE FROM alobListCriteriaAlob              
  DELETE FROM alobListPattern                   
  DELETE FROM dynamic_object_create_path_detail 
  DELETE FROM dynamic_object_create_path        
  DELETE FROM alobListCriteriaToPath            
  DELETE FROM alobListPatternList    
  DELETE FROM alobListAttributeTemplate 
  DELETE FROM alobListCriteriaToFunctionGroup
  DELETE FROM alobListDefineAndUsePBTs
  DELETE FROM alobListDefineServicesAndLinkToEvUsers
  DELETE FROM alobListFunctionGroup
  DELETE FROM alobListFunctionGroupPrecedence
  DELETE FROM alobListPath
  DELETE FROM alobListSetSendStateChangeEvent
  DELETE FROM alobListSetThresholds
  DELETE FROM alobListSetWeights
  DELETE FROM alobListStaticFunctionGroup
  DELETE FROM alobListUserIDGroup
  DELETE FROM alobFunctionGroupTemplate
          

  SET NOCOUNT ON                                
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.5.bcp', 'PatternList', '${sCodepage}', 1.0, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.1.bcp', 'Pattern', '${sCodepage}', 1.0, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.2.bcp', 'CriteriaToPattern', '${sCodepage}', 1.0, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.3.bcp', 'Path', '${sCodepage}', 1.0, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.4.bcp', 'CriteriaToPath', '${sCodepage}', 1.0, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  DELETE FROM ev_watch_attr_filter -- else attributes build up over time
  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  -- Insert new records into table when there is no pre-existing data.
  INSERT INTO  ev_watch_attr_filter
  (
    cid,
    attribute_name,
    extended,
    active,
    salience,
    val_inserted_pat,
    val_deleted_pat,
    enqueue
  )
  SELECT
    t.cid,
    t.attribute_name,
    t.extended,
    t.active,
    t.salience,
    t.val_inserted_pat,
    t.val_inserted_pat,
    t.enqueue
  FROM ev_watch_attr_filter_temp t
  LEFT OUTER JOIN expanded_ev_watch_attr_filter AS eewaf
    ON eewaf.cid            = t.cid
   AND eewaf.attribute_name = t.attribute_name
  WHERE eewaf.cid IS NULL

  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  COMMIT TRANSACTION absConfig                   
  RETURN 0
GO
  EXECUTE alobLoadWrapperAbsConfig
SQL_SCRIPT

  echo "" >> "${sFilenameLog}"
  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    cat "${sFilenameSqlOut}" >> "${sFilenameLog}"
    return 1
  fi

} # close of bcp_in_1_0



bcp_in_1_1()
{
awk -F"\x09" -v OFS="\x09" '

# /archive/scripts/absConfig.awk


BEGIN {

  nCountTypeRecord = 6

  # iStateRecord = 1 # Pattern Data
  # iStateRecord = 2 # CriteriaToPattern Data
  # iStateRecord = 3 # Path Data
  # iStateRecord = 4 # CriteriaToPath Data
  # iStateRecord = 5 # PatternList Data
  # iStateRecord = 6 # Header


  # Initialize the record count for each of the record types
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    nCountRecord[ iStateRecord ] = 0
  }

  # Establish the number of fields per record type
  nCountField[ 1 ] = 7
  nCountField[ 2 ] = 3
  nCountField[ 3 ] = 4
  nCountField[ 4 ] = 6
  nCountField[ 5 ] = 2
  nCountField[ 6 ] = 1

  # Establish the filenames to receive bcp data
  if ( "" == FILENAME_PREFIX_OUTPUT ) {
    FILENAME_PREFIX_OUTPUT = "./absConfig.ksh."
  }
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    sFilenameBcp[ iStateRecord ] = FILENAME_PREFIX_OUTPUT iStateRecord ".bcp"
  }

  # Initialize the files to receive bcp data
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    system( "rm -f " sFilenameBcp[ iStateRecord ] )
    close(  "rm -f " sFilenameBcp[ iStateRecord ] )
    system( "touch " sFilenameBcp[ iStateRecord ] )
    close(  "touch " sFilenameBcp[ iStateRecord ] )
  }

  # Initialize the state of the current record
  iStateRecord = 0 
  
}

END {

  # Output the bcp files
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    for( iRecord = 1; iRecord <= nCountRecord[ iStateRecord ]; iRecord++ ) {
      print sRecord[ iStateRecord, iRecord ] > sFilenameBcp[ iStateRecord ]
    }
  } 
}


# Ignorable lines 

# Target: Remark lines
# #
/^[[:space:]]*#/ {

  next
}

# Target: Blank lines
/^[[:space:]]*$/ {

  next
}

# Target: Column headers
# Pattern       Class   Attribute       When	Operator        Operand1   Operand2
( ( 1 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]/ ) { next }

# idCriteria      idPattern       idPatternRelated
( ( 2 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# sNamePathLob    idLevel sName
( ( 3 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]/ ) { next }

# idCriteria      sNamePathLob    idPattern       idLevel
( ( 4 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# ListName	Operand
( ( 5 == iStateRecord ) && /^[[:space:]]*[Ll][Ii][Ss][Tt][Nn][Aa][Mm][Ee][[:space:]]/ ) { next }

# ListName	Operand
( ( 6 == iStateRecord ) && /^[[:space:]]*[Vv][Ee][Rr][Ss][Ii][Oo][Nn][[:space:]]/ ) { next }

# Header lines

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 1
  next
}

# Target: iStateRecord
# CriteriaToPattern
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 2
  next
}

# Target: iStateRecord
# Path
/^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 3
  next
}

# Target: iStateRecord
# CriteriaToPath
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 4
  next
}

# Target: iStateRecord
# PatternList
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 5
  next
}

# Target: iStateRecord
# Header
/^[[:space:]]*[Hh][Ee][Aa][Dd][Ee][Rr][[:space:]]*$/ {

  iStateRecord = 6
  next
}


# Data Records

# Target: 
// { 
  # HACK
  # print iStateRecord, $0
}

# Target: sRecord
( 0 != iStateRecord ) {

  # HACK
  # print "In the data record for iStateRecord = " iStateRecord

  # Break $0 into an array and then reform into bcp output
  delete sField
  split( $0, sField, FS )

  sRecordBcp = ""
  for( iField = 1; iField <= nCountField[ iStateRecord ] ; iField++ ) {
    sRecordBcp = sRecordBcp OFS sField[ iField ]
  } 
  sub( OFS, "", sRecordBcp )

  # Store bcp record for later output ( see END )
  sRecord[ iStateRecord, ++nCountRecord[ iStateRecord ] ] = sRecordBcp

}


' "${sFilenameIn}"


  # Ok at this point we need to copy the bcp files to the database server for ease of loading:

  cp -v ./absConfig.ksh.[123456].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.[123456].bcp 1>> "${sFilenameLog}"

  echo "" >> "${sFilenameLog}"
  echo "bcp_in creating sql in script:" >> "${sFilenameLog}"

  rm -f "${sFilenameSqlIn}"
  touch "${sFilenameSqlIn}"

  cat <<SQL_SCRIPT > "${sFilenameSqlIn}"


Use Object
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadWrapperAbsConfig' AND type = 'P' )
  DROP PROCEDURE alobLoadWrapperAbsConfig
GO
   
${sTruncate}

CREATE PROCEDURE alobLoadWrapperAbsConfig
  
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF

  TRUNCATE TABLE alobFilePattern
  TRUNCATE TABLE alobFileCriteriaToPattern
  TRUNCATE TABLE alobFilePath
  TRUNCATE TABLE alobFileCriteriaToPath
  TRUNCATE TABLE alobFilePatternList
  TRUNCATE TABLE alobFilePath7
  TRUNCATE TABLE alobFilePath8
  TRUNCATE TABLE alobFileCriteriaToPath9
  TRUNCATE TABLE alobFileDefineAndUsePBTs
  TRUNCATE TABLE alobFileDefineServicesAndLinkToEvUsers
  TRUNCATE TABLE alobFileSetSendStateChangeEvent
  TRUNCATE TABLE alobFileSetWeights
  TRUNCATE TABLE alobFileSetThresholds
  TRUNCATE TABLE alobFileFunctionGroup
  TRUNCATE TABLE alobFileCriteriaToFunctionGroup
  TRUNCATE TABLE alobFileUserIDGroup
  TRUNCATE TABLE alobFileFunctionGroupPrecedence



  BEGIN TRANSACTION absConfig                   

  DECLARE @nError INTEGER

  DELETE FROM alobListCriteriaAlob              
  DELETE FROM alobListPattern                   
  DELETE FROM dynamic_object_create_path_detail 
  DELETE FROM dynamic_object_create_path        
  DELETE FROM alobListCriteriaToPath            
  DELETE FROM alobListPatternList  
  DELETE FROM alobListAttributeTemplate             
  DELETE FROM alobListCriteriaToFunctionGroup
  DELETE FROM alobListDefineAndUsePBTs
  DELETE FROM alobListDefineServicesAndLinkToEvUsers
  DELETE FROM alobListFunctionGroup
  DELETE FROM alobListFunctionGroupPrecedence
  DELETE FROM alobListPath
  DELETE FROM alobListSetSendStateChangeEvent
  DELETE FROM alobListSetThresholds
  DELETE FROM alobListSetWeights
  DELETE FROM alobListStaticFunctionGroup
  DELETE FROM alobListUserIDGroup
  DELETE FROM alobFunctionGroupTemplate

  SET NOCOUNT ON                                
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.5.bcp', 'PatternList', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.1.bcp', 'Pattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.2.bcp', 'CriteriaToPattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.3.bcp', 'Path', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.4.bcp', 'CriteriaToPath', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  DELETE FROM ev_watch_attr_filter -- else attributes build up over time
  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  -- Insert new records into table when there is no pre-existing data.
  INSERT INTO  ev_watch_attr_filter
  (
    cid,
    attribute_name,
    extended,
    active,
    salience,
    val_inserted_pat,
    val_deleted_pat,
    enqueue
  )
  SELECT
    t.cid,
    t.attribute_name,
    t.extended,
    t.active,
    t.salience,
    t.val_inserted_pat,
    t.val_inserted_pat,
    t.enqueue
  FROM ev_watch_attr_filter_temp t
  LEFT OUTER JOIN expanded_ev_watch_attr_filter AS eewaf
    ON eewaf.cid            = t.cid
   AND eewaf.attribute_name = t.attribute_name
  WHERE eewaf.cid IS NULL

  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  COMMIT TRANSACTION absConfig                   
  RETURN 0
GO
  EXECUTE alobLoadWrapperAbsConfig
SQL_SCRIPT

  echo "" >> "${sFilenameLog}"
  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    cat "${sFilenameSqlOut}" >> "${sFilenameLog}"
    return 1
  fi

} # close of bcp_in_1_1

bcp_in_1_2()
{
awk -F"\x09" -v OFS="\x09" '

# /archive/scripts/absConfig.awk


BEGIN {

  nCountTypeRecord = 17

  # iStateRecord =  1 # Pattern Data
  # iStateRecord =  2 # CriteriaToPattern Data
  # iStateRecord =  3 # Path Data
  # iStateRecord =  4 # CriteriaToPath Data
  # iStateRecord =  5 # PatternList Data
  # iStateRecord =  6 # Header
  # iStateRecord =  7 # CriteriaToFunctionGroup
  # iStateRecord =  8 # FunctionGroup
  # iStateRecord =  9 # FunctionGroupRelevance
  # iStateRecord = 10 # SetThresholds
  # iStateRecord = 11 # SetSendStateChangeEvent
  # iStateRecord = 12 # DefineAndUsePBTs
  # iStateRecord = 13 # SetWeights
  # iStateRecord = 14 # DefineServicesAndLinkToEvUsers
  # iStateRecord = 15 # UserIDGroup
  # iStateRecord = 16 # AutoCopyEVService
  # iStateRecord = 17 # SetBSPriority


  # Initialize the record count for each of the record types
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    nCountRecord[ iStateRecord ] = 0
  }

  # Establish the number of fields per record type
  nCountField[ 1 ] = 7
  nCountField[ 2 ] = 3
  nCountField[ 3 ] = 5
  nCountField[ 4 ] = 7
  nCountField[ 5 ] = 2
  nCountField[ 6 ] = 1
  nCountField[ 7 ] = 3
  nCountField[ 8 ] = 3
  nCountField[ 9 ] = 2
  nCountField[10 ] = 5
  nCountField[11 ] = 2
  nCountField[12 ] = 15
  nCountField[13 ] = 2
  nCountField[14 ] = 8
  nCountField[15 ] = 2
  nCountField[16 ] = 2
  nCountField[17 ] = 2



  # Establish the filenames to receive bcp data
  if ( "" == FILENAME_PREFIX_OUTPUT ) {
    FILENAME_PREFIX_OUTPUT = "./absConfig.ksh."
  }
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    sFilenameBcp[ iStateRecord ] = FILENAME_PREFIX_OUTPUT iStateRecord ".bcp"
  }

  # Initialize the files to receive bcp data
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    system( "rm -f " sFilenameBcp[ iStateRecord ] )
    close(  "rm -f " sFilenameBcp[ iStateRecord ] )
    system( "touch " sFilenameBcp[ iStateRecord ] )
    close(  "touch " sFilenameBcp[ iStateRecord ] )
  }

  # Initialize the state of the current record
  iStateRecord = 0 
  
}

END {

  # Output the bcp files
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    for( iRecord = 1; iRecord <= nCountRecord[ iStateRecord ]; iRecord++ ) {
      print sRecord[ iStateRecord, iRecord ] > sFilenameBcp[ iStateRecord ]
    }
  } 
}


# Ignorable lines 

# Target: Remark lines
# #
/^[[:space:]]*#/ {

  next
}

# Target: Blank lines
/^[[:space:]]*$/ {

  next
}  

# Target: Column headers
# Pattern       Class   Attribute       When	Operator        Operand1   Operand2
( ( 1 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]/ ) { next }

# idCriteria      idPattern       idPatternRelated
( ( 2 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# sNamePathLob    idLevel sName
( ( 3 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]/ ) { next }

# idCriteria      sNamePathLob    idPattern       idLevel
( ( 4 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# ListName	Operand
( ( 5 == iStateRecord ) && /^[[:space:]]*[Ll][Ii][Ss][Tt][Nn][Aa][Mm][Ee][[:space:]]/ ) { next }

# ListName	Operand
( ( 6 == iStateRecord ) && /^[[:space:]]*[Vv][Ee][Rr][Ss][Ii][Oo][Nn][[:space:]]/ ) { next }

# Criteria   Pattern   FunctionGroup
( ( 7 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# FunctionGroup   FunctionName   Instance
( ( 8 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# FunctionGroup   Relevance
( ( 9 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# Instance   EventType   AlertState   Priority   Value
( ( 10 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 11 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   PBTName   Relevance   Enabled   MinAlertState   MaxAlertState   MinPercentage MaxPercentage
#            EventType EventName  EventText EventAlertState EventPriority   EventFlags    EventState
( ( 12 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   NewWeight
( ( 13 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   DisplayName   ServiceID   BusinessRole   RedImpact   YellowImpact   Flags   UserIDGroup
( ( 14 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   UserID
( ( 15 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 16 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 17 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }


# Header lines

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 1
  next
}

# Target: iStateRecord
# CriteriaToPattern
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 2
  next
}

# Target: iStateRecord
# Path
/^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 3
  next
}

# Target: iStateRecord
# CriteriaToPath
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Hh][[:space:]]*$/ {
  iStateRecord = 4
  next
}

# Target: iStateRecord
# PatternList
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 5
  next
}

# Target: iStateRecord
# Header
/^[[:space:]]*[Hh][Ee][Aa][Dd][Ee][Rr][[:space:]]*$/ {

  iStateRecord = 6
  next
}

# Target: iStateRecord
# CriteriaToFunctionGroup
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {
  iStateRecord = 7
  next
}


# Target: iStateRecord
# FunctionGroup
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 8
  next
}

# Target: iStateRecord
# FunctionGroupRelevance
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][Rr][Ee][Ll][Ee][Vv][Aa][Nn][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 9
  next
}


# Target: iStateRecord
# SetThresholds
/^[[:space:]]*[Ss][Ee][Tt][Tt][Hh][Rr][Ee][Ss][Hh][Oo][Ll][Dd][Ss][[:space:]]*$/ {

  iStateRecord = 10
  next
}

# Target: iStateRecord
# SetSendStateChangeEvent
/^[[:space:]]*[Ss][Ee][Tt][Ss][Ee][Nn][Dd][Ss][Tt][Aa][Tt][Ee][Cc][Hh][Aa][Nn][Gg][Ee][Ee][Vv][Ee][Nn][Tt][[:space:]]*$/ {

  iStateRecord = 11
  next
}

# Target: iStateRecord
# DefineAndUsePBTs
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Aa][Nn][Dd][Uu][Ss][Ee][Pp][Bb][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 12
  next
}

# Target: iStateRecord
# SetWeights
/^[[:space:]]*[Ss][Ee][Tt][Ww][Ee][Ii][Gg][Hh][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 13
  next
}

# Target: iStateRecord
# DefineServicesAndLinkToEvUsers
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Ss][Ee][Rr][Vv][Ii][Cc][Ee][Ss][Aa][Nn][Dd][Ll][Ii][Nn][Kk][Tt][Oo][Ee][Vv][Uu][Ss][Ee][Rr][Ss][[:space:]]*$/ {

  iStateRecord = 14
  next
}

# Target: iStateRecord
# UserIDGroup
/^[[:space:]]*[Uu][Ss][Ee][Rr][Ii][Dd][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 15
  next
}

# Target: iStateRecord
# AutoCopyEVService
/^[[:space:]]*[Aa][Uu][Tt][Oo][Cc][Oo][Pp][Yy][Ee][Vv][Ss][Ee][Rr][Vv][Ii][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 16
  next
}

# Target: iStateRecord
# SetBSPriority
/^[[:space:]]*[Ss][Ee][Tt][Bb][Ss][Pp][Rr][Ii][Oo][Rr][Ii][Tt][Yy][[:space:]]*$/ {

  iStateRecord = 17
  next
}




# Data Records

# Target: 
// { 
  # HACK
  # print iStateRecord, $0
}

# Target: sRecord
( 0 != iStateRecord ) {

  # HACK
  # print "In the data record for iStateRecord = " iStateRecord

  # Break $0 into an array and then reform into bcp output
  delete sField
  split( $0, sField, FS )

  sRecordBcp = ""
  for( iField = 1; iField <= nCountField[ iStateRecord ] ; iField++ ) {
    sRecordBcp = sRecordBcp OFS sField[ iField ]
  } 
  sub( OFS, "", sRecordBcp )

  # Store bcp record for later output ( see END )
  sRecord[ iStateRecord, ++nCountRecord[ iStateRecord ] ] = sRecordBcp

}


' "${sFilenameIn}"


  # Ok at this point we need to copy the bcp files to the database server for ease of loading:

  cp -v ./absConfig.ksh.[123456789].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  cp -v ./absConfig.ksh.1[01234567].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.[123456789].bcp 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.1[01234567].bcp 1>> "${sFilenameLog}"

  echo "" >> "${sFilenameLog}"
  echo "bcp_in creating sql in script:" >> "${sFilenameLog}"

  rm -f "${sFilenameSqlIn}"
  touch "${sFilenameSqlIn}"

  cat <<SQL_SCRIPT > "${sFilenameSqlIn}"


Use Object
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadWrapperAbsConfig' AND type = 'P' )
  DROP PROCEDURE alobLoadWrapperAbsConfig
GO
   
${sTruncate}

CREATE PROCEDURE alobLoadWrapperAbsConfig
  
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF

  TRUNCATE TABLE alobFilePattern
  TRUNCATE TABLE alobFileCriteriaToPattern
  TRUNCATE TABLE alobFilePath
  TRUNCATE TABLE alobFileCriteriaToPath
  TRUNCATE TABLE alobFilePatternList
  TRUNCATE TABLE alobFilePath7
  TRUNCATE TABLE alobFilePath8
  TRUNCATE TABLE alobFileCriteriaToPath9
  TRUNCATE TABLE alobFileDefineAndUsePBTs
  TRUNCATE TABLE alobFileDefineServicesAndLinkToEvUsers
  TRUNCATE TABLE alobFileSetSendStateChangeEvent
  TRUNCATE TABLE alobFileSetWeights
  TRUNCATE TABLE alobFileSetThresholds
  TRUNCATE TABLE alobFileFunctionGroup
  TRUNCATE TABLE alobFileCriteriaToFunctionGroup
  TRUNCATE TABLE alobFileUserIDGroup
  TRUNCATE TABLE alobFileFunctionGroupPrecedence
  TRUNCATE TABLE alobFileAutoCopyEVService
  TRUNCATE TABLE alobFileSetBSPriority
  
  BEGIN TRANSACTION absConfig                   

  DECLARE @nError INTEGER

  DECLARE @now NVARCHAR(40)
  DECLARE @verbose TINYINT
  
  ${sTiming}

  DECLARE @start DATETIME
  DECLARE @start1 DATETIME
  SET @start = CURRENT_TIMESTAMP
  DECLARE @elapsed_ms INT
  DECLARE @sqlCounter1 INT
  DECLARE @sqlCounter2 INT
  DECLARE @sqlCounter3 INT
  DECLARE @sqlCounter4 INT
  DECLARE @sqlCounter5 INT
  DECLARE @sqlCounter6 INT
  DECLARE @sqlCounter7 INT
  DECLARE @sqlCounter8 INT
  DECLARE @sqlCounter9 INT
  DECLARE @sqlCounter10 INT
  DECLARE @sqlCounter11 INT
  DECLARE @sqlCounter12 INT
  DECLARE @sqlCounter13 INT
  DECLARE @sqlCounter14 INT
  DECLARE @sqlCounter15 INT
  DECLARE @sqlCounter16 INT
  DECLARE @sqlCounter17 INT
  DECLARE @sqlCounter18 INT

  SET @sqlCounter1 = 0
  SET @sqlCounter2 = 0
  SET @sqlCounter3 = 0
  SET @sqlCounter4 = 0
  SET @sqlCounter5 = 0
  SET @sqlCounter6 = 0
  SET @sqlCounter7 = 0
  SET @sqlCounter8 = 0
  SET @sqlCounter9 = 0
  SET @sqlCounter10 = 0
  SET @sqlCounter11= 0
  SET @sqlCounter12 = 0
  SET @sqlCounter13 = 0
  SET @sqlCounter14 = 0
  SET @sqlCounter15 = 0
  SET @sqlCounter16 = 0
  SET @sqlCounter17 = 0
  SET @sqlCounter18 = 0

  SET @start1 = CURRENT_TIMESTAMP

  DELETE FROM alobListCriteriaAlob              
  DELETE FROM alobListPattern                   
  DELETE FROM dynamic_object_create_path_detail 
  DELETE FROM dynamic_object_create_path        
  DELETE FROM alobListCriteriaToPath            
  DELETE FROM alobListPatternList  
  DELETE FROM alobListAttributeTemplate 
  DELETE FROM alobListCriteriaToFunctionGroup
  DELETE FROM alobListDefineAndUsePBTs
  DELETE FROM alobListDefineServicesAndLinkToEvUsers
  DELETE FROM alobListFunctionGroup
  DELETE FROM alobListFunctionGroupPrecedence
  DELETE FROM alobListPath
  DELETE FROM alobListSetSendStateChangeEvent
  DELETE FROM alobListSetThresholds
  DELETE FROM alobListSetWeights
  DELETE FROM alobListStaticFunctionGroup
  DELETE FROM alobListUserIDGroup
  DELETE FROM alobFunctionGroupTemplate
  DELETE FROM alobListAutoCopyEVService
  DELETE FROM alobListSetBSPriority

  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
        SET @start1 = CURRENT_TIMESTAMP
  END              

  SET NOCOUNT ON                                
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.5.bcp', 'PatternList', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.1.bcp', 'Pattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.2.bcp', 'CriteriaToPattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.10.bcp', 'SetThresholds', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.11.bcp', 'SetSendStateChangeEvent', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.12.bcp', 'DefineAndUsePBTs', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.13.bcp', 'SetWeights', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter8 = @elapsed_ms + @sqlCounter8
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.15.bcp', 'UserIDGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.14.bcp', 'DefineServicesAndLinkToEvUsers', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.16.bcp', 'AutoCopyEVService', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
        SET @start1 = CURRENT_TIMESTAMP
  END 

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.17.bcp', 'SetBSPriority', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END              
             

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.8.bcp', 'FunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter12 = @elapsed_ms + @sqlCounter12
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.9.bcp', 'FunctionGroupRelevance', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter13 = @elapsed_ms + @sqlCounter13
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.7.bcp', 'CriteriaToFunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter14 = @elapsed_ms + @sqlCounter14
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.3.bcp', 'Path', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter15 = @elapsed_ms + @sqlCounter15
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.4.bcp', 'CriteriaToPath', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter16 = @elapsed_ms + @sqlCounter16
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobBuildFunctionGroupTemplate   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter17 = @elapsed_ms + @sqlCounter17
        SET @start1 = CURRENT_TIMESTAMP
  END  

  DELETE FROM ev_watch_attr_filter -- else attributes build up over time
  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  -- Insert new records into table when there is no pre-existing data.
  INSERT INTO  ev_watch_attr_filter
  (
    cid,
    attribute_name,
    extended,
    active,
    salience,
    val_inserted_pat,
    val_deleted_pat,
    enqueue
  )
  SELECT
    t.cid,
    t.attribute_name,
    t.extended,
    t.active,
    t.salience,
    t.val_inserted_pat,
    t.val_inserted_pat,
    t.enqueue
  FROM ev_watch_attr_filter_temp t
  LEFT OUTER JOIN expanded_ev_watch_attr_filter AS eewaf
    ON eewaf.cid            = t.cid
   AND eewaf.attribute_name = t.attribute_name
  WHERE eewaf.cid IS NULL

  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
            
  IF @verbose >= 1
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('   alobLoadWrapperAbsConfig took %d ms', 0, 1,@elapsed_ms) WITH NOWAIT
    RAISERROR('     sqlCounter1 took %d ms', 0, 1,@sqlCounter1) WITH NOWAIT
    RAISERROR('     sqlCounter2 took %d ms', 0, 1,@sqlCounter2) WITH NOWAIT
    RAISERROR('     sqlCounter3 took %d ms', 0, 1,@sqlCounter3) WITH NOWAIT
    RAISERROR('     sqlCounter4 took %d ms', 0, 1,@sqlCounter4) WITH NOWAIT
    RAISERROR('     sqlCounter5 took %d ms', 0, 1,@sqlCounter5) WITH NOWAIT
    RAISERROR('     sqlCounter6 took %d ms', 0, 1,@sqlCounter6) WITH NOWAIT
    RAISERROR('     sqlCounter7 took %d ms', 0, 1,@sqlCounter7) WITH NOWAIT
    RAISERROR('     sqlCounter8 took %d ms', 0, 1,@sqlCounter8) WITH NOWAIT
    RAISERROR('     sqlCounter9 took %d ms', 0, 1,@sqlCounter9) WITH NOWAIT
    RAISERROR('     sqlCounter10 took %d ms', 0, 1,@sqlCounter10) WITH NOWAIT
    RAISERROR('     sqlCounter11 took %d ms', 0, 1,@sqlCounter11) WITH NOWAIT
    RAISERROR('     sqlCounter12 took %d ms', 0, 1,@sqlCounter12) WITH NOWAIT
    RAISERROR('     sqlCounter13 took %d ms', 0, 1,@sqlCounter13) WITH NOWAIT
    RAISERROR('     sqlCounter14 took %d ms', 0, 1,@sqlCounter14) WITH NOWAIT
    RAISERROR('     sqlCounter15 took %d ms', 0, 1,@sqlCounter15) WITH NOWAIT
    RAISERROR('     sqlCounter16 took %d ms', 0, 1,@sqlCounter16) WITH NOWAIT
    RAISERROR('     sqlCounter17 took %d ms', 0, 1,@sqlCounter17) WITH NOWAIT
    RAISERROR('     sqlCounter18 took %d ms', 0, 1,@sqlCounter18) WITH NOWAIT

  END




  COMMIT TRANSACTION absConfig                   
  RETURN 0
GO
  EXECUTE alobLoadWrapperAbsConfig
SQL_SCRIPT

  echo "" >> "${sFilenameLog}"
  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    cat "${sFilenameSqlOut}" >> "${sFilenameLog}"
    return 1
  fi

} # close of bcp_in_1_2

bcp_in_1_3()
{

awk -F"\x09" -v OFS="\x09" '

# /archive/scripts/absConfig.awk


BEGIN {

  nCountTypeRecord = 24

  # iStateRecord =  1 # Pattern Data
  # iStateRecord =  2 # CriteriaToPattern Data
  # iStateRecord =  3 # Path Data
  # iStateRecord =  4 # CriteriaToPath Data
  # iStateRecord =  5 # PatternList Data
  # iStateRecord =  6 # Header
  # iStateRecord =  7 # CriteriaToFunctionGroup
  # iStateRecord =  8 # FunctionGroup
  # iStateRecord =  9 # FunctionGroupRelevance
  # iStateRecord = 10 # SetThresholds
  # iStateRecord = 11 # SetSendStateChangeEvent
  # iStateRecord = 12 # DefineAndUsePBTs
  # iStateRecord = 13 # SetWeights
  # iStateRecord = 14 # DefineServicesAndLinkToEvUsers
  # iStateRecord = 15 # UserIDGroup
  # iStateRecord = 16 # AutoCopyEVService
  # iStateRecord = 17 # SetBSPriority
  # iStateRecord = 18 # BuildBusinessSystem
  # iStateRecord = 19 # SeedResources
  # iStateRecord = 20 # LinkTypes
  # iStateRecord = 21 # ClassList
  # iStateRecord = 22 # Exclusions
  # iStateRecord = 23 # LeafNodes
  # iStateRecord = 24 # PassThroughNodes



  # Initialize the record count for each of the record types
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    nCountRecord[ iStateRecord ] = 0
  }

  # Establish the number of fields per record type
  nCountField[ 1 ] = 7
  nCountField[ 2 ] = 3
  nCountField[ 3 ] = 6
  nCountField[ 4 ] = 7
  nCountField[ 5 ] = 2
  nCountField[ 6 ] = 1
  nCountField[ 7 ] = 3
  nCountField[ 8 ] = 3
  nCountField[ 9 ] = 2
  nCountField[10 ] = 5
  nCountField[11 ] = 2
  nCountField[12 ] = 15
  nCountField[13 ] = 2
  nCountField[14 ] = 8
  nCountField[15 ] = 2
  nCountField[16 ] = 2
  nCountField[17 ] = 2
  nCountField[18 ] = 8
  nCountField[19 ] = 3
  nCountField[20 ] = 5
  nCountField[21 ] = 2
  nCountField[22 ] = 4
  nCountField[23 ] = 4
  nCountField[24 ] = 4


  # Establish the filenames to receive bcp data
  if ( "" == FILENAME_PREFIX_OUTPUT ) {
    FILENAME_PREFIX_OUTPUT = "./absConfig.ksh."
  }
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    sFilenameBcp[ iStateRecord ] = FILENAME_PREFIX_OUTPUT iStateRecord ".bcp"
  }

  # Initialize the files to receive bcp data
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    system( "rm -f " sFilenameBcp[ iStateRecord ] )
    close(  "rm -f " sFilenameBcp[ iStateRecord ] )
    system( "touch " sFilenameBcp[ iStateRecord ] )
    close(  "touch " sFilenameBcp[ iStateRecord ] )
  }

  # Initialize the state of the current record
  iStateRecord = 0 
  
}

END {

  # Output the bcp files
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    for( iRecord = 1; iRecord <= nCountRecord[ iStateRecord ]; iRecord++ ) {
      print sRecord[ iStateRecord, iRecord ] > sFilenameBcp[ iStateRecord ]
    }
  } 
}


# Ignorable lines 

# Target: Remark lines
# #
/^[[:space:]]*#/ {

  next
}

# Target: Blank lines
/^[[:space:]]*$/ {

  next
}  

# Target: Column headers
# Pattern       Class   Attribute       When	Operator        Operand1   Operand2
( ( 1 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]/ ) { next }

# idCriteria      idPattern       idPatternRelated
( ( 2 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# sNamePathLob    idLevel sName
( ( 3 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]/ ) { next }

# idCriteria      sNamePathLob    idPattern       idLevel
( ( 4 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# ListName	Operand
( ( 5 == iStateRecord ) && /^[[:space:]]*[Ll][Ii][Ss][Tt][Nn][Aa][Mm][Ee][[:space:]]/ ) { next }

# ListName	Operand
( ( 6 == iStateRecord ) && /^[[:space:]]*[Vv][Ee][Rr][Ss][Ii][Oo][Nn][[:space:]]/ ) { next }

# Criteria   Pattern   FunctionGroup
( ( 7 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# FunctionGroup   FunctionName   Instance
( ( 8 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# FunctionGroup   Relevance
( ( 9 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# Instance   EventType   AlertState   Priority   Value
( ( 10 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 11 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   PBTName   Relevance   Enabled   MinAlertState   MaxAlertState   MinPercentage MaxPercentage
#            EventType EventName  EventText EventAlertState EventPriority   EventFlags    EventState
( ( 12 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   NewWeight
( ( 13 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   DisplayName   ServiceID   BusinessRole   RedImpact   YellowImpact   Flags   UserIDGroup
( ( 14 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   UserID
( ( 15 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 16 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 17 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   SeedResources	LinkTypes	EndLevel	Exclusions	LeafNodes
( ( 18 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Criteria	Pattern
( ( 19 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   SourceClasses	LinkType	DestinationClasses
( ( 20 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class
( ( 21 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 22 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 23 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 24 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }



# Header lines

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 1
  next
}

# Target: iStateRecord
# CriteriaToPattern
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 2
  next
}

# Target: iStateRecord
# Path
/^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 3
  next
}

# Target: iStateRecord
# CriteriaToPath
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Hh][[:space:]]*$/ {
  iStateRecord = 4
  next
}

# Target: iStateRecord
# PatternList
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 5
  next
}

# Target: iStateRecord
# Header
/^[[:space:]]*[Hh][Ee][Aa][Dd][Ee][Rr][[:space:]]*$/ {

  iStateRecord = 6
  next
}

# Target: iStateRecord
# CriteriaToFunctionGroup
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {
  iStateRecord = 7
  next
}


# Target: iStateRecord
# FunctionGroup
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 8
  next
}

# Target: iStateRecord
# FunctionGroupRelevance
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][Rr][Ee][Ll][Ee][Vv][Aa][Nn][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 9
  next
}


# Target: iStateRecord
# SetThresholds
/^[[:space:]]*[Ss][Ee][Tt][Tt][Hh][Rr][Ee][Ss][Hh][Oo][Ll][Dd][Ss][[:space:]]*$/ {

  iStateRecord = 10
  next
}

# Target: iStateRecord
# SetSendStateChangeEvent
/^[[:space:]]*[Ss][Ee][Tt][Ss][Ee][Nn][Dd][Ss][Tt][Aa][Tt][Ee][Cc][Hh][Aa][Nn][Gg][Ee][Ee][Vv][Ee][Nn][Tt][[:space:]]*$/ {

  iStateRecord = 11
  next
}

# Target: iStateRecord
# DefineAndUsePBTs
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Aa][Nn][Dd][Uu][Ss][Ee][Pp][Bb][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 12
  next
}

# Target: iStateRecord
# SetWeights
/^[[:space:]]*[Ss][Ee][Tt][Ww][Ee][Ii][Gg][Hh][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 13
  next
}

# Target: iStateRecord
# DefineServicesAndLinkToEvUsers
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Ss][Ee][Rr][Vv][Ii][Cc][Ee][Ss][Aa][Nn][Dd][Ll][Ii][Nn][Kk][Tt][Oo][Ee][Vv][Uu][Ss][Ee][Rr][Ss][[:space:]]*$/ {

  iStateRecord = 14
  next
}

# Target: iStateRecord
# UserIDGroup
/^[[:space:]]*[Uu][Ss][Ee][Rr][Ii][Dd][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 15
  next
}

# Target: iStateRecord
# AutoCopyEVService
/^[[:space:]]*[Aa][Uu][Tt][Oo][Cc][Oo][Pp][Yy][Ee][Vv][Ss][Ee][Rr][Vv][Ii][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 16
  next
}

# Target: iStateRecord
# SetBSPriority
/^[[:space:]]*[Ss][Ee][Tt][Bb][Ss][Pp][Rr][Ii][Oo][Rr][Ii][Tt][Yy][[:space:]]*$/ {

  iStateRecord = 17
  next
}

# Target: iStateRecord
# BuildBusinessSystem
/^[[:space:]]*[Bb][Uu][Ii][Ll][Dd][Bb][Uu][Ss][Ii][Nn][Ee][Ss][Ss][Ss][Yy][Ss][Tt][Ee][Mm][[:space:]]*$/ {

  iStateRecord = 18
  next
}

# Target: iStateRecord
# SeedResources
/^[[:space:]]*[Ss][Ee][Ee][Dd][Rr][Ee][Ss][Oo][Uu][Rr][Cc][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 19
  next
}

# Target: iStateRecord
# LinkTypes
/^[[:space:]]*[Ll][Ii][Nn][Kk][Tt][Yy][Pp][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 20
  next
}

# Target: iStateRecord
# ClassList
/^[[:space:]]*[Cc][Ll][Aa][Ss][Ss][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 21
  next
}

# Target: iStateRecord
# Exclusions
/^[[:space:]]*[Ee][Xx][Cc][Ll][Uu][Ss][Ii][Oo][Nn][Ss][[:space:]]*$/ {

  iStateRecord = 22
  next
}

# Target: iStateRecord
# LeafNodes
/^[[:space:]]*[Ll][Ee][Aa][Ff][Nn][Oo][Dd][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 23
  next
}

# Target: iStateRecord
# PassThroughNodes
/^[[:space:]]*[Pp][Aa][Ss][Ss][Tt][Hh][Rr][Oo][Uu][Gg][Hh][Nn][Oo][Dd][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 24
  next
}


# Data Records

# Target: 
// { 
  # HACK
  # print iStateRecord, $0
}

# Target: sRecord
( 0 != iStateRecord ) {

  # HACK
  # print "In the data record for iStateRecord = " iStateRecord

  # Break $0 into an array and then reform into bcp output
  delete sField
  split( $0, sField, FS )

  sRecordBcp = ""
  for( iField = 1; iField <= nCountField[ iStateRecord ] ; iField++ ) {
    sRecordBcp = sRecordBcp OFS sField[ iField ]
  } 
  sub( OFS, "", sRecordBcp )

  # Store bcp record for later output ( see END )
  sRecord[ iStateRecord, ++nCountRecord[ iStateRecord ] ] = sRecordBcp

}


' "${sFilenameIn}"


  # Ok at this point we need to copy the bcp files to the database server for ease of loading:

  cp -v ./absConfig.ksh.[123456789].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  cp -v ./absConfig.ksh.1[0123456789].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  cp -v ./absConfig.ksh.2[01234].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"

  rm -f ./absConfig.ksh.[123456789].bcp 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.1[0123456789].bcp 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.2[01234].bcp 1>> "${sFilenameLog}"

  echo "" >> "${sFilenameLog}"
  echo "bcp_in creating sql in script:" >> "${sFilenameLog}"

  rm -f "${sFilenameSqlIn}"
  touch "${sFilenameSqlIn}"

  cat <<SQL_SCRIPT > "${sFilenameSqlIn}"


Use Object
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadWrapperAbsConfig' AND type = 'P' )
  DROP PROCEDURE alobLoadWrapperAbsConfig
GO
   
${sTruncate}

CREATE PROCEDURE alobLoadWrapperAbsConfig
  
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF

  TRUNCATE TABLE alobFilePattern
  TRUNCATE TABLE alobFileCriteriaToPattern
  TRUNCATE TABLE alobFilePath
  TRUNCATE TABLE alobFileCriteriaToPath
  TRUNCATE TABLE alobFilePatternList
  TRUNCATE TABLE alobFilePath7
  TRUNCATE TABLE alobFilePath8
  TRUNCATE TABLE alobFilePath27
  TRUNCATE TABLE alobFileCriteriaToPath9
  TRUNCATE TABLE alobFileDefineAndUsePBTs
  TRUNCATE TABLE alobFileDefineServicesAndLinkToEvUsers
  TRUNCATE TABLE alobFileSetSendStateChangeEvent
  TRUNCATE TABLE alobFileSetWeights
  TRUNCATE TABLE alobFileSetThresholds
  TRUNCATE TABLE alobFileFunctionGroup
  TRUNCATE TABLE alobFileCriteriaToFunctionGroup
  TRUNCATE TABLE alobFileUserIDGroup
  TRUNCATE TABLE alobFileFunctionGroupPrecedence
  TRUNCATE TABLE alobFileAutoCopyEVService
  TRUNCATE TABLE alobFileSetBSPriority
  TRUNCATE TABLE alobFileBuildRelationshipView
  TRUNCATE TABLE alobFileSeedResources
  TRUNCATE TABLE alobFileLinkTypes
  TRUNCATE TABLE alobFileClassList
  TRUNCATE TABLE alobFileExclusions
  TRUNCATE TABLE alobFileLeafNodes
  TRUNCATE TABLE alobFilePassThroughNodes
  
  
  BEGIN TRANSACTION absConfig                   

  DECLARE @nError INTEGER

  DECLARE @now NVARCHAR(40)
  DECLARE @verbose TINYINT
  
  ${sTiming}

  DECLARE @start DATETIME
  DECLARE @start1 DATETIME
  SET @start = CURRENT_TIMESTAMP
  DECLARE @elapsed_ms INT
  DECLARE @sqlCounter1 INT
  DECLARE @sqlCounter2 INT
  DECLARE @sqlCounter3 INT
  DECLARE @sqlCounter4 INT
  DECLARE @sqlCounter5 INT
  DECLARE @sqlCounter6 INT
  DECLARE @sqlCounter7 INT
  DECLARE @sqlCounter8 INT
  DECLARE @sqlCounter9 INT
  DECLARE @sqlCounter10 INT
  DECLARE @sqlCounter11 INT
  DECLARE @sqlCounter12 INT
  DECLARE @sqlCounter13 INT
  DECLARE @sqlCounter14 INT
  DECLARE @sqlCounter15 INT
  DECLARE @sqlCounter16 INT
  DECLARE @sqlCounter17 INT
  DECLARE @sqlCounter18 INT
  DECLARE @sqlCounter19 INT

  SET @sqlCounter1 = 0
  SET @sqlCounter2 = 0
  SET @sqlCounter3 = 0
  SET @sqlCounter4 = 0
  SET @sqlCounter5 = 0
  SET @sqlCounter6 = 0
  SET @sqlCounter7 = 0
  SET @sqlCounter8 = 0
  SET @sqlCounter9 = 0
  SET @sqlCounter10 = 0
  SET @sqlCounter11= 0
  SET @sqlCounter12 = 0
  SET @sqlCounter13 = 0
  SET @sqlCounter14 = 0
  SET @sqlCounter15 = 0
  SET @sqlCounter16 = 0
  SET @sqlCounter17 = 0
  SET @sqlCounter18 = 0
  SET @sqlCounter19 = 0

  SET @start1 = CURRENT_TIMESTAMP

  DELETE FROM alobListCriteriaAlob              
  DELETE FROM alobListPattern                   
  DELETE FROM dynamic_object_create_path_detail 
  DELETE FROM dynamic_object_create_path        
  DELETE FROM alobListCriteriaToPath            
  DELETE FROM alobListPatternList  
  DELETE FROM alobListAttributeTemplate 
  DELETE FROM alobListCriteriaToFunctionGroup
  DELETE FROM alobListDefineAndUsePBTs
  DELETE FROM alobListDefineServicesAndLinkToEvUsers
  DELETE FROM alobListFunctionGroup
  DELETE FROM alobListFunctionGroupPrecedence
  DELETE FROM alobListPath
  DELETE FROM alobListSetSendStateChangeEvent
  DELETE FROM alobListSetThresholds
  DELETE FROM alobListSetWeights
  DELETE FROM alobListStaticFunctionGroup
  DELETE FROM alobListUserIDGroup
  DELETE FROM alobFunctionGroupTemplate
  DELETE FROM alobListAutoCopyEVService
  DELETE FROM alobListSetBSPriority
  DELETE FROM alobListBuildRelationshipView
  DELETE FROM alobListSeedResources
  DELETE FROM alobListLinkTypes
  DELETE FROM alobListClassList
  DELETE FROM alobListExclusions
  DELETE FROM alobListLeafNodes
  DELETE FROM alobListPassThroughNodes


  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
        SET @start1 = CURRENT_TIMESTAMP
  END              

  SET NOCOUNT ON                                
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.5.bcp', 'PatternList', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.1.bcp', 'Pattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.2.bcp', 'CriteriaToPattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.10.bcp', 'SetThresholds', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.11.bcp', 'SetSendStateChangeEvent', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.12.bcp', 'DefineAndUsePBTs', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.13.bcp', 'SetWeights', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter8 = @elapsed_ms + @sqlCounter8
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.15.bcp', 'UserIDGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.14.bcp', 'DefineServicesAndLinkToEvUsers', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.16.bcp', 'AutoCopyEVService', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
        SET @start1 = CURRENT_TIMESTAMP
  END 

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.17.bcp', 'SetBSPriority', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.21.bcp', 'ClassList', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END 
             
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.22.bcp', 'Exclusions', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END             
 
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.23.bcp', 'LeafNodes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END  

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.24.bcp', 'PassThroughNodes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter19 = @elapsed_ms + @sqlCounter19
        SET @start1 = CURRENT_TIMESTAMP
  END   
 
           
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.19.bcp', 'SeedResources', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END 
             
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.20.bcp', 'LinkTypes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END   
           
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.18.bcp', 'BuildBusinessSystem', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END              
       

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.8.bcp', 'FunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter12 = @elapsed_ms + @sqlCounter12
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.9.bcp', 'FunctionGroupRelevance', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter13 = @elapsed_ms + @sqlCounter13
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.7.bcp', 'CriteriaToFunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter14 = @elapsed_ms + @sqlCounter14
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.3.bcp', 'Path', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter15 = @elapsed_ms + @sqlCounter15
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.4.bcp', 'CriteriaToPath', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter16 = @elapsed_ms + @sqlCounter16
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobBuildFunctionGroupTemplate   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter17 = @elapsed_ms + @sqlCounter17
        SET @start1 = CURRENT_TIMESTAMP
  END  

  DELETE FROM ev_watch_attr_filter -- else attributes build up over time
  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  -- Insert new records into table when there is no pre-existing data.
  INSERT INTO  ev_watch_attr_filter
  (
    cid,
    attribute_name,
    extended,
    active,
    salience,
    val_inserted_pat,
    val_deleted_pat,
    enqueue
  )
  SELECT
    t.cid,
    t.attribute_name,
    t.extended,
    t.active,
    t.salience,
    t.val_inserted_pat,
    t.val_inserted_pat,
    t.enqueue
  FROM ev_watch_attr_filter_temp t
  LEFT OUTER JOIN expanded_ev_watch_attr_filter AS eewaf
    ON eewaf.cid            = t.cid
   AND eewaf.attribute_name = t.attribute_name
  WHERE eewaf.cid IS NULL

  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
            
  IF @verbose >= 1
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('   alobLoadWrapperAbsConfig took %d ms', 0, 1,@elapsed_ms) WITH NOWAIT
    RAISERROR('     sqlCounter1 took %d ms', 0, 1,@sqlCounter1) WITH NOWAIT
    RAISERROR('     sqlCounter2 took %d ms', 0, 1,@sqlCounter2) WITH NOWAIT
    RAISERROR('     sqlCounter3 took %d ms', 0, 1,@sqlCounter3) WITH NOWAIT
    RAISERROR('     sqlCounter4 took %d ms', 0, 1,@sqlCounter4) WITH NOWAIT
    RAISERROR('     sqlCounter5 took %d ms', 0, 1,@sqlCounter5) WITH NOWAIT
    RAISERROR('     sqlCounter6 took %d ms', 0, 1,@sqlCounter6) WITH NOWAIT
    RAISERROR('     sqlCounter7 took %d ms', 0, 1,@sqlCounter7) WITH NOWAIT
    RAISERROR('     sqlCounter8 took %d ms', 0, 1,@sqlCounter8) WITH NOWAIT
    RAISERROR('     sqlCounter9 took %d ms', 0, 1,@sqlCounter9) WITH NOWAIT
    RAISERROR('     sqlCounter10 took %d ms', 0, 1,@sqlCounter10) WITH NOWAIT
    RAISERROR('     sqlCounter11 took %d ms', 0, 1,@sqlCounter11) WITH NOWAIT
    RAISERROR('     sqlCounter12 took %d ms', 0, 1,@sqlCounter12) WITH NOWAIT
    RAISERROR('     sqlCounter13 took %d ms', 0, 1,@sqlCounter13) WITH NOWAIT
    RAISERROR('     sqlCounter14 took %d ms', 0, 1,@sqlCounter14) WITH NOWAIT
    RAISERROR('     sqlCounter15 took %d ms', 0, 1,@sqlCounter15) WITH NOWAIT
    RAISERROR('     sqlCounter16 took %d ms', 0, 1,@sqlCounter16) WITH NOWAIT
    RAISERROR('     sqlCounter17 took %d ms', 0, 1,@sqlCounter17) WITH NOWAIT
    RAISERROR('     sqlCounter18 took %d ms', 0, 1,@sqlCounter18) WITH NOWAIT
    RAISERROR('     sqlCounter19 took %d ms', 0, 1,@sqlCounter19) WITH NOWAIT

  END




  COMMIT TRANSACTION absConfig                   
  RETURN 0
GO
  EXECUTE alobLoadWrapperAbsConfig
SQL_SCRIPT

  echo "" >> "${sFilenameLog}"
  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    cat "${sFilenameSqlOut}" >> "${sFilenameLog}"
    return 1
  fi

} # close of bcp_in_1_3

#
# New header 1.4 to handle a new column for the Pattern section
#
bcp_in_1_4()
{

awk -F"\x09" -v OFS="\x09" '

# /archive/scripts/absConfig.awk


BEGIN {

  nCountTypeRecord = 24

  # iStateRecord =  1 # Pattern Data
  # iStateRecord =  2 # CriteriaToPattern Data
  # iStateRecord =  3 # Path Data
  # iStateRecord =  4 # CriteriaToPath Data
  # iStateRecord =  5 # PatternList Data
  # iStateRecord =  6 # Header
  # iStateRecord =  7 # CriteriaToFunctionGroup
  # iStateRecord =  8 # FunctionGroup
  # iStateRecord =  9 # FunctionGroupRelevance
  # iStateRecord = 10 # SetThresholds
  # iStateRecord = 11 # SetSendStateChangeEvent
  # iStateRecord = 12 # DefineAndUsePBTs
  # iStateRecord = 13 # SetWeights
  # iStateRecord = 14 # DefineServicesAndLinkToEvUsers
  # iStateRecord = 15 # UserIDGroup
  # iStateRecord = 16 # AutoCopyEVService
  # iStateRecord = 17 # SetBSPriority
  # iStateRecord = 18 # BuildBusinessSystem
  # iStateRecord = 19 # SeedResources
  # iStateRecord = 20 # LinkTypes
  # iStateRecord = 21 # ClassList
  # iStateRecord = 22 # Exclusions
  # iStateRecord = 23 # LeafNodes
  # iStateRecord = 24 # PassThroughNodes



  # Initialize the record count for each of the record types
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    nCountRecord[ iStateRecord ] = 0
  }

  # Establish the number of fields per record type
  nCountField[ 1 ] = 8
  nCountField[ 2 ] = 3
  nCountField[ 3 ] = 6
  nCountField[ 4 ] = 7
  nCountField[ 5 ] = 2
  nCountField[ 6 ] = 1
  nCountField[ 7 ] = 3
  nCountField[ 8 ] = 3
  nCountField[ 9 ] = 2
  nCountField[10 ] = 5
  nCountField[11 ] = 2
  nCountField[12 ] = 15
  nCountField[13 ] = 2
  nCountField[14 ] = 8
  nCountField[15 ] = 2
  nCountField[16 ] = 2
  nCountField[17 ] = 2
  nCountField[18 ] = 8
  nCountField[19 ] = 3
  nCountField[20 ] = 5
  nCountField[21 ] = 2
  nCountField[22 ] = 4
  nCountField[23 ] = 4
  nCountField[24 ] = 4


  # Establish the filenames to receive bcp data
  if ( "" == FILENAME_PREFIX_OUTPUT ) {
    FILENAME_PREFIX_OUTPUT = "./absConfig.ksh."
  }
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    sFilenameBcp[ iStateRecord ] = FILENAME_PREFIX_OUTPUT iStateRecord ".bcp"
  }

  # Initialize the files to receive bcp data
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    system( "rm -f " sFilenameBcp[ iStateRecord ] )
    close(  "rm -f " sFilenameBcp[ iStateRecord ] )
    system( "touch " sFilenameBcp[ iStateRecord ] )
    close(  "touch " sFilenameBcp[ iStateRecord ] )
  }

  # Initialize the state of the current record
  iStateRecord = 0 
  
}

END {

  # Output the bcp files
  for( iStateRecord = 1; iStateRecord <= nCountTypeRecord; iStateRecord++ ) {
    for( iRecord = 1; iRecord <= nCountRecord[ iStateRecord ]; iRecord++ ) {
      print sRecord[ iStateRecord, iRecord ] > sFilenameBcp[ iStateRecord ]
    }
  } 
}


# Ignorable lines 

# Target: Remark lines
# #
/^[[:space:]]*#/ {

  next
}

# Target: Blank lines
/^[[:space:]]*$/ {

  next
}  

# Target: Column headers
# Pattern       Class   Attribute       When	Operator        Operand1   Operand2    PatternUsage
( ( 1 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]/ ) { next }

# idCriteria      idPattern       idPatternRelated
( ( 2 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# sNamePathLob    idLevel sName
( ( 3 == iStateRecord ) && /^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]/ ) { next }

# idCriteria      sNamePathLob    idPattern       idLevel
( ( 4 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# ListName	Operand
( ( 5 == iStateRecord ) && /^[[:space:]]*[Ll][Ii][Ss][Tt][Nn][Aa][Mm][Ee][[:space:]]/ ) { next }

# ListName	Operand
( ( 6 == iStateRecord ) && /^[[:space:]]*[Vv][Ee][Rr][Ss][Ii][Oo][Nn][[:space:]]/ ) { next }

# Criteria   Pattern   FunctionGroup
( ( 7 == iStateRecord ) && /^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][[:space:]]/ ) { next }

# FunctionGroup   FunctionName   Instance
( ( 8 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# FunctionGroup   Relevance
( ( 9 == iStateRecord ) && /^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]/ ) { next }

# Instance   EventType   AlertState   Priority   Value
( ( 10 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 11 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   PBTName   Relevance   Enabled   MinAlertState   MaxAlertState   MinPercentage MaxPercentage
#            EventType EventName  EventText EventAlertState EventPriority   EventFlags    EventState
( ( 12 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   NewWeight
( ( 13 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   DisplayName   ServiceID   BusinessRole   RedImpact   YellowImpact   Flags   UserIDGroup
( ( 14 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   UserID
( ( 15 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 16 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Value
( ( 17 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   SeedResources	LinkTypes	EndLevel	Exclusions	LeafNodes
( ( 18 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Criteria	Pattern
( ( 19 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   SourceClasses	LinkType	DestinationClasses
( ( 20 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class
( ( 21 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 22 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 23 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }

# Instance   Class	ObjectID	InstanceID
( ( 24 == iStateRecord ) && /^[[:space:]]*[Ii][Nn][Ss][Tt][Aa][Nn][Cc][Ee][[:space:]]/ ) { next }



# Header lines

# Target: iStateRecord
# Pattern
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 1
  next
}

# Target: iStateRecord
# CriteriaToPattern
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Tt][Ee][Rr][Nn][[:space:]]*$/ {

  iStateRecord = 2
  next
}

# Target: iStateRecord
# Path
/^[[:space:]]*[Pp][Aa][Tt][Hh][[:space:]]*$/ {

  iStateRecord = 3
  next
}

# Target: iStateRecord
# CriteriaToPath
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Pp][Aa][Tt][Hh][[:space:]]*$/ {
  iStateRecord = 4
  next
}

# Target: iStateRecord
# PatternList
/^[[:space:]]*[Pp][Aa][Tt][Tt][Ee][Rr][Nn][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 5
  next
}

# Target: iStateRecord
# Header
/^[[:space:]]*[Hh][Ee][Aa][Dd][Ee][Rr][[:space:]]*$/ {

  iStateRecord = 6
  next
}

# Target: iStateRecord
# CriteriaToFunctionGroup
/^[[:space:]]*[Cc][Rr][Ii][Tt][Ee][Rr][Ii][Aa][Tt][Oo][Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {
  iStateRecord = 7
  next
}


# Target: iStateRecord
# FunctionGroup
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 8
  next
}

# Target: iStateRecord
# FunctionGroupRelevance
/^[[:space:]]*[Ff][Uu][Nn][Cc][Tt][Ii][Oo][Nn][Gg][Rr][Oo][Uu][Pp][Rr][Ee][Ll][Ee][Vv][Aa][Nn][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 9
  next
}


# Target: iStateRecord
# SetThresholds
/^[[:space:]]*[Ss][Ee][Tt][Tt][Hh][Rr][Ee][Ss][Hh][Oo][Ll][Dd][Ss][[:space:]]*$/ {

  iStateRecord = 10
  next
}

# Target: iStateRecord
# SetSendStateChangeEvent
/^[[:space:]]*[Ss][Ee][Tt][Ss][Ee][Nn][Dd][Ss][Tt][Aa][Tt][Ee][Cc][Hh][Aa][Nn][Gg][Ee][Ee][Vv][Ee][Nn][Tt][[:space:]]*$/ {

  iStateRecord = 11
  next
}

# Target: iStateRecord
# DefineAndUsePBTs
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Aa][Nn][Dd][Uu][Ss][Ee][Pp][Bb][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 12
  next
}

# Target: iStateRecord
# SetWeights
/^[[:space:]]*[Ss][Ee][Tt][Ww][Ee][Ii][Gg][Hh][Tt][Ss][[:space:]]*$/ {

  iStateRecord = 13
  next
}

# Target: iStateRecord
# DefineServicesAndLinkToEvUsers
/^[[:space:]]*[Dd][Ee][Ff][Ii][Nn][Ee][Ss][Ee][Rr][Vv][Ii][Cc][Ee][Ss][Aa][Nn][Dd][Ll][Ii][Nn][Kk][Tt][Oo][Ee][Vv][Uu][Ss][Ee][Rr][Ss][[:space:]]*$/ {

  iStateRecord = 14
  next
}

# Target: iStateRecord
# UserIDGroup
/^[[:space:]]*[Uu][Ss][Ee][Rr][Ii][Dd][Gg][Rr][Oo][Uu][Pp][[:space:]]*$/ {

  iStateRecord = 15
  next
}

# Target: iStateRecord
# AutoCopyEVService
/^[[:space:]]*[Aa][Uu][Tt][Oo][Cc][Oo][Pp][Yy][Ee][Vv][Ss][Ee][Rr][Vv][Ii][Cc][Ee][[:space:]]*$/ {

  iStateRecord = 16
  next
}

# Target: iStateRecord
# SetBSPriority
/^[[:space:]]*[Ss][Ee][Tt][Bb][Ss][Pp][Rr][Ii][Oo][Rr][Ii][Tt][Yy][[:space:]]*$/ {

  iStateRecord = 17
  next
}

# Target: iStateRecord
# BuildBusinessSystem
/^[[:space:]]*[Bb][Uu][Ii][Ll][Dd][Bb][Uu][Ss][Ii][Nn][Ee][Ss][Ss][Ss][Yy][Ss][Tt][Ee][Mm][[:space:]]*$/ {

  iStateRecord = 18
  next
}

# Target: iStateRecord
# SeedResources
/^[[:space:]]*[Ss][Ee][Ee][Dd][Rr][Ee][Ss][Oo][Uu][Rr][Cc][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 19
  next
}

# Target: iStateRecord
# LinkTypes
/^[[:space:]]*[Ll][Ii][Nn][Kk][Tt][Yy][Pp][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 20
  next
}

# Target: iStateRecord
# ClassList
/^[[:space:]]*[Cc][Ll][Aa][Ss][Ss][Ll][Ii][Ss][Tt][[:space:]]*$/ {

  iStateRecord = 21
  next
}

# Target: iStateRecord
# Exclusions
/^[[:space:]]*[Ee][Xx][Cc][Ll][Uu][Ss][Ii][Oo][Nn][Ss][[:space:]]*$/ {

  iStateRecord = 22
  next
}

# Target: iStateRecord
# LeafNodes
/^[[:space:]]*[Ll][Ee][Aa][Ff][Nn][Oo][Dd][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 23
  next
}

# Target: iStateRecord
# PassThroughNodes
/^[[:space:]]*[Pp][Aa][Ss][Ss][Tt][Hh][Rr][Oo][Uu][Gg][Hh][Nn][Oo][Dd][Ee][Ss][[:space:]]*$/ {

  iStateRecord = 24
  next
}


# Data Records

# Target: 
// { 
  # HACK
  # print iStateRecord, $0
}

# Target: sRecord
( 0 != iStateRecord ) {

  # HACK
  # print "In the data record for iStateRecord = " iStateRecord

  # Break $0 into an array and then reform into bcp output
  delete sField
  split( $0, sField, FS )

  sRecordBcp = ""
  for( iField = 1; iField <= nCountField[ iStateRecord ] ; iField++ ) {
    sRecordBcp = sRecordBcp OFS sField[ iField ]
  } 
  sub( OFS, "", sRecordBcp )

  # Store bcp record for later output ( see END )
  sRecord[ iStateRecord, ++nCountRecord[ iStateRecord ] ] = sRecordBcp

}


' "${sFilenameIn}"


  # Ok at this point we need to copy the bcp files to the database server for ease of loading:

  cp -v ./absConfig.ksh.[123456789].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  cp -v ./absConfig.ksh.1[0123456789].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"
  cp -v ./absConfig.ksh.2[01234].bcp //${db_server}/Logs$ 1>> "${sFilenameLog}"

  rm -f ./absConfig.ksh.[123456789].bcp 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.1[0123456789].bcp 1>> "${sFilenameLog}"
  rm -f ./absConfig.ksh.2[01234].bcp 1>> "${sFilenameLog}"

  echo "" >> "${sFilenameLog}"
  echo "bcp_in creating sql in script:" >> "${sFilenameLog}"

  rm -f "${sFilenameSqlIn}"
  touch "${sFilenameSqlIn}"

  cat <<SQL_SCRIPT > "${sFilenameSqlIn}"


Use Object
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadWrapperAbsConfig' AND type = 'P' )
  DROP PROCEDURE alobLoadWrapperAbsConfig
GO
   
${sTruncate}

CREATE PROCEDURE alobLoadWrapperAbsConfig
  
AS
  -- setup database options
  SET CONCAT_NULL_YIELDS_NULL OFF

  TRUNCATE TABLE alobFilePattern
  TRUNCATE TABLE alobFilePattern31
  TRUNCATE TABLE alobFileCriteriaToPattern
  TRUNCATE TABLE alobFilePath
  TRUNCATE TABLE alobFileCriteriaToPath
  TRUNCATE TABLE alobFilePatternList
  TRUNCATE TABLE alobFilePath7
  TRUNCATE TABLE alobFilePath8
  TRUNCATE TABLE alobFilePath27
  TRUNCATE TABLE alobFileCriteriaToPath9
  TRUNCATE TABLE alobFileDefineAndUsePBTs
  TRUNCATE TABLE alobFileDefineServicesAndLinkToEvUsers
  TRUNCATE TABLE alobFileSetSendStateChangeEvent
  TRUNCATE TABLE alobFileSetWeights
  TRUNCATE TABLE alobFileSetThresholds
  TRUNCATE TABLE alobFileFunctionGroup
  TRUNCATE TABLE alobFileCriteriaToFunctionGroup
  TRUNCATE TABLE alobFileUserIDGroup
  TRUNCATE TABLE alobFileFunctionGroupPrecedence
  TRUNCATE TABLE alobFileAutoCopyEVService
  TRUNCATE TABLE alobFileSetBSPriority
  TRUNCATE TABLE alobFileBuildRelationshipView
  TRUNCATE TABLE alobFileSeedResources
  TRUNCATE TABLE alobFileLinkTypes
  TRUNCATE TABLE alobFileClassList
  TRUNCATE TABLE alobFileExclusions
  TRUNCATE TABLE alobFileLeafNodes
  TRUNCATE TABLE alobFilePassThroughNodes
  
  
  BEGIN TRANSACTION absConfig                   

  DECLARE @nError INTEGER

  DECLARE @now NVARCHAR(40)
  DECLARE @verbose TINYINT
  
  ${sTiming}

  DECLARE @start DATETIME
  DECLARE @start1 DATETIME
  SET @start = CURRENT_TIMESTAMP
  DECLARE @elapsed_ms INT
  DECLARE @sqlCounter1 INT
  DECLARE @sqlCounter2 INT
  DECLARE @sqlCounter3 INT
  DECLARE @sqlCounter4 INT
  DECLARE @sqlCounter5 INT
  DECLARE @sqlCounter6 INT
  DECLARE @sqlCounter7 INT
  DECLARE @sqlCounter8 INT
  DECLARE @sqlCounter9 INT
  DECLARE @sqlCounter10 INT
  DECLARE @sqlCounter11 INT
  DECLARE @sqlCounter12 INT
  DECLARE @sqlCounter13 INT
  DECLARE @sqlCounter14 INT
  DECLARE @sqlCounter15 INT
  DECLARE @sqlCounter16 INT
  DECLARE @sqlCounter17 INT
  DECLARE @sqlCounter18 INT
  DECLARE @sqlCounter19 INT

  SET @sqlCounter1 = 0
  SET @sqlCounter2 = 0
  SET @sqlCounter3 = 0
  SET @sqlCounter4 = 0
  SET @sqlCounter5 = 0
  SET @sqlCounter6 = 0
  SET @sqlCounter7 = 0
  SET @sqlCounter8 = 0
  SET @sqlCounter9 = 0
  SET @sqlCounter10 = 0
  SET @sqlCounter11= 0
  SET @sqlCounter12 = 0
  SET @sqlCounter13 = 0
  SET @sqlCounter14 = 0
  SET @sqlCounter15 = 0
  SET @sqlCounter16 = 0
  SET @sqlCounter17 = 0
  SET @sqlCounter18 = 0
  SET @sqlCounter19 = 0

  SET @start1 = CURRENT_TIMESTAMP

  DELETE FROM alobListCriteriaAlob              
  DELETE FROM alobListPattern                   
  DELETE FROM dynamic_object_create_path_detail 
  DELETE FROM dynamic_object_create_path        
  DELETE FROM alobListCriteriaToPath            
  DELETE FROM alobListPatternList  
  DELETE FROM alobListAttributeTemplate 
  DELETE FROM alobListCriteriaToFunctionGroup
  DELETE FROM alobListDefineAndUsePBTs
  DELETE FROM alobListDefineServicesAndLinkToEvUsers
  DELETE FROM alobListFunctionGroup
  DELETE FROM alobListFunctionGroupPrecedence
  DELETE FROM alobListPath
  DELETE FROM alobListSetSendStateChangeEvent
  DELETE FROM alobListSetThresholds
  DELETE FROM alobListSetWeights
  DELETE FROM alobListStaticFunctionGroup
  DELETE FROM alobListUserIDGroup
  DELETE FROM alobFunctionGroupTemplate
  DELETE FROM alobListAutoCopyEVService
  DELETE FROM alobListSetBSPriority
  DELETE FROM alobListBuildRelationshipView
  DELETE FROM alobListSeedResources
  DELETE FROM alobListLinkTypes
  DELETE FROM alobListClassList
  DELETE FROM alobListExclusions
  DELETE FROM alobListLeafNodes
  DELETE FROM alobListPassThroughNodes


  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
        SET @start1 = CURRENT_TIMESTAMP
  END              

  SET NOCOUNT ON                                
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.5.bcp', 'PatternList', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.1.bcp', 'Pattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.2.bcp', 'CriteriaToPattern', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.10.bcp', 'SetThresholds', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.11.bcp', 'SetSendStateChangeEvent', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.12.bcp', 'DefineAndUsePBTs', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.13.bcp', 'SetWeights', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter8 = @elapsed_ms + @sqlCounter8
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.15.bcp', 'UserIDGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.14.bcp', 'DefineServicesAndLinkToEvUsers', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.16.bcp', 'AutoCopyEVService', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
        SET @start1 = CURRENT_TIMESTAMP
  END 

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.17.bcp', 'SetBSPriority', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.21.bcp', 'ClassList', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END 
             
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.22.bcp', 'Exclusions', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END             
 
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.23.bcp', 'LeafNodes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END  

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.24.bcp', 'PassThroughNodes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter19 = @elapsed_ms + @sqlCounter19
        SET @start1 = CURRENT_TIMESTAMP
  END   
 
           
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.19.bcp', 'SeedResources', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END 
             
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.20.bcp', 'LinkTypes', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END   
           
  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.18.bcp', 'BuildBusinessSystem', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter18 = @elapsed_ms + @sqlCounter18
        SET @start1 = CURRENT_TIMESTAMP
  END              
       

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.8.bcp', 'FunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter12 = @elapsed_ms + @sqlCounter12
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.9.bcp', 'FunctionGroupRelevance', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter13 = @elapsed_ms + @sqlCounter13
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.7.bcp', 'CriteriaToFunctionGroup', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter14 = @elapsed_ms + @sqlCounter14
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.3.bcp', 'Path', '${sCodepage}', ${sVersionIn}, ${sVerbose}
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter15 = @elapsed_ms + @sqlCounter15
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobLoadConfigBcp '//${db_server}/Logs$/absConfig.ksh.4.bcp', 'CriteriaToPath', '${sCodepage}', ${sVersionIn}, ${sVerbose}   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter16 = @elapsed_ms + @sqlCounter16
        SET @start1 = CURRENT_TIMESTAMP
  END              

  EXECUTE @nError = alobBuildFunctionGroupTemplate   
  IF ( 0 <> @nError )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter17 = @elapsed_ms + @sqlCounter17
        SET @start1 = CURRENT_TIMESTAMP
  END  

  DELETE FROM ev_watch_attr_filter -- else attributes build up over time
  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END

  -- Insert new records into table when there is no pre-existing data.
  INSERT INTO  ev_watch_attr_filter
  (
    cid,
    attribute_name,
    extended,
    active,
    salience,
    val_inserted_pat,
    val_deleted_pat,
    enqueue
  )
  SELECT
    t.cid,
    t.attribute_name,
    t.extended,
    t.active,
    t.salience,
    t.val_inserted_pat,
    t.val_inserted_pat,
    t.enqueue
  FROM ev_watch_attr_filter_temp t
  LEFT OUTER JOIN expanded_ev_watch_attr_filter AS eewaf
    ON eewaf.cid            = t.cid
   AND eewaf.attribute_name = t.attribute_name
  WHERE eewaf.cid IS NULL

  IF ( 0 <> @@error )
  BEGIN
    ROLLBACK TRANSACTION absConfig
    RETURN -30000
  END
            
  IF @verbose >= 1
  BEGIN
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('   alobLoadWrapperAbsConfig took %d ms', 0, 1,@elapsed_ms) WITH NOWAIT
    RAISERROR('     sqlCounter1 took %d ms', 0, 1,@sqlCounter1) WITH NOWAIT
    RAISERROR('     sqlCounter2 took %d ms', 0, 1,@sqlCounter2) WITH NOWAIT
    RAISERROR('     sqlCounter3 took %d ms', 0, 1,@sqlCounter3) WITH NOWAIT
    RAISERROR('     sqlCounter4 took %d ms', 0, 1,@sqlCounter4) WITH NOWAIT
    RAISERROR('     sqlCounter5 took %d ms', 0, 1,@sqlCounter5) WITH NOWAIT
    RAISERROR('     sqlCounter6 took %d ms', 0, 1,@sqlCounter6) WITH NOWAIT
    RAISERROR('     sqlCounter7 took %d ms', 0, 1,@sqlCounter7) WITH NOWAIT
    RAISERROR('     sqlCounter8 took %d ms', 0, 1,@sqlCounter8) WITH NOWAIT
    RAISERROR('     sqlCounter9 took %d ms', 0, 1,@sqlCounter9) WITH NOWAIT
    RAISERROR('     sqlCounter10 took %d ms', 0, 1,@sqlCounter10) WITH NOWAIT
    RAISERROR('     sqlCounter11 took %d ms', 0, 1,@sqlCounter11) WITH NOWAIT
    RAISERROR('     sqlCounter12 took %d ms', 0, 1,@sqlCounter12) WITH NOWAIT
    RAISERROR('     sqlCounter13 took %d ms', 0, 1,@sqlCounter13) WITH NOWAIT
    RAISERROR('     sqlCounter14 took %d ms', 0, 1,@sqlCounter14) WITH NOWAIT
    RAISERROR('     sqlCounter15 took %d ms', 0, 1,@sqlCounter15) WITH NOWAIT
    RAISERROR('     sqlCounter16 took %d ms', 0, 1,@sqlCounter16) WITH NOWAIT
    RAISERROR('     sqlCounter17 took %d ms', 0, 1,@sqlCounter17) WITH NOWAIT
    RAISERROR('     sqlCounter18 took %d ms', 0, 1,@sqlCounter18) WITH NOWAIT
    RAISERROR('     sqlCounter19 took %d ms', 0, 1,@sqlCounter19) WITH NOWAIT

  END




  COMMIT TRANSACTION absConfig                   
  RETURN 0
GO
  EXECUTE alobLoadWrapperAbsConfig
SQL_SCRIPT

  echo "" >> "${sFilenameLog}"
  cat "${sFilenameSqlIn}" >> "${sFilenameLog}"

  ${sSqlCall} -i"${sFilenameSqlIn}" -o"${sFilenameSqlOut}"

  echo "Checking for errors in output" >> "${sFilenameLog}"

  check_sql_output "${sFilenameSqlOut}" 2>> "${sFilenameLog}"

  if [ $? -ne 0 ];then
    cat "${sFilenameSqlOut}" >> "${sFilenameLog}"
    return 1
  fi

} # close of bcp_in_1_4


# Get Database Server, User, and Password
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

# Get Install Directory
dirInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

# Default Values

bIn=0
bOut=0
bTrusted=0
sVersion=""

# check for tbsm ISQL utility
rc=`which tbsmsql.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find TBSMSQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi

# check for REGDMP utility
rc=`which REGDMP.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find REGDMP.EXE.  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure REGDMP.EXE is in the path." >&2
  exit 1
fi

# Get Database Server, User, Password from Registry
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

# Get codepage
sCodepage=$(dumplocaleinfo | awk '/Current ANSI Code Page/ { print $5}')

bVerbose=0
nCountParameter=$#
bExistIn=0
bTruncate=0
bTiming=0

# Process the command line
while getopts "i:o:c:f:S:U:P:d:vth?" c; do
  case $c in
  i) sFilenameIn=$OPTARG;bIn=1;;
  o) sFilenameOut=$OPTARG;bOut=1;;
  c) sCodepage=$OPTARG;;
  f) sVersion=$OPTARG;;
  S) db_server=$OPTARG;;
  U) db_user=$OPTARG;;
  P) db_password=$OPTARG;;
  t) bTruncate=1;;
  v) bVerbose=1;;
  d) bTiming=$OPTARG;;
  h|?)  Usage; exit 0;;
  *)  Usage >&2; exit 1;;
  esac
done
shift $OPTIND-1

sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
sFilenameLog=${sPathInstall}/Logs/$0.${sTimestamp}.log
sFilenameSqlIn=${sPathInstall}/Logs/$0.${sTimestamp}.sql.in
sFilenameSqlOut=${sPathInstall}/Logs/$0.${sTimestamp}.sql.out
sFilenameBcpOut=${sPathInstall}/Logs/$0.${sTimestamp}.bcp.out

if [ $bTiming -ne 0 ]; then;
  echo ${sTimestamp}
fi

touch "${sFilenameLog}"


# Force db_server to uppercase for conformity
db_server=$( echo $db_server | awk '// { print toupper( $0 )}' )

# Create sBbcpParameter # these will be the default parameters for invoking bcp
sBcpParameter="-V 65 -c -b 1000 -S$db_server -U$db_user -P$db_password -C$sCodepage";

# Create sSqlCall # these will be the default parameters for invoking tbsmsql
sSqlCall="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 -dObject";

if [ $bIn -eq 0 ] && [ $bOut -eq 0 ]; then
  echo "Must specify at least one of the following options:"
  echo "  -i<filename>"
  echo "  -o<filename>"
  Usage
  exit 1
fi

get_sVersionDefault # sVersionDefault=
if [ $? -ne 0 ]; then;
  error_display "${sFilenameLog}"
  if [ $? -ne 0 ]; then;
    echo "Error occurred, see log file ${sFilenameLog}"
  fi
  exit 3
fi


if [ $bTruncate -eq 1 ]; then;
  sTruncate=" TRUNCATE TABLE alobListCriteriaAlob              
 TRUNCATE TABLE alobListPattern                   
 TRUNCATE TABLE alobListCriteriaToPath            
 TRUNCATE TABLE alobListPatternList               
 TRUNCATE TABLE ev_watch_attr_filter
 TRUNCATE TABLE ev_watch_insert_filter             
 TRUNCATE TABLE dynamic_object_create_path_detail 
 TRUNCATE TABLE expanded_ev_watch_attr_filter    
 DELETE FROM dynamic_object_create_path 
 TRUNCATE TABLE alobListAttributeTemplate 
 TRUNCATE TABLE alobListCriteriaToFunctionGroup
 TRUNCATE TABLE alobListDefineAndUsePBTs
 TRUNCATE TABLE alobListDefineServicesAndLinkToEvUsers
 TRUNCATE TABLE alobListFunctionGroup
 TRUNCATE TABLE alobListFunctionGroupPrecedence
 TRUNCATE TABLE alobListPath
 TRUNCATE TABLE alobListSetSendStateChangeEvent
 TRUNCATE TABLE alobListSetThresholds
 TRUNCATE TABLE alobListSetWeights
 TRUNCATE TABLE alobListStaticFunctionGroup
 TRUNCATE TABLE alobListUserIDGroup
 TRUNCATE TABLE alobFunctionGroupTemplate
 TRUNCATE TABLE alobListAutoCopyEVService
 TRUNCATE TABLE alobCLLobTemplate
 TRUNCATE TABLE alobListBuildRelationshipView
 TRUNCATE TABLE alobListClassList
 TRUNCATE TABLE alobListExclusions
 TRUNCATE TABLE alobListLeafNodes
 TRUNCATE TABLE alobListSeedResources
 TRUNCATE TABLE alobListLinkTypes
 TRUNCATE TABLE alobListSeedResourcesResources
       
 GO"
fi

sVerbose=" @verbose = ${bTiming}"

if [ $bTiming -ne 0 ]; then;
  sTiming=" SET @verbose = ${bTiming}"
else
  sTiming=" SET @verbose = 0"            
fi

if [ $bIn -eq 1 ]; then
  get_sVersionIn                
fi

# cleanup sVersionOut
sVersionOut="$sVersion"
if [ "$sVersionOut" != "1.0" ] && [ "$sVersionOut" != "1.1" ] && [ "$sVersionOut" != "1.2" ] && [ "$sVersionOut" != "1.3" ] && [ "$sVersionOut" != "1.4" ]; then
  sVersionOut=""
fi

# determine sVersionOut for files.
if [ $bOut -eq 1 ] && [ "$sVersionOut" = "" ]; then
  sVersionOut=$sVersionDefault
fi

#echo "sVersionDefault: " $sVersionDefault
#echo "sVersionIn: " $sVersionIn
#echo "sVersionOut: " $sVersionOut

if [ "$sVersionIn" = "1.0" ]; then
  sFunctionIn="bcp_in_1_0"
fi

if [ "$sVersionIn" = "1.1" ]; then
  sFunctionIn="bcp_in_1_1"
fi

if [ "$sVersionIn" = "1.2" ]; then
  sFunctionIn="bcp_in_1_2"
fi

if [ "$sVersionIn" = "1.3" ]; then
  sFunctionIn="bcp_in_1_3"
fi

if [ "$sVersionIn" = "1.4" ]; then
  sFunctionIn="bcp_in_1_4"
fi

if [ "$sVersionOut" = "1.0" ]; then
  sFunctionOut="bcp_out_1_0"
fi

if [ "$sVersionOut" = "1.1" ]; then
  sFunctionOut="bcp_out_1_1"
fi

if [ "$sVersionOut" = "1.2" ]; then
  sFunctionOut="bcp_out_1_2"
fi

if [ "$sVersionOut" = "1.3" ]; then
  sFunctionOut="bcp_out_1_3"
fi

# No change to the output routines invoked between version 1.4 and 1.3
# Version 1.4 will re-use output function of version 1.3
if [ "$sVersionOut" = "1.4" ]; then
  sFunctionOut="bcp_out_1_3"
fi

if [ "$sVersionDefault" = "1.0" ]; then
  sFunctionBackup="bcp_out_1_0"
fi

if [ "$sVersionDefault" = "1.1" ]; then
  sFunctionBackup="bcp_out_1_1"
fi

if [ "$sVersionDefault" = "1.2" ]; then
  sFunctionBackup="bcp_out_1_2"
fi

if [ "$sVersionDefault" = "1.3" ]; then
  sFunctionBackup="bcp_out_1_3"
fi

if [ "$sVersionDefault" = "1.4" ]; then
  sFunctionBackup="bcp_out_1_3"
fi

#echo "sFunctionIn:  " "$sFunctionIn"
#echo "sFunctionOut: " $sFunctionOut
#echo "sFunctionBackup: " $sFunctionBackup
#exit
#
#
#

# Output config file if requested.
if [ $bOut -eq 1 ]; then;
  $sFunctionOut
  if [ $? -ne 0 ]; then;
    echo "Error occurred, see log file ${sFilenameLog}"
    exit 3
  fi
fi

# Input config file if requested, but first backup the configuration that is in the database.
if [ $bIn -eq 1 ]; then;
  if [ -e "${sFilenameIn}" ]; then;
    bExistIn=1
  else
    echo "Error: -i file does not exist."
    Usage
    exit 1
  fi
  sPathInstall=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's/\\/\//g' | sed 's/\/$//g' )
  sFilenameOut=${sPathInstall}/Logs/abs.backup.$(date +'%Y''%m''%d'_'%H''%M''%S').txt
  if [ $bTiming -ne 0 ]; then;
    sTimestamp2=$(date +'%Y''%m''%d'_'%H''%M''%S')
    echo ${sTimestamp2}
  fi
  if [ $bTruncate -eq 0 ]; then;
    $sFunctionBackup
  fi
  if [ $bTiming -ne 0 ]; then;
    sTimestamp2=$(date +'%Y''%m''%d'_'%H''%M''%S')
    echo ${sTimestamp2}
  fi
  if [ $? -ne 0 ]; then;
    error_display "${sFilenameLog}"
    if [ $? -ne 0 ]; then;
      echo "Error occurred, see log file ${sFilenameLog}"
    fi
    exit 3
  fi
  $sFunctionIn
  if [ $? -ne 0 ]; then;
    error_display "${sFilenameLog}"
    if [ $? -ne 0 ]; then;
      echo "Error occurred, see log file ${sFilenameLog}"
    fi
    exit 3
  fi
fi

# User did not request output give help
if [ nCountParameter -eq 0 ]; then;
  Usage
  exit 1
fi

if [ $bVerbose -eq 1 ]; then;
  cat "${sFilenameLog}"
fi

if [ $bTiming -ne 0 ]; then;
  sTimestamp=$(date +'%Y''%m''%d'_'%H''%M''%S')
  echo ${sTimestamp}
fi

echo "Processing complete. " >&2
