-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/_listxrcvolumepairs.sql, tbsm.sql, tbsm_base31e
-- SID: 1.4.1.3
-- Date Modified: 08/08/15 07:42:58
--
-- _listxrcvolumepairs.sql
--
-- Module Description
--   Create _ListXRCVolumePairs method

use Object
---------------------------------------------------------------------------------------------------
IF EXISTS(SELECT * FROM sysobjects WHERE name = '_ListXRCVolumePairs' AND type = 'P')
  DROP PROCEDURE _ListXRCVolumePairs
GO

PRINT 'Creating _ListXRCVolumePairs'
GO

-- File: archive/db/sql/procs/_listxrcvolumepairs.sql, tbsm.sql, tbsm_base31e
-- SID: 1.4.1.3
-- Date Modified: 08/08/15 07:42:58
/*
Lists the disk devices under the calling object
 */
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
CREATE PROCEDURE _ListXRCVolumePairs
  @cid              ClassID,
  @id               ObjID,
  @LocationPattern  NVARCHAR(255) = NULL,
  @NamePattern      NVARCHAR(255) = NULL,
  @MinAlert         INT = 0,
  @MinPriority      INT = 0,
  @OpenNotes        INT = 2
AS
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _ListXRCVolumePairs
  ELSE
    SAVE TRAN _ListXRCVolumePairs

  DECLARE @CPNetName NVARCHAR(17)

  IF @cid = 'LOB'
    SELECT @cid = dst_cid, @id = dst_id FROM link
    WHERE link_type = 'LOBL' AND src_cid = @cid AND src_id = @id AND deleted = 0

  -- if invoked on a system data mover, find the xrc folder who shares the same enterprise

  IF @cid = 'SDM' BEGIN
    SELECT @cid = c5.dst_cid, @id = c5.dst_id, @CPNetName = os._SystemId
      FROM containment c0
      JOIN containment c1 ON c1.src_cid = 'LPAR' AND c1.dst_cid = c0.src_cid AND c1.dst_id = c0.src_id AND c1.deleted = 0
      JOIN containment c2 ON c2.src_cid = 'MACH' AND c2.dst_cid = c1.src_cid AND c2.dst_id = c1.src_id AND c2.deleted = 0
      JOIN containment c3 ON c3.src_cid = 'COMP' AND c3.dst_cid = c2.src_cid AND c3.dst_id = c2.src_id AND c3.deleted = 0
      JOIN containment c4 ON c4.src_cid = 'ENT'  AND c4.dst_cid = c3.src_cid AND c4.dst_id = c3.src_id AND c4.deleted = 0
      JOIN containment c5 ON c5.dst_cid = 'XRCF' AND c5.src_cid = c4.src_cid AND c5.src_id = c4.src_id AND c5.deleted = 0
      JOIN OS_V os ON os.id = c0.src_id
    WHERE c0.dst_cid = @cid AND c0.dst_id = @id AND c0.deleted = 0 AND c0.src_cid = 'OS'
  END

  -- CREATE TABLE of all appropriate operating systems under the calling object
  CREATE TABLE #descendent
  (
    cid   ClassID,
    id    ObjID,
    gen   INT
  )
  EXEC asisp_descendents @cid, @id, 'IN (''XRCP'')'

  -- SET up additional filtering criteria based ON parameters
  DECLARE @NameExpr NVARCHAR(255), @AlertExpr NVARCHAR(40), @PriorityExpr NVARCHAR(40), @NotesExpr NVARCHAR(40)
  DECLARE @NetNameExpr NVARCHAR(64)

  IF @NamePattern IS NOT NULL
    SET @NameExpr = ' AND name LIKE ''' + @NamePattern + ''''
  ELSE
    SET @NameExpr = ''

  IF @MinAlert IS NOT NULL AND @MinAlert <> 0
    SET @AlertExpr = ' AND _AlertStateID >= ' + CONVERT(NVARCHAR(9), @MinAlert)
  ELSE
    SET @AlertExpr = ''

  IF @MinPriority IS NOT NULL AND @MinPriority <> 0
    SET @PriorityExpr = ' AND _PriorityID <= ' + CONVERT(NVARCHAR(9), @MinPriority) -- yes, priorities work IN reverse!
  ELSE
    SET @PriorityExpr = ''

  IF @OpenNotes IS NOT NULL AND @OpenNotes <> 2
    SET @NotesExpr = ' AND _HasOwnershipNote = ' + CONVERT(NCHAR(1), @OpenNotes)
  ELSE
    SET @NotesExpr = ''

  IF @CPNetName IS NOT NULL
    SET @NetNameExpr = ' AND _CPNetName = ''' + @CPNetName + ''''
  ELSE
    SET @NetNameExpr = ''

  -- Populate TABLE with all disk devices under the operating systems that match the criteria
  CREATE TABLE #obj
  (
    src_cid            ClassID,
    src_id             ObjID,
    cid                ClassID,
    id                 ObjID,
    Location           NVARCHAR(255) NULL,
    name               ObjName NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        NVARCHAR(255) NULL
  )
  EXEC("
  INSERT #obj
    SELECT
      c.src_cid,
      c.src_id,
      d.cid,
      d.id,
      obj.name,
      obj.name,
      obj._AlertStateID,
      obj._PriorityID,
      obj._CurrentStateID,
      obj._DesiredStateID,
      obj._HasOwnershipNote,
      obj._Desc
    FROM #descendent AS d
      JOIN XRCVolumePair_V AS obj ON obj.id = d.id
      JOIN containment AS c ON c.dst_cid = d.cid AND c.dst_id = d.id AND c.deleted = 0
    WHERE 1=1"
      + @NameExpr
      + @AlertExpr
      + @PriorityExpr
      + @NotesExpr
      + @NetNameExpr
  )

  DECLARE @rc INT, @src_cid ClassID, @loop INT, @cname NVARCHAR(24)
  SELECT @rc = COUNT(*) FROM #obj
  SET @cname = ''
  SET @loop = 0
  WHILE @rc > 0 AND @loop < 10
  BEGIN
    DECLARE srcc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, cname FROM #obj JOIN obj_class AS c ON c.cid = src_cid
    OPEN srcc
    FETCH NEXT FROM srcc INTO @src_cid, @cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC("
      UPDATE obj SET src_cid = c.src_cid, src_id = c.src_id, Location = parent.name + '/' + Location
      FROM #obj AS obj
      JOIN containment AS c ON c.dst_cid = obj.src_cid AND c.dst_id = obj.src_id
      JOIN " + @cname + "_C AS parent ON parent.id = obj.src_id AND obj.src_cid = c.dst_cid
      WHERE obj.src_cid = '" + @src_cid + "' AND obj.src_cid <> 'BUSC'
      ")
      FETCH NEXT FROM srcc INTO @src_cid, @cname
    END
    DEALLOCATE srcc
    SELECT @rc = COUNT(*) FROM #obj WHERE src_cid <> 'ENT'
    SET @loop = @loop + 1
  END

  -- Filter out paths that don't match parameter, IF specified
  IF @LocationPattern IS NOT NULL
    DELETE #obj WHERE Location NOT LIKE @LocationPattern

  -- obtain the localized strings
  DECLARE @local_string NVARCHAR(255)
  CREATE TABLE #localized_strings
  (
    key_name         sysname,
    english_string   VARCHAR(128),
    local_string     NVARCHAR(255) NULL DEFAULT NULL
  )
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Location'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Type'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'name'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Description'
  EXEC asisp_getLocalizedString NULL, NULL, @local_string OUTPUT, '#localized_strings'

  -- store the localized strings in variables
  DECLARE @location_heading NVARCHAR(255), @type_heading NVARCHAR(255), @name_heading NVARCHAR(255), @description_heading NVARCHAR(255)
  SELECT @location_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Location'
  SELECT @type_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Type'
  SELECT @name_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='name'
  SELECT @description_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Description'

  DROP TABLE #localized_strings

  SET NOCOUNT OFF
  -- RETURN results
  EXEC('
  SELECT
    obj._AlertStateID,
    obj._HasOwnershipNote,
    obj.Location AS N'''+@location_heading+''',
    c.label AS N'''+@type_heading+''',
    name AS N'''+@name_heading+''',
    obj._PriorityID,
    obj._CurrentStateID,
    obj._DesiredStateID,
    Description AS N'''+@description_heading+''',
    obj.cid,
    obj.id
  FROM
    #obj AS obj
      JOIN obj_class AS c ON c.cid = obj.cid
  ORDER BY
    obj.Location
  ')

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN _ListXRCVolumePairs
GO

PRINT 'Done.'
GO
