-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2008 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp1.sql, tbsm.sql.porting, tbsm_base31e
-- SID: 1.12.1.12
-- Date Modified: 08/03/03 05:27:29
--
-- Module Description
--   Defines the alobLoadConfigBcp1 stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- alobLoadConfigBcp1
PRINT 'Creating procedure alobLoadConfigBcp1'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'alobLoadConfigBcp1' AND type = 'P')
  DROP PROCEDURE alobLoadConfigBcp1
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/alobloadconfigbcp1.sql, tbsm.sql.porting, tbsm_base31e
-- SID: 1.12.1.12
-- Date Modified: 08/03/03 05:27:29
CREATE PROCEDURE alobLoadConfigBcp1
  @sFilename      NVARCHAR(256),
  @sCodepage      NVARCHAR(16),
  @verbose        TINYINT = 0
AS

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN alobLoadConfigBcp1
  ELSE
    SAVE TRAN alobLoadConfigBcp1

  -- Standard Announcement 
  RAISERROR( 'alobLoadConfigBcp1 @sFilename=''%s'', @sCodepage=''%s''', 0, 1, @sFilename, @sCodepage )

  -- Declare variables
  DECLARE @nError INTEGER

  -- Bcp in the file since BULK INSERT is problematic
  EXEC @nError = alobLoadConfigFile @sFilename, 'alobFilePattern', @sCodepage, @verbose = @verbose

  IF @nError <> 0
  BEGIN
    RAISERROR('GTMAB1002E Unable to load configuration file %s', 3, 1, @sFilename)
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END
  ELSE
    PRINT 'Loaded file ' + @sFilename

  -- HACK
  -- SELECT * FROM alobFilePattern

  ---------------------------------------------------------------------------------------------------
  -- Integrity check

  -- Check that all the columns that should have data do have data.
  -- iPattern
  IF EXISTS ( SELECT * FROM alobFilePattern WHERE iPattern IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Pattern', 'Pattern' )
    SELECT * FROM alobFilePattern WHERE iPattern IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  -- iClass
  IF EXISTS ( SELECT * FROM alobFilePattern WHERE iClass IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Class', 'Pattern')
    SELECT * FROM alobFilePattern WHERE iClass IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  -- sNameAttribute
  IF EXISTS ( SELECT * FROM alobFilePattern WHERE sNameAttribute IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Attribute', 'Pattern')
    SELECT * FROM alobFilePattern WHERE sNameAttribute IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  -- sWhen
  IF EXISTS ( SELECT * FROM alobFilePattern WHERE sWhen IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'When', 'Pattern' )
    SELECT * FROM alobFilePattern WHERE sWhen IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  -- sOperator
  IF EXISTS ( SELECT * FROM alobFilePattern WHERE sOperator IS NULL )
  BEGIN
    RAISERROR('GTMAB1021E There is a NULL value in the %s column in the %s section.', 3, 1, 'Operator', 'Pattern' )
    SELECT * FROM alobFilePattern WHERE sOperator IS NULL
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END  

  -- Check the for invalid class or rejected class in patterns 
  CREATE TABLE #check_class
  (
    iClass	   ClassID                 PRIMARY KEY ( iClass ),
    bEnabled       INTEGER           NULL
  )

  -- Normalize table to be backwards compatible.  Allow _ for obj_attr attributes.
  UPDATE alobFilePattern
  SET sNameAttribute = SUBSTRING ( sNameAttribute, 2, LEN(sNameAttribute) - 1)
  WHERE sNameAttribute LIKE '[_][^_]%' AND sNameAttribute NOT IN ( '_id', '_name', '_revision', '_nchild', 
                            '_description', '_ctime', '_mtime', '_userid', '_deleted' ) -- universal class attributes


  INSERT INTO #check_class (   iClass, bEnabled )
  SELECT DISTINCT            d.iClass,     NULL
  FROM alobFilePattern AS d

  EXECUTE alobCheckClassEnabled '#check_class'

  DECLARE @iClass   ClassID 
  DECLARE @iPattern INTEGER
  SELECT  @iClass = NULL
  SET ROWCOUNT 1
  SELECT @iClass = cc.iClass, @iPattern = d.iPattern
  FROM #check_class     AS cc
  INNER JOIN alobFilePattern AS d  ON d.iClass = cc.iClass
  WHERE 
           cc.bEnabled = 0 
  SET ROWCOUNT 0

  IF ( @iClass IS NOT NULL )
  BEGIN
    RAISERROR('GTMAB1003E The class %s in pattern %d in the Pattern section is not allowed.', 3, 1, @iClass, @iPattern )
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END  

  -- Check for unacceptable attributes
  CREATE TABLE #check_attribute
  (
    iClass	   ClassID,
    sNameAttribute NVARCHAR(256) NOT NULL,
    bEnabled       INTEGER           NULL
  )
  INSERT INTO #check_attribute (  iClass,   sNameAttribute, bEnabled )
  SELECT DISTINCT               d.iClass, d.sNameAttribute,     NULL
  FROM alobFilePattern AS d

  UPDATE #check_attribute 
  SET bEnabled = 1
  WHERE sNameAttribute IN (
    SELECT DISTINCT a.name
    FROM obj_attr AS a
      JOIN isa_chain AS i ON i.base_cid = a.cid
      JOIN attr_type AS t ON t.atype = a.atype,
      #check_attribute AS c
    WHERE
      i.cid = c.iClass
      AND t.aname IN ( 'ASIString', 'BYTE', 'SHORT', 'USHORT', 'LONG', 'ULONG', 'FLOAT', 'DOUBLE', 'ASISystemTime' )
      AND a.flag_rule_field = 1
      AND NOT EXISTS
      (
        SELECT 1
        FROM obj_attr AS ab 
          JOIN isa_chain AS ib ON ib.base_cid = ab.cid
        WHERE
          ib.cid = i.cid
          AND ab.name = a.name
          AND ib.generation < i.generation
          AND ab.flag_rule_field = 0
      )
   )
   OR sNameAttribute IN ( SELECT sNameAttribute 
                          FROM alobListAttributeEnabled AS ae
                          WHERE  ae.bEnabled = 1)
                          

  DECLARE @sNameAttribute   NVARCHAR(256)  
  SELECT  @sNameAttribute = NULL
  SET ROWCOUNT 1
  SELECT @sNameAttribute = ca.sNameAttribute, @iPattern = d.iPattern
  FROM #check_attribute AS ca
  INNER JOIN alobFilePattern AS  d ON d.iClass = ca.iClass AND d.sNameAttribute = ca.sNameAttribute
  WHERE
           ca.bEnabled IS NULL
  SET ROWCOUNT 0

  IF ( @sNameAttribute IS NOT NULL )
  BEGIN
    RAISERROR('GTMAB1004E The attribute %s in pattern %d in the Pattern section is not allowed.', 11, 1, @sNameAttribute, @iPattern )
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END  

  -- Check for unacceptable When attribute
  DECLARE @sWhen NVARCHAR( 20 ) 
  SELECT  @sWhen = NULL
  SET ROWCOUNT 1
  SELECT @sWhen = d.sWhen, @iPattern = d.iPattern
  FROM alobFilePattern AS d
  WHERE
           d.sWhen NOT IN ( 'Previous', 'Current' ) 
  SET ROWCOUNT 0
  IF ( @sWhen IS NOT NULL )
  BEGIN
    RAISERROR('GTMAB1005E The When field value of %s in pattern %d in the Pattern section is not allowed.', 3, 1, @sWhen, @iPattern )
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  -- Check for unacceptable Operator
  DECLARE @sOperator NVARCHAR(40)
  SET ROWCOUNT 1
  SELECT @sOperator = d.sOperator, @iPattern = d.iPattern, @iClass = d.iClass, @sNameAttribute = d.sNameAttribute
  FROM alobFilePattern AS d
  WHERE
        d.sOperator NOT IN ( 'LIKE', 'NOT LIKE', '=', '<>', '>', '<', '>=', '<=', 'BETWEEN', 'NOT BETWEEN', 'IS', 'LIKE LIST', 'NOT LIKE LIST', 'EGREP')
        OR
        (d.sOperator = 'IS'
         AND d.sOperand1 NOT IN ('NULL', 'NOT NULL'))
  SET ROWCOUNT 0
  IF (@sOperator IS NOT NULL)
  BEGIN
    RAISERROR('GTMAB1044E Invalid Operator %s for Pattern %d, Class %s, and Attribute %s in the Pattern section.', 3, 1, @sOperator, @iPattern, @iClass, @sNameAttribute )
    ROLLBACK TRANSACTION alobLoadConfigBcp1
    RETURN -30000
  END

  ---------------------------------------------------------------------------------------------------
  -- Modify the database

  -- Update existing records
  UPDATE alobListPattern
  SET
    iClass            = d.iClass, 
    sNameAttribute    = d.sNameAttribute, 
    iWhen             = CASE d.sWhen 
                          WHEN 'Previous' THEN 0
                          WHEN 'Current'  THEN 1
                          ELSE 1
                        END,
    sOperator         = d.sOperator,
    sOperand1         = d.sOperand1, 
    sOperand2         = d.sOperand2, 
    bEnabled          = 1,
    bDeleted          = 0
  FROM  alobListPattern AS p
  INNER JOIN alobFilePattern AS d ON d.iPattern = p.iPattern

  -- Insert new records
  INSERT INTO alobListPattern (   iPattern,   iClass,   sNameAttribute,   iWhen, sOperator,   sOperand1,   sOperand2, 
                                  bEnabled,   bDeleted ) 
  SELECT                        
    d.iPattern, 
    d.iClass, 
    d.sNameAttribute, 
      CASE d.sWhen 
        WHEN 'Previous' THEN 0
        WHEN 'Current'  THEN 1
        ELSE 1
      END,
    d.sOperator, 
    d.sOperand1, 
    d.sOperand2, 
    1,
    0
  FROM alobListPattern AS p
  RIGHT OUTER JOIN alobFilePattern AS d ON d.iPattern = p.iPattern
  WHERE 
           p.iPattern IS NULL


  -- Figure out if the attributes are extended or not 
  CREATE TABLE #attribute
  (
    iClass                  ClassID,
    sNameAttribute          NVARCHAR(256) NOT NULL,
    bExtended               INTEGER       NULL,
    bModel                  INTEGER       NULL,
    sTypeTarget             NVARCHAR( 32) NULL
  )

  INSERT INTO #attribute (   iClass,   sNameAttribute, bExtended, bModel )
  SELECT DISTINCT          p.iClass, p.sNameAttribute,      NULL,   NULL
  FROM alobListPattern AS p

  EXECUTE alobGetAttributeType '#attribute'

  DELETE FROM ev_watch_attr_filter_temp -- else attributes build up over time

  INSERT INTO  ev_watch_attr_filter_temp (  cid,  attribute_name, extended, active, salience, 
                                            val_inserted_pat, val_deleted_pat, enqueue )
  SELECT 
    d.iClass,
    d.sNameAttribute,
    a.bExtended,
    1,
    0,
    '%',
    '%',
    1
  FROM       alobFilePattern AS d
  INNER JOIN #attribute AS a ON a.iClass         = d.iClass
                            AND a.sNameAttribute = d.sNameAttribute

  ---------------------------------------------------------------------------------------------------
  -- Determine the data type of the attribute; used in alobEvaluatePattern
  UPDATE #attribute
  SET
    sTypeTarget = 'NVARCHAR(256)'
  FROM #attribute AS a
  WHERE
           a.bModel = 0


  DECLARE @sNameTable         ObjName
  DECLARE @bExtended          INTEGER
  DECLARE @sTypeTarget        NVARCHAR( 32 )
  DECLARE @sNameAttributeFull NVARCHAR( 257 )

  DECLARE cursorTarget CURSOR FOR 
    SELECT DISTINCT iClass, sNameAttribute, bExtended, cname + '_V'
    FROM       #attribute AS a
    INNER JOIN obj_class  AS oc ON oc.cid = a.iClass
    WHERE 
             a.sTypeTarget IS NULL
  OPEN cursorTarget
  FETCH NEXT FROM cursorTarget INTO @iClass, @sNameAttribute, @bExtended, @sNameTable
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    SET @sNameAttributeFull = CASE @bExtended 
                                WHEN 1 THEN '_' + @sNameAttribute
                                ELSE @sNameAttribute
                              END

    SELECT @sTypeTarget = UPPER( st.name ) FROM systypes AS st
    INNER JOIN syscolumns AS sc ON sc.xtype = st.xusertype
    INNER JOIN sysobjects AS so ON so.id    = sc.id
    WHERE 
             sc.name = @sNameAttributeFull
         AND so.name = @sNameTable

    UPDATE #attribute
    SET
      sTypeTarget = @sTypeTarget 
    FROM #attribute AS a
    WHERE  
             a.iClass         = @iClass
         AND a.sNameAttribute = @sNameAttribute

    -- hack
    PRINT 'SELECT * FROM #attribute'
    SELECT * FROM #attribute

    FETCH NEXT FROM cursorTarget INTO @iClass, @sNameAttribute, @bExtended, @sNameTable
  END
  CLOSE cursorTarget
  DEALLOCATE cursorTarget

/*
  SELECT UPPER( st.name ) FROM systypes AS st
  INNER JOIN syscolumns AS sc ON sc.xtype = st.xusertype
  INNER JOIN sysobjects AS so ON so.id    = sc.id
  WHERE 
           sc.name = '_IPAddress'
       AND so.name = 'OS_V'
*/
  UPDATE #attribute
  SET
    sTypeTarget = a.sTypeTarget + '( 512 )'
  FROM       #attribute AS a
  WHERE 
           a.sTypeTarget IN ( 'NVARCHAR', 'NCHAR', 'VARCHAR', 'CHAR', 'BINARY', 'VARBINARY' )

  -- Update sTypeTarget in alobListPattern
  UPDATE alobListPattern
  SET
    sTypeTarget       = a.sTypeTarget
  FROM  alobListPattern AS p
  INNER JOIN #attribute AS a ON a.iClass         = p.iClass 
                            AND a.sNameAttribute = p.sNameAttribute
  



  -- HACK
  -- SELECT * FROM alobListPattern

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN alobLoadConfigBcp1
GO
