-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_create_dynamic_xrcp.sql, tbsm_base, tbsm_base31e
-- SID: 1.6.4.2
-- Date Modified: 08/08/20 17:06:57
--
-- asisp_create_dynamic_xrcp.sql
--
-- Module Description
--   Creates asisp_create_dynamic_XRCP

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_create_dynamic_XRCP' AND type = 'P')
  DROP PROCEDURE asisp_create_dynamic_XRCP
GO

PRINT 'Creating asisp_create_dynamic_XRCP'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_create_dynamic_xrcp.sql, tbsm_base, tbsm_base31e
-- SID: 1.6.4.2
-- Date Modified: 08/08/20 17:06:57
CREATE PROCEDURE asisp_create_dynamic_XRCP
  @_msgid     NVARCHAR(255), 
  @_msgtext   NVARCHAR(255), 
  @_SourceCID ClassID, 
  @_SourceID  ObjID, 
  @_hostdate  DATETIME,
  @_seq_no    NVARCHAR(50),
  @_xrcsName  NVARCHAR(255),
  @_xscsName  NVARCHAR(255),
  @_xrcpName  NVARCHAR(255),
  @verbose TINYINT=0
AS
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_create_dynamic_XRCP
  ELSE
    SAVE TRAN asisp_create_dynamic_XRCP

  DECLARE @now DATETIME
  SET @now = CURRENT_TIMESTAMP

  DECLARE @xrcfID ObjID
  DECLARE @xrcsID ObjID
  DECLARE @xscsID ObjID
  DECLARE @xrcpID ObjID

  DECLARE @CPNetName         NVARCHAR(17)
  DECLARE @MasterSessionName NVARCHAR(8)
  DECLARE @SystemName        NVARCHAR(8)
  DECLARE @PrimaryVolume     NVARCHAR(6)
  DECLARE @SecondaryVolume   NVARCHAR(6)
  DECLARE @primarySCID       NVARCHAR(2) 
  DECLARE @primarySCSN       NVARCHAR(2) 
  DECLARE @pos               INT

  RAISERROR('asisp_create_dynamic_XRCP called with xrcsName=%s, xscsName=%s, xrcpName=%s, msgid=%s, msgtext=%s, SourceCID=%s, SourceID=%d', 
            0, 1, @_xrcsName, @_xscsName, @_xrcpName, @_msgid, @_msgtext, @_SourceCID, @_SourceID) WITH NOWAIT

  /* find the XRC session that the volume pair belongs to */

  SELECT @xrcfID = ent_c2.dst_id
    FROM containment lpar_c
    JOIN containment mach_c ON mach_c.src_cid = 'MACH' AND mach_c.dst_cid = 'LPAR' AND mach_c.dst_id = lpar_c.src_id AND mach_c.deleted = 0
    JOIN containment comp_c ON comp_c.src_cid = 'COMP' AND comp_c.dst_cid = 'MACH' AND comp_c.dst_id = mach_c.src_id AND comp_c.deleted = 0
    JOIN containment ent_c1 ON ent_c1.src_cid = 'ENT' AND ent_c1.dst_cid = 'COMP' AND ent_c1.dst_id = comp_c.src_id AND ent_c1.deleted = 0
    JOIN containment ent_c2 ON ent_c2.src_cid = 'ENT' AND ent_c2.src_id = ent_c1.src_id AND ent_c2.dst_cid = 'XRCF' AND ent_c2.deleted = 0
    WHERE lpar_c.src_cid = 'LPAR' AND lpar_c.dst_cid = 'OS' AND lpar_c.dst_id = @_SourceID AND lpar_c.deleted = 0

  SELECT @xrcsID = v.id, @CPNetName = v._CPNetName, @MasterSessionName = v._MasterSessionName, @SystemName = v._SystemName
    FROM containment xrcf_c
    JOIN containment xrcm_c ON xrcm_c.src_cid = 'XRCM' AND xrcm_c.src_id = xrcf_c.dst_id AND xrcm_c.dst_cid = 'XRCS' AND xrcm_c.deleted = 0
    JOIN XRCSession_V v ON v.id = xrcm_c.dst_id AND v.name = @_xrcsName
    WHERE xrcf_c.src_cid = 'XRCF' AND xrcf_c.src_id = @xrcfID AND xrcf_c.dst_cid = 'XRCM' AND xrcf_c.deleted = 0
  IF @xrcsID IS NULL
    SELECT @xrcsID = v.id, @CPNetName = v._CPNetName, @MasterSessionName = v._MasterSessionName, @SystemName = v._SystemName
      FROM containment xrcf_c
      JOIN XRCSession_V v ON v.id = xrcf_c.dst_id AND v.name = @_xrcsName
      WHERE xrcf_c.src_cid = 'XRCF' AND xrcf_c.src_id = @xrcfID AND xrcf_c.dst_cid = 'XRCS' AND xrcf_c.deleted = 0

  IF @xrcsID IS NOT NULL
  BEGIN
    RAISERROR('xrcs id = %d', 0, 1, @xrcsID) WITH NOWAIT

    SELECT @xscsID = c.dst_id
      FROM containment c
      JOIN XRCControlSession_V xscs ON xscs.id = c.dst_id AND xscs.name = @_xscsName
      WHERE c.src_cid = 'XRCS' AND c.src_id = @xrcsID AND c.dst_cid = 'XSCS' AND c.deleted = 0

    IF @xscsID IS NOT NULL
    BEGIN /* we need to read basic control session data */
      RAISERROR('Reading control session %s', 0, 1, @_xscsName) WITH NOWAIT
      SELECT @primarySCID = _PrimarySCID from XRCVolumePair_C where _PrimarySCSN = @_xscsName and deleted = 0
    END

    SET @pos = PATINDEX('%,%', @_xrcpName)
    IF (@pos > 1) AND (@pos < LEN(@_xrcpName)) BEGIN
      SET @PrimaryVolume   = SUBSTRING(@_xrcpName, 1, @pos-1)
      SET @SecondaryVolume = SUBSTRING(@_xrcpName, @pos+1, LEN(@_xrcpName)-@pos)
    END

    RAISERROR('pair name = %s, primary volume = %s, secondary volume = %s', 0, 1, @_xrcpName, @PrimaryVolume, @SecondaryVolume) WITH NOWAIT

    IF (@xscsID IS NOT NULL) AND (@xscsID > 0) AND (@PrimaryVolume IS NOT NULL) AND (@primarySCID IS NOT NULL)
    BEGIN
      RAISERROR('xscs id = %d', 0, 1, @xscsID) WITH NOWAIT

      SELECT @xrcpID = xrcp.id
        FROM containment c 
        JOIN XRCVolumePair_V xrcp ON xrcp.id = c.dst_id AND xrcp.name = @_xrcpName
        WHERE c.src_cid = 'XSCS' AND c.src_id = @xscsID AND c.dst_cid = 'XRCP' AND c.deleted = 0

      IF @xrcpID IS NULL
      BEGIN
        RAISERROR('Creating volume pair %s', 0, 1, @_xrcpName) WITH NOWAIT

        EXEC @xrcpID = asisp_createXRCP @_xrcpName, 'XSCS', @xscsID, 
                                        @CPNetName=@CPNetName,
                                        @Discovered=@now,
                                        @HostDiscovery=@_hostdate,
                                        @LastDiscovered=@now,
                                        @PrimaryVolume=@PrimaryVolume,
                                        @SecondaryVolume=@SecondaryVolume,
                                        @MasterSessionName=@MasterSessionName,
                                        @SessionName=@_xrcsName,
                                        @SynchStatus='PND',         /* initial status is pending */
                                        @SystemName=@SystemName,
                                        @PrimarySCID = @primarySCID,
                                        @PrimarySCSN = @_xscsName

        IF @xrcpID IS NOT NULL AND @xrcpID > 0
          EXEC _UpdateXRCSPairCounts 'XRCS', @xrcsID, @verbose=@verbose
      END
      RAISERROR('xrcp id = %d', 0, 1, @xrcpID) WITH NOWAIT
    END
    ELSE
    BEGIN
      RAISERROR('Could not find/create XRC control session %s', 3, 1, @_xscsName) WITH NOWAIT
    END
  END
  ELSE
  BEGIN
    RAISERROR('Could not find XRC session %s', 3, 1, @_xrcsName) WITH NOWAIT
  END

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN asisp_create_dynamic_XRCP

GO

PRINT 'Done'
GO
