-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldexceptions.sql, tbsm.sql, tbsm_base31e
-- SID: 1.4.6.19
-- Date Modified: 08/03/24 11:52:39
--
-- Module Description:
--   Defines the asisp_deleteoldexceptions stored procedure
--
--   Summary of changes due to APAR OA16939
--      - Move all declarations of variables/tables to the top 
--            of the file. This cuts down on stored procedures recompiles
--      - Put logic in EXEC asisp_update_excp_counts "_AlertState", "1"
--            directly in line and make it more direct (no dynamic sql)
--      - Remove cleanup to AlertStateHistory table and place in another procedure (asisp_deleteoldalertstatehistory)
--            This proc calls the new cleanup proc.  Eventually we could make that scheduled separately
--      - 

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_deleteoldexceptions
PRINT 'Creating procedure asisp_deleteoldexceptions'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_deleteoldexceptions' AND type = 'P')
  DROP PROCEDURE asisp_deleteoldexceptions
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldexceptions.sql, tbsm.sql, tbsm_base31e
-- SID: 1.4.6.19
-- Date Modified: 08/03/24 11:52:39
-- Deletes rows for all exceptions that are marked for deletion and older than a given number of days
CREATE PROCEDURE asisp_deleteoldexceptions
  @daystokeep INT = 7,
  @daysToKeepAlert INT = 548,
  @limit INT = 10000,
  @maxRowsToDelete INT = 50000,
  @options INT =0,            --- controls what steps in this procedure are actually taken
                              --- IF @options=0 then this proc does everything
                              --- IF @options=1 then clearing events with priority=5 (Ignore) occurs
                              --- IF @options=2 do the guts of the proc, clean up Exception_C table
                              --- IF @options=3 then clear jcl errors
                              --- IF @options=4 AlertStateHistory Cleanup   
                              --- IF @options=5 then clearing events with priority=0 (Unknown) and 
                              --- AlertSate = 0(Unknown) or 1(Green) occurs
  @verbose INT = 0
AS

  SET NOCOUNT ON
  -- Standard transaction logic
  DECLARE @tcount INT
  DECLARE @oldest DATETIME
  SET @oldest = DATEADD(day, -@daystokeep, CURRENT_TIMESTAMP)
  DECLARE @num_exch INT, @num_excp INT, @num_bch INT, @num_bcjh INT, @err INT, @now NVARCHAR( 30 ), @rc INT
  DECLARE @num_ignore INT, @num_unknown INT, @steprc INT
  DECLARE @dtime DATETIME, @dtime_twh DATETIME, @dtime_str NVARCHAR( 30 ), @total_rows INT
  DECLARE @deleteUpTo int , @maxInd int
  DECLARE @cmd nvarchar(4000)


  CREATE TABLE #tempexcp    
  (    
    tid          INT NOT NULL,    
    torig_count  INT NULL,  
    texcp_count  INT DEFAULT 0  
  )    
  CREATE INDEX tid_#tempexcp ON #tempexcp (tid)    


  CREATE TABLE #tempehist  
  (  
    tid         INT NOT NULL,  
    texcp_count INT NOT NULL  
  )  
  CREATE INDEX tid_#tempehist ON #tempehist(tid)  


  IF @verbose >= 1
  BEGIN
     SELECT @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
     RAISERROR('%s: asisp_deleteoldexceptions: Starting', 0, 1, @now)  WITH NOWAIT
  END



  IF @options=0 or @options=1 BEGIN 
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_deleteoldexceptions_1
  ELSE
    SAVE TRANSACTION asisp_deleteoldexceptions_1


  /* 
     first, send a clearing "OK" for all ignore priority exceptions that fit the time window - there's no way
     for the customer to get rid of them from the console so we'll take care of them here.
  */

  -- limit the number of exceptions we'll clear to 250 at a time
  INSERT INTO StagedEXCP (ctime, parent_cid, parent_id, TIME, EXCP_NAME, EXCP_CD, nEXCH, AlertState, Priority, SEQ_NO)
    SELECT TOP 250 CURRENT_TIMESTAMP, _ParentCID, _ParentID, CURRENT_TIMESTAMP, name, 'SC - Cleared by asisp_deleteoldexceptions', 1, _AlertState, _Priority, _SEQ_NO
    FROM Exception_V e 
    WHERE e.deleted = 0 AND e._Priority=5 AND e._DATE < @oldest 
    AND NOT EXISTS (SELECT 1 -- don't clear an exception that already has another instance of the exception queued for processing
                    FROM StagedEXCP se 
                    WHERE se.parent_cid = e._ParentCID 
                    AND se.parent_id    = e._ParentID
                    AND se.EXCP_NAME    = e.name
                    AND se.AlertState   = e._AlertState
                    AND se.Priority     = e._Priority)
    ORDER BY _DATE -- by sorting by date, we take care of the oldest exceptions first
  SELECT @num_ignore = @@ROWCOUNT, @err = @@ERROR
  
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to schedule ignore exceptions for clearing. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions_1
    RETURN -30000
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_deleteoldexceptions_1

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Scheduled %d ignore exceptions for clearing', 0, 1, @now, @num_ignore) WITH NOWAIT
  END
  END --- of option 0 or option 1 doing send a clearing "OK"


  IF @options=0 or @options=5
  BEGIN 
    SELECT @tcount = @@TRANCOUNT
    IF @tcount = 0
      BEGIN TRANSACTION asisp_deleteoldexceptions_5
    ELSE
      SAVE TRANSACTION asisp_deleteoldexceptions_5

  /* 
     first, send a clearing "OK" for all Unknown Priority AND Unknown/Green Alert State exceptions
     that fit the time window - there's no way
     for the customer to get rid of them from the console so we'll take care of them here.
  */

    -- limit the number of exceptions we'll clear to 250 at a time
    INSERT INTO StagedEXCP (ctime, parent_cid, parent_id, TIME, EXCP_NAME, EXCP_CD, nEXCH, AlertState, Priority, SEQ_NO)
    SELECT TOP 250 CURRENT_TIMESTAMP, _ParentCID, _ParentID, CURRENT_TIMESTAMP, name, 'SC - Cleared by asisp_deleteoldexceptions', 1, _AlertState, _Priority, _SEQ_NO
    FROM Exception_V e 
    WHERE e.deleted = 0 AND e._Priority=0 AND e._AlertState in (0,1) AND e._DATE < @oldest 
    AND NOT EXISTS (SELECT 1 -- don't clear an exception that already has another instance of the exception queued for processing
                    FROM StagedEXCP se 
                    WHERE se.parent_cid = e._ParentCID 
                    AND se.parent_id    = e._ParentID
                    AND se.EXCP_NAME    = e.name
                    AND se.AlertState   = e._AlertState
                    AND se.Priority     = e._Priority)
    ORDER BY _DATE -- by sorting by date, we take care of the oldest exceptions first

    SELECT @num_unknown = @@ROWCOUNT, @err = @@ERROR
  
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: Fail to schedule unknown exceptions for clearing. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_deleteoldexceptions_5
      RETURN -30000
    END

    IF @tcount = 0
      COMMIT TRANSACTION asisp_deleteoldexceptions_5

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: Scheduled %d unknown exceptions for clearing', 0, 1, @now, @num_unknown) WITH NOWAIT
    END
  END --- of option 0 or option 5 doing send a clearing "OK" for Unknown exceptions.


IF @options = 0 or @options=2 BEGIN 
-- Standard transaction logic
--  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION asisp_deleteoldexceptions
  ELSE
    SAVE TRANSACTION asisp_deleteoldexceptions

  --- REPLACING THE CALL TO EXEC asisp_update_excp_counts "_AlertState", "1"
  ----  with inline code 
  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: starting process of updating exception counts for alertstate', 0, 1, @now) WITH NOWAIT
    END

  INSERT INTO #tempehist (tid, texcp_count)  
    SELECT _ParentID, COUNT(_ParentID)  
      FROM ExceptionHistory_A WITH (NOLOCK)
      GROUP BY _ParentID  
     SET @rc=@@ROWCOUNT    
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)    
    RAISERROR('%s: asisp_deleteoldexceptions: done insert into #tempehist records = %d', 0, 1, @now, @rc) WITH NOWAIT      
    
  -- Retreive Exception records whose counts will be updated     
   INSERT INTO #tempexcp (tid, torig_count)  
          SELECT id, _nEXCH FROM Exception_V WITH (NOLOCK)
        WHERE deleted = 0 AND _AlertState = 1
     SET @rc=@@ROWCOUNT    

  IF @verbose >= 1
    BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)    
    RAISERROR('%s: asisp_deleteoldexceptions: done insert into #tempexcp records=ed = %d', 0, 1, @now, @rc) WITH NOWAIT      
  END
  
    
  --  Calculate number of ExceptionHistory records      
  --  Better performance achieved by using two temp tables instead of updating one  
  --    temp table with ExceptionHistory_A count directly  

    
    UPDATE te SET texcp_count = eh.texcp_count  
      FROM #tempexcp AS te  
      JOIN #tempehist AS eh ON te.tid = eh.tid  and te.texcp_count <> eh.texcp_count
      
    SELECT @rc = @@ROWCOUNT    

  IF @verbose >= 1
    BEGIN
     SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)    
     RAISERROR('%s: asisp_deleteoldexceptions: done update into  records=ed = %d', 0, 1, @now, @rc) WITH NOWAIT      
    END 

  IF @verbose >= 1
   BEGIN    
     SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)    
     RAISERROR('%s: asisp_deleteoldexceptions: Temp records updated = %d', 0, 1, @now, @rc) WITH NOWAIT      
   END  
    
  --  Update exception counts based on ExceptionHistory records    
  UPDATE ev SET _nEXCH = te.texcp_count FROM Exception_V AS ev    
    JOIN #tempexcp AS te ON te.tid = ev.id   
    WHERE te.torig_count <> te.texcp_count  
       

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: finished process of updating exception counts for alertstate', 0, 1, @now) WITH NOWAIT
    END

  --- END OF RELPLACING CALL TO asisp_update_excp_counts


  UPDATE c SET deleted = 1 FROM Exception_C AS c
    JOIN Exception_C AS a ON a.id = c.id WHERE a._nEXCH = 0 AND a._AlertState = 1

  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to update Exception_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: Updated deleted=1 on Exception_C - count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

  --
  -- Code updating LatestObjectActivity is deleted. Code became too cluttered leaving it here
  -- We determined that this table is only referenced by _DeleteIdleObjects which is not invoked by anyone.

  -- If any exceptions that are candidates for deletion have uncleared OwnedEvent records these need
  -- to be cleared so the notes may be properly closed.
  UPDATE OwnedEvent
  SET status = 1
  FROM Exception_V AS e
    JOIN OwnedEvent AS oe ON oe.eventid = e.id AND oe.eventcid = 'EXCP'
  WHERE e.ctime < @oldest
    AND e.deleted = 1
    AND oe.status = 0
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to update from OwnedEvent. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done updating OwnedEvent on Exception_V - count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

  -- Delete all the exceptions that can be deleted.
  DELETE FROM Exception_C WHERE ctime < @oldest AND deleted = 1
  SET @num_excp = @@ROWCOUNT
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from Exception_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting Exception_C - count %d.', 0, 1, @now, @num_excp) WITH NOWAIT
    END

  DELETE FROM BatchCycleHist WHERE eventcid = 'EXCP' AND eventid NOT IN (SELECT id FROM Exception_C)
  SET @num_bch = @@ROWCOUNT
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from BatchCycleHist. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting BatchCycleHist - count %d.', 0, 1, @now, @rc) WITH NOWAIT
    END

  DELETE FROM BatchCycleJobHist WHERE eventcid = 'EXCP' AND eventid NOT IN (SELECT id FROM Exception_C)
  SET @num_bcjh = @@ROWCOUNT
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from BatchCycleJobHist. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting BatchCycleJobHist - count %d.', 0, 1, @now, @rc) WITH NOWAIT
    END

  DELETE FROM ProcessedStagedEXCP WHERE ptime < @oldest
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from ProcessedStagedEXCP. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting ProcessedStagedEXCP - count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

  DELETE ExceptionHistory_A
    WHERE _ParentID NOT IN (SELECT id FROM Exception_V WHERE _AlertState <> 1 AND deleted = 0)
    AND ctime < @oldest
  SET @num_exch = @@ROWCOUNT
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from ExceptionHistory_A. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting ExceptionHistory_A - count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

  DELETE FROM ExceptionHistory_C WHERE id NOT IN (SELECT id FROM ExceptionHistory_A) 
    AND ctime < @oldest
  SET @num_exch = @@ROWCOUNT
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Fail to delete from ExceptionHistory_C. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_deleteoldexceptions
    RETURN -30000
  END

  IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_deleteoldexceptions: done deleting ExceptionHistory_C - count %d', 0, 1, @now, @rc) WITH NOWAIT
    END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_deleteoldexceptions: Total deleted: #Ignores=%d, #Exception=%d, #ExceptionHistory=%d, #BatchCycleHist=%d, #BatchCycleJobHist=%d, #Unknowns=%d', 0, 1,
              @now, @num_ignore, @num_excp, @num_exch, @num_bch, @num_bcjh, @num_unknown) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION asisp_deleteoldexceptions
---- THIS IS THE END OF OPTION 2 PUT A MESSAGE OUT OF HERE
  END --- if options = 0 or 2 


  IF @options=0 or @options=3 BEGIN
	  IF @verbose >= 1
	  BEGIN
	    SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
	    RAISERROR('%s: asisp_deleteoldexceptions: calling clear_jclerrors to delete jcl errors posted to Generic objects', 0, 1, @now) WITH NOWAIT
	  END
	  -- delete jcl errors exceptions posted to the GOBJ objects on each OS
	  EXEC clear_jclerrors
	  
	
	  IF @verbose >= 1
	    BEGIN
	      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
	      RAISERROR('%s: asisp_deleteoldexceptions: done with clear_jclerrors and commiting transaction - count %d.', 0, 1, @now, @rc) WITH NOWAIT
	    END
   END  --- OF options 0 or option 3 


 IF @options=0 or @options=4 BEGIN  

  -- Delete old AlertStateHistory data 
  exec asisp_deleteoldalertstatehistory @verbose=@verbose, 
                                        @daysToKeepAlert=@daysToKeepAlert, 
                                        @limit=@limit,
                                        @maxRowsToDelete=@maxRowsToDelete, 
                                        @options=0

      
 END -- end of option 4 

  SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
  RAISERROR('%s: asisp_deleteoldexceptions: Total deleted: #Ignores=%d, #Exception=%d, #ExceptionHistory=%d, #BatchCycleHist=%d, #BatchCycleJobHist=%d, #Unknowns=%d', 0, 1,
            @now, @num_ignore, @num_excp, @num_exch, @num_bch, @num_bcjh, @num_unknown) WITH NOWAIT


  SET ROWCOUNT 0
  SET NOCOUNT OFF

GO