-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2008 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_update_cl_idcache.sql, tbsm.sql, tbsm_base31e
-- SID: 1.15
-- Date Modified: 08/05/01 10:26:00
--
-- Module Description
--   Defines the asisp_update_CL_IDCache stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_update_CL_IDCache
PRINT 'Creating procedure asisp_update_CL_IDCache'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_update_CL_IDCache' AND type = 'P')
  DROP PROCEDURE asisp_update_CL_IDCache
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2008 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_update_cl_idcache.sql, tbsm.sql, tbsm_base31e
-- SID: 1.15
-- Date Modified: 08/05/01 10:26:00
--
-- If @table is specified, it must have the following form:
/*
  CREATE TABLE #temp
  (
    cid      ClassID,
    id       ObjID,
    name     ObjName NULL,
    action   TINYINT, -- 0 means new, 1 means rename, 2 means delete
    old_name ObjName NULL,
    tstamp   BINARY(8)
  )
*/
CREATE PROCEDURE asisp_update_CL_IDCache
  @cid     ClassID,
  @id      ObjID,
  @table   sysname = NULL,  -- The name of the input table
  @verbose TINYINT = 0
AS
  SET NOCOUNT ON
  DECLARE @rc INT, @now NVARCHAR(100), @tcount INT, @err INT
  DECLARE @obj_path NVARCHAR(2000), @obj_upath NVARCHAR(400), @src_cid ClassID, @src_id ObjID

  -- Validate the table existence if specified by @table
  IF @table IS NOT NULL
  BEGIN
    DECLARE @tname sysname
    IF SUBSTRING(@table, 1, 1) = '#'
      SET @tname = 'tempdb..' + @table
    IF OBJECT_ID(@tname) IS NULL
    BEGIN
      SET @now = CONVERT(NVARCHAR(100), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: Could not find table %s.', 11, 1, @now, @table) WITH NOWAIT
      RETURN -30000
    END
  END

  -- temp tables
  CREATE TABLE #lob
  (
    cid         ClassID,
    id          ObjID,
    name        NVARCHAR(100)  NULL,
    path        NVARCHAR(2000) NULL,
    upath       NVARCHAR(400)  NULL,
    gen         INT,
    description NVARCHAR(255)  NULL,
    flags       INT     NOT NULL DEFAULT 0,
    src_cid     ClassID     NULL,
    src_id      ObjID       NULL,
    done        TINYINT NOT NULL DEFAULT 0,
    ident       INT IDENTITY(1,1)
  )
  CREATE TABLE #non_dups
  (
    cid      ClassID,
    id       ObjID,
    name     ObjName,
    src_cid  ClassID,
    src_id   ObjID
  )
  CREATE TABLE #dups
  (
    cid     ClassID,
    id      ObjID,
    name    ObjName,
    src_cid ClassID,
    src_id  ObjID,
    lnk_cid ClassID       NULL,
    lnk_id  ObjID         NULL,
    name_id NVARCHAR(128) NULL,
    gen     INT,
    ident   INT      IDENTITY(1,1)
  )
  CREATE TABLE #links
  (
    src_cid   ClassID,
    src_id    ObjID,
    dst_cid   ClassID,
    dst_id    ObjID,
    link_type NVARCHAR(4),
    id        INT IDENTITY(1,1)
  )
  CREATE TABLE #renames
  (
    cid      ClassID,
    id       ObjID,
    name     ObjName,
    old_name ObjName,
    tstamp   BINARY(8)
  )
  CREATE TABLE #dup_renames
  (
    cid        ClassID,
    id         ObjID,
    name       ObjName NULL,
    old_name   ObjName NULL,
    max_tstamp BINARY(8),
    min_tstamp BINARY(8)
  )
  CREATE TABLE #deletes
  (
    cid ClassID,
    id  ObjID
  )
  CREATE TABLE #CL_IDCache
  (
    cid     ClassID PRIMARY KEY (cid, oid),
    oid     ObjID,
    value   NVARCHAR(425),
    changed TINYINT DEFAULT 0,
    new     TINYINT DEFAULT 0
  )
  CREATE TABLE #names
  (
    name ObjName,
    row_type TINYINT -- 0=added, 1=renamed, 2=deleted
  )
  CREATE TABLE #desc
  (
    cid ClassID,
    id  ObjID,
    gen INT
  )
  CREATE INDEX lobidx ON #lob (cid, id, name, src_cid, src_id)
  CREATE INDEX lnkidx ON #links (src_cid, src_id, dst_cid, dst_id, link_type)
  CREATE INDEX cacheidx ON #CL_IDCache (cid, oid, value)

  -- Indexes specific to database migration
  IF @table IS NULL
  BEGIN
    CREATE INDEX LINK_SRCCID_SRCID_DSTCID_DSTID_LINKTYPE ON #links (src_cid, src_id, dst_cid, dst_id, link_type)
    CREATE INDEX LINK_DSTCID_DSTID ON #links (dst_cid, dst_id)
    CREATE INDEX LOB_GEN_NAME_ID_SRCCID_SRCID ON #lob (gen, name, id, src_cid, src_id)
  END
  
  --
  -- Populate the path and name variables
  -- 
  IF @cid = 'LOB' AND @table IS NULL
  BEGIN
    INSERT INTO #lob (cid, id, name, path, upath, gen, description, flags)
    SELECT @cid, id, name, name, RTRIM(@cid) + ':' + CAST(id AS NVARCHAR(9)), 0, _Desc, _flags
    FROM LineOfBusiness_V
    WHERE id = @id
      AND deleted = 0
    SET @rc = @@ROWCOUNT
    -- Check to ensure that the LOB exists
    IF @rc = 0
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: ERROR: LOB %d does not exist or has been deleted.', 11, 1, @now, @id) WITH NOWAIT
      RETURN -30000
    END

    --
    -- Get the path info for this LOB
    --
    -- Start with the resources name
    SELECT @obj_path = name, @obj_upath = RTRIM(cid) + ':' + CAST(id AS NVARCHAR(9)), @src_cid = cid, @src_id = id
    FROM #lob
    WHERE cid = @cid AND id = @id
    -- Get the source object
    SELECT @src_cid = c.src_cid, @src_id = c.src_id
    FROM containment AS c
    WHERE c.deleted = 0
      AND c.dst_cid = @src_cid AND c.dst_id = @src_id
    -- loop
    WHILE @rc > 0 AND @src_cid IS NOT NULL
    BEGIN
      IF @src_cid = 'LOB'
      BEGIN
        SELECT @obj_path = l.name + '/' + @obj_path, @obj_upath = 'LOB:' + CAST(l.id AS NVARCHAR(9)) + '/' + @obj_upath,
          @src_cid = c.src_cid, @src_id = c.src_id
        FROM LineOfBusiness_V AS l
          JOIN containment AS c ON c.dst_cid = 'LOB' AND c.dst_id = l.id
        WHERE c.deleted = 0 AND l.deleted = 0
          AND c.dst_cid = @src_cid AND c.dst_id = @src_id
        SET @rc = @@ROWCOUNT
      END -- IF @src_cid = 'LOB'
      ELSE
      BEGIN
        SET @obj_path = 'LOBC/' + @obj_path
        SET @obj_upath = 'LOBC:1/' + @obj_upath
        SET @rc = 0
      END
    END -- WHILE @rc > 0

    -- Check to make sure we were able to go all the way up the hierarchy
    IF @src_cid <> 'LOBC'
    BEGIN
      SET @now = CONVERT(NVARCHAR(40), CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: ERROR: Could not get to LOBC from LOB %d.  The LOB is orphaned.', 11, 1, @now, @id) WITH NOWAIT
      RETURN -30000
    END

    -- Update the LOB with the new path info
    UPDATE #lob SET path = @obj_path, upath = @obj_upath
    
  END -- IF @cid = 'LOB'
  ELSE IF @table IS NULL -- LOBC
    INSERT INTO #lob (cid, id, name, path, upath, gen)
    SELECT @cid, 1, 'LOBC', 'LOBC', 'LOBC:1', 0

  --
  -- Traverse the LOB tree, populating the #lob temp table
  --
  DECLARE @continue TINYINT, @gen INT
  SET @continue = CASE WHEN @table IS NULL THEN 1 ELSE 0 END
  SET @gen = 1
  WHILE @continue = 1
  BEGIN

    -- Get the link information
    INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
    SELECT DISTINCT c.src_cid, c.src_id, c.dst_cid, c.dst_id, 'LOBC'
    FROM #lob AS parent
      JOIN containment AS c ON c.src_cid = parent.cid AND c.src_id = parent.id AND c.dst_cid = 'LOB'
      LEFT OUTER JOIN #links AS e ON e.src_cid = c.src_cid AND e.src_id = c.src_id
        AND e.dst_cid = c.dst_cid AND e.dst_id = c.dst_id AND e.link_type = 'LOBC'
    WHERE e.dst_cid IS NULL
      AND c.deleted = 0

    -- Get children via LOBC links
    INSERT INTO #lob (cid, id, name, path, upath, gen, description, flags)
    SELECT 'LOB', child.id, child.name, parent.path + '/' + child.name, parent.upath + '/LOB:' + CAST(child.id AS NVARCHAR(9)), @gen, child.description, child._flags
    FROM #lob AS parent
      JOIN #links AS c ON c.src_cid = parent.cid AND c.src_id = parent.id AND c.dst_cid = 'LOB' AND c.link_type = 'LOBC'
      JOIN LineOfBusiness_V AS child ON child.id = c.dst_id
    WHERE child.deleted = 0
      -- AND c.deleted = 0
      AND parent.gen = @gen - 1
    SET @rc = @@ROWCOUNT

    -- set the end-of-loop condition
    IF @rc = 0
      SET @continue = 0

    -- Determine if all of the resources have unique names within their context
    INSERT INTO #dups (cid, id, name, src_cid, src_id, gen)
    SELECT DISTINCT o.cid, o.id, o.name, l.src_cid, l.src_id, @gen
    FROM #lob AS o
      JOIN #links AS l ON l.dst_cid = o.cid AND l.dst_id = o.id
    WHERE o.gen = @gen
      AND EXISTS (
        SELECT *
        FROM #lob AS d
          JOIN #links AS dl ON dl.dst_cid = d.cid AND dl.dst_id = d.id
        WHERE d.gen = o.gen AND d.name = o.name AND d.id <> o.id
          AND dl.src_cid = l.src_cid AND dl.src_id = l.src_id
      )
    SET @rc = @@ROWCOUNT

    SET @gen = @gen + 1
  END -- WHILE @bcontinue = 1

  -- Populate the #lob table based on the contents of the input table
  IF @table IS NOT NULL
  BEGIN
    -- get the LOBs
    EXEC('
    INSERT INTO #lob (cid, id, name, path, description, src_cid, src_id, gen)
    SELECT t.cid, t.id, l.name, l.name, l.description, c.src_cid, c.src_id, 0
    FROM ' + @table + ' AS t
      JOIN LineOfBusiness_C AS l ON l.id = t.id AND t.cid = ''LOB''
      JOIN containment AS c ON c.dst_cid = t.cid AND c.dst_id = t.id
    WHERE l.deleted = 0
      AND c.deleted = 0
      -- only process new LOBs
      AND t.action = 0
    ')
    SET @rc = @@ROWCOUNT

    -- Populate the path column
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: Populate the path column', 0, 1, @now) WITH NOWAIT
    END

    WHILE @rc > 0
    BEGIN

      -- Update for parent LOBC
      UPDATE l
      SET l.path = 'LOBC/' + l.path, l.done = 1
      FROM #lob AS l
      WHERE l.src_cid = 'LOBC'
        AND l.done = 0

      -- Update for LOB parent
      UPDATE l
      SET l.path = obj.name + '/' + l.path, l.src_cid = c.src_cid, l.src_id = c.src_id
      FROM #lob AS l
        JOIN LineOfBusiness_C AS obj ON obj.id = l.src_id AND l.src_cid = 'LOB'
        JOIN containment AS c ON c.dst_cid = l.src_cid AND c.dst_id = l.src_id
      WHERE c.deleted = 0 AND obj.deleted = 0 AND l.done = 0
      SET @rc = @@ROWCOUNT

    END -- WHILE @rc > 0

    -- Register duplicates
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: Register duplicates', 0, 1, @now) WITH NOWAIT
    END

    UPDATE lob SET src_cid=l.src_cid, src_id=l.src_id FROM #lob lob
      JOIN link l ON lob.cid=l.dst_cid AND lob.id=l.dst_id
      WHERE l.deleted = 0

    INSERT INTO #dups (cid, id, name, src_cid, src_id, gen)
    SELECT DISTINCT o.cid, o.id, o.name, l.src_cid, l.src_id, @gen
    FROM LineOfBusiness_C AS o
      JOIN containment AS l ON l.dst_cid = 'LOB' AND l.dst_id = o.id
      -- restrict the #dups table to LOBs that have the same name as those in our set
      JOIN #lob AS lob ON lob.name = o.name AND lob.src_cid=l.src_cid AND lob.src_id=l.src_id AND lob.id<>o.id
    WHERE l.deleted = 0
      AND o.deleted = 0

    -- Collect a list of renames
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: Collect a list of renames', 0, 1, @now) WITH NOWAIT
    END

    EXEC('
    INSERT INTO #renames (cid, id, name, old_name, tstamp)
    SELECT DISTINCT t.cid, t.id, t.name, t.old_name, t.tstamp
    FROM ' + @table + ' AS t
      JOIN CL_IDCache AS c ON c.cid = t.cid AND c.oid = t.id
    WHERE t.action = 1
    ')

    -- Collect a list of deletes
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: Collect a list of deletes', 0, 1, @now) WITH NOWAIT
    END

    EXEC('
    INSERT INTO #deletes (cid, id)
    SELECT cid, id
    FROM ' + @table + ' 
    WHERE action = 2
    ')

  END -- IF @table IS NOT NULL

  -- eliminate anything that is not an LOB
  DELETE FROM #lob WHERE cid <> 'LOB'

  -- Get the LOBL links
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Get the LOBL links', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #links (src_cid, src_id, dst_cid, dst_id, link_type)
  SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, 'LOBL'
  FROM #lob AS t
    JOIN link AS l ON l.src_cid = t.cid AND l.src_id = t.id
  WHERE l.deleted = 0
    AND l.link_type = 'LOBL'

  -- Delete non-dup objects/dup objects with the lowest ids
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Delete dup objects with the lowest ids', 0, 1, @now) WITH NOWAIT
  END

  UPDATE d
    SET lnk_cid = l.dst_cid, lnk_id = l.dst_id
  FROM #dups AS d
    JOIN #links AS l ON l.src_cid = d.cid AND l.src_id = d.id
  WHERE l.link_type = 'LOBL'

  UPDATE #dups
    SET name_id = CASE
                    WHEN lnk_cid IS NOT NULL AND lnk_id IS NOT NULL
                      THEN name + lnk_cid + CAST(lnk_id AS NVARCHAR(9))
                    ELSE name END

  DELETE d
  FROM #dups AS d
  WHERE NOT EXISTS (
    SELECT *
    FROM #dups AS d1
    WHERE d1.src_cid = d.src_cid AND d1.src_id = d.src_id AND d1.name_id = d.name_id AND d1.cid = d.cid AND d1.id < d.id
  )

  -- Process dups
  UPDATE lob
  SET lob.path = lob.path + '(' +
    CASE WHEN d.cid IS NULL THEN RTRIM(l.dst_cid) + ':' + CAST(l.dst_id AS NVARCHAR(9))
    ELSE RTRIM(lob.cid) + ':' + CAST(lob.id AS NVARCHAR(9)) END +
    ')'
  FROM #lob AS lob
    LEFT OUTER JOIN #links AS l ON l.src_cid = lob.cid AND l.src_id = lob.id
      AND l.link_type = 'LOBL' AND l.dst_cid <> 'LOB'
    LEFT OUTER JOIN #dups AS d ON d.cid = lob.cid AND d.id = lob.id
  WHERE (l.src_cid IS NOT NULL OR d.cid IS NOT NULL)

  -- Process LOBs that have duplicate path info that do not have the same parents
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Process LOBs that have dup path info that do not have same parents', 0, 1, @now) WITH NOWAIT
  END

  UPDATE l
  SET l.path = l.path + '(' + RTRIM(l.cid) + ':' + CAST(l.id AS NVARCHAR(9)) + ')'
  FROM #lob AS l
    JOIN #lob AS l2 ON l.path = l2.path AND l.ident <> l2.ident
  WHERE EXISTS (
    SELECT *
    FROM #lob AS l3
    WHERE l3.path = l.path AND l3.id < l.id
  )

  -- Insert new records into the CL_IDCache table
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Insert new records into CL_IDCache', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #CL_IDCache (oid, cid, value, new)
  SELECT l.id, l.cid, l.path, 1
  FROM #lob AS l
    LEFT OUTER JOIN CL_IDCache AS c ON c.cid = l.cid AND c.oid = l.id
  WHERE c.cid IS NULL

  --
  -- Process renames
  --

  -- Consolidate multiple renames
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Consolidate multiple renames', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #dup_renames (cid, id, max_tstamp, min_tstamp)
  SELECT r.cid, r.id, MAX(r.tstamp), MIN(r.tstamp)
  FROM #renames AS r
    JOIN #renames AS r1 ON r1.cid = r.cid AND r1.id = r.id
  WHERE r.tstamp <> r1.tstamp
  GROUP BY r.cid, r.id

  -- Get the extreme name values
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Get the extreme name values', 0, 1, @now) WITH NOWAIT
  END

  UPDATE d
  SET d.name = (SELECT name FROM #renames WHERE tstamp = d.max_tstamp),
      d.old_name = (SELECT old_name FROM #renames WHERE tstamp = d.min_tstamp)
  FROM #dup_renames AS d

  -- Delete the old entries from #renames
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Delete old entries from #renames', 0, 1, @now) WITH NOWAIT
  END

  DELETE r
  FROM #renames AS r
    JOIN #dup_renames AS d ON d.cid = r.cid AND d.id = r.id

  -- Insert the new entries
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Insert new entries', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #renames (cid, id, name, old_name, tstamp)
  SELECT cid, id, name, old_name, max_tstamp
  FROM #dup_renames

  INSERT INTO #CL_IDCache (cid, oid, value)
  SELECT cid, oid, value
  FROM CL_IDCache
  WHERE cid = 'LOB' AND keyword = 'CL_InstID' AND deleted = 0

  -- Cursor over the renames
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Cursor over the renames', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @c_cid ClassID, @c_id ObjID, @c_name ObjName, @c_old_name ObjName, @c_oldpath NVARCHAR(2000), @c_newpath NVARCHAR(2000)
  DECLARE @phy_cid ClassID, @phy_id ObjID
  DECLARE c_ren INSENSITIVE CURSOR FOR
    SELECT DISTINCT cid, id, name, old_name
    FROM #renames
  OPEN c_ren
  FETCH NEXT FROM c_ren INTO @c_cid, @c_id, @c_name, @c_old_name
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    -- Get the path from the CL_IDCache table
    SET @c_oldpath = NULL
    SELECT @c_oldpath = value
    FROM #CL_IDCache
    WHERE cid = @c_cid AND oid = @c_id

    -- Get the physical cid/id
    SELECT @phy_cid = NULL, @phy_id = NULL
    SELECT @phy_cid = phy_cid, @phy_id = phy_id
    FROM lob_link
    WHERE src_id = 0 AND dst_id = @c_id
      AND @c_cid = 'LOB'

    -- Form the new path string
    IF CHARINDEX(REVERSE('/' + @c_old_name), REVERSE(@c_oldpath)) = 1
      SET @c_newpath = SUBSTRING(@c_oldpath, 1, LEN(@c_oldpath) - LEN(@c_old_name)) + @c_name
    ELSE
      SET @c_newpath = NULL

    -- Check to see if the name has the cid and id appended to it
    IF @c_newpath IS NULL
      AND CHARINDEX(REVERSE('/' + @c_old_name + '(' + RTRIM(@c_cid) + ':' + CAST(@c_id AS NVARCHAR(9)) + ')'), REVERSE(@c_oldpath)) = 1
      SET @c_newpath = SUBSTRING(@c_oldpath, 1, LEN(@c_oldpath) - LEN(@c_old_name + '(' + RTRIM(@c_cid) + ':' + CAST(@c_id AS NVARCHAR(9)) + ')')) + @c_name

    -- Check to see if the name has the physical cid and id appended to it
    IF @c_newpath IS NULL
      AND CHARINDEX(REVERSE('/' + @c_old_name + '(' + RTRIM(@phy_cid) + ':' + CAST(@phy_id AS NVARCHAR(9)) + ')'), REVERSE(@c_oldpath)) = 1
      AND @phy_cid IS NOT NULL AND @phy_id IS NOT NULL
      SET @c_newpath = SUBSTRING(@c_oldpath, 1, LEN(@c_oldpath) - LEN(@c_old_name + '(' + RTRIM(@phy_cid) + ':' + CAST(@phy_id AS NVARCHAR(9)) + ')')) + @c_name

    DELETE FROM #desc

    -- Insert current record into the #desc table
    IF @c_newpath IS NOT NULL
    BEGIN
      INSERT INTO #desc (cid, id, gen)
      VALUES (@c_cid, @c_id, 0)
      SET @rc = 1

      -- Perform the rename for gen 0
      UPDATE c
      SET c.value = @c_newpath + SUBSTRING(c.value, LEN(@c_oldpath), LEN(c.value) - LEN(@c_oldpath)),
          c.changed = 1
      FROM #CL_IDCache AS c
        JOIN #desc AS d ON d.cid = c.cid AND d.id = c.oid
      WHERE d.gen = 0

      -- Append the cid:id for physical resources
      UPDATE c
      SET c.value = c.value + '(' + RTRIM(@phy_cid) + ':' + CAST(@phy_id AS NVARCHAR(9)) + ')'
      FROM #CL_IDCache AS c
    JOIN #desc AS d ON d.cid = c.cid AND d.id = c.oid
      WHERE d.gen = 0
        AND @phy_cid IS NOT NULL AND @phy_id IS NOT NULL

    END

    -- loop over the descendents
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: loop over the descendents', 0, 1, @now) WITH NOWAIT
    END

    SET @gen = 0
    WHILE @rc > 0
    BEGIN

      -- Get the next level of children to rename
      INSERT INTO #desc (cid, id, gen)
      SELECT c.dst_cid, c.dst_id, @gen + 1
      FROM containment AS c
        JOIN #desc AS d ON c.src_cid = d.cid AND c.src_id = d.id
      WHERE d.gen = @gen
        AND c.deleted = 0
      SET @rc = @@ROWCOUNT

      -- Perform the rename
      UPDATE c
      SET c.value = @c_newpath + SUBSTRING(c.value, LEN(@c_oldpath) + 1, LEN(c.value) - LEN(@c_oldpath)),
          c.changed = 1
      FROM #CL_IDCache AS c
        JOIN #desc AS d ON d.cid = c.cid AND d.id = c.oid
      WHERE d.gen = @gen + 1

      SET @gen = @gen + 1
    END

    FETCH NEXT FROM c_ren INTO @c_cid, @c_id, @c_name, @c_old_name
  END
  DEALLOCATE c_ren

  --
  -- Handle removing or adding duplicate nomenclature based on the deletes and renames
  --
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Handle removing/adding dup nomenclature based on deletes and renames', 0, 1, @now) WITH NOWAIT
  END

  -- Get a list of names to consider
  INSERT INTO #names (name, row_type)
  SELECT DISTINCT name, 0 -- added
  FROM #renames

  INSERT INTO #names (name, row_type)
  SELECT DISTINCT old_name, 1 -- renamed
  FROM #renames

  INSERT INTO #names (name, row_type)
  SELECT DISTINCT l.name, 2 -- removed / deleted
  FROM LineOfBusiness_C AS l
    JOIN #deletes AS d ON d.cid = 'LOB' AND d.id = l.id

  -- Process LOBs that have names that match the list accumulated that are also in the CL_IDCache table
  DELETE FROM #dups

  -- find new duplicates
  INSERT INTO #dups (cid, id, name, src_cid, src_id, gen)
  SELECT 'LOB', l.id, l.name, c.src_cid, c.src_id, 0
  FROM CL_IDCache AS cl
    JOIN LineOfBusiness_C AS l ON l.id = cl.oid AND cl.cid = 'LOB'
    JOIN #names AS n ON n.name = l.name
    JOIN containment AS c ON c.dst_cid = 'LOB' AND c.dst_id = l.id
  WHERE c.deleted = 0
    AND n.row_type = 0 -- added name
    AND EXISTS (
    SELECT *
    FROM LineOfBusiness_C AS d
      JOIN containment AS c1 ON c1.dst_cid = 'LOB' AND c1.dst_id = d.id
    WHERE d.name = l.name AND d.id <> l.id
      AND c1.src_cid = c.src_cid AND c1.src_id = c.src_id
      AND d.deleted = 0
      AND c1.deleted = 0
  )

  -- Delete non-dup objects/dup objects with the lowest ids
  UPDATE d
    SET lnk_cid = l.dst_cid, lnk_id = l.dst_id
  FROM #dups AS d
    JOIN #links AS l ON l.src_cid = d.cid AND l.src_id = d.id
  WHERE l.link_type = 'LOBL'

  UPDATE #dups
    SET name_id = CASE
                    WHEN lnk_cid IS NOT NULL AND lnk_id IS NOT NULL
                      THEN name + lnk_cid + CAST(lnk_id AS NVARCHAR(9))
                    ELSE name END

  DELETE d
  FROM #dups AS d
  WHERE NOT EXISTS (
    SELECT *
    FROM #dups AS d1
    WHERE d1.src_cid = d.src_cid AND d1.src_id = d.src_id AND d1.name_id = d.name_id AND d1.cid = d.cid AND d1.id < d.id
  )

  -- Update the #CL_IDCache table
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Update #CL_IDCache table', 0, 1, @now) WITH NOWAIT
  END

  UPDATE c
  SET c.value = c.value + '(' + RTRIM(c.cid) + ':' + CAST(c.oid AS NVARCHAR(9)) + ')',
      c.changed = 1
  FROM #CL_IDCache AS c
    JOIN #dups AS d ON d.cid = c.cid AND d.id = c.oid
    LEFT OUTER JOIN lob_link AS l ON l.dst_id = c.oid AND l.src_id = 0 AND c.cid = 'LOB'
  -- make sure the entry does not already have the cid:id appended to its value
  WHERE (CHARINDEX(REVERSE(d.name + '(' + RTRIM(c.cid) + ':' + CAST(c.oid AS NVARCHAR(9)) + ')'), REVERSE(c.value)) <> 1
    AND CHARINDEX(REVERSE(d.name + '(' + RTRIM(l.phy_cid) + ':' + CAST(l.phy_id AS NVARCHAR(9)) + ')'), REVERSE(c.value)) <> 1)

  -- Find renames that would potentially eliminate duplicates
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Find renames that would potentially eliminate dups', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #non_dups (cid, id, name, src_cid, src_id)
  SELECT 'LOB', l.id, l.name, c.src_cid, c.src_id
  FROM #names AS n
    JOIN #renames AS r ON r.old_name = n.name
    JOIN LineOfBusiness_C AS l ON l.name = n.name AND l.id <> r.id AND r.cid = 'LOB'
    JOIN containment AS c ON c.dst_cid = 'LOB' AND c.dst_id = l.id
    JOIN #CL_IDCache AS cl ON cl.cid = c.dst_cid AND cl.oid = c.dst_id
  WHERE n.row_type in (1,2) -- renamed or deleted/removed
    -- do not check for deleted, as we could be handling deleted resources here that will cause nondeleted resources
    -- to no longer be duplicates
    -- AND c.deleted = 0 AND l.deleted = 0
    AND CHARINDEX(REVERSE(l.name + '(LOB:' + CAST(l.id AS NVARCHAR(9)) + ')'), REVERSE(cl.value)) = 1

  -- Delete rows in #non_dups that are still duplicates
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Delete rows in #non_dups that are still dups', 0, 1, @now) WITH NOWAIT
  END

  DELETE n
  FROM #non_dups AS n
  WHERE EXISTS (
    SELECT *
    FROM LineOfBusiness_C AS d
      JOIN containment AS c1 ON c1.dst_cid = 'LOB' AND c1.dst_id = d.id
    WHERE d.name = n.name AND d.id < n.id AND n.cid = 'LOB'
--  Just because the parent ID is different, doesn't mean that the parent name is different.
--  These could still be duplicates.  
--    AND c1.src_cid = n.src_cid AND c1.src_id = n.src_id 
      AND c1.deleted = 0
      AND d.deleted = 0
  )

  -- Process LOBs that have duplicate path info that do not have the same parents
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Process LOBs that have dup path info that do not have same parents', 0, 1, @now) WITH NOWAIT
  END

  UPDATE cl
  SET cl.value = cl.value + '(' + RTRIM(cl.cid) + ':' + CAST(cl.oid AS NVARCHAR(9)) + ')',
      cl.changed = 1
  FROM #CL_IDCache AS cl
    JOIN #CL_IDCache AS cl2 ON cl.value = cl2.value AND cl.cid = cl2.cid AND cl.oid > cl2.oid

  -- Update the #CL_IDCache table: append the cid:id values for dups if not already present
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: Update #CL_IDCache - append cid/id values for dups if not already present', 0, 1, @now) WITH NOWAIT
  END

  UPDATE c
  SET c.value = SUBSTRING(c.value, 1, LEN(c.value) - LEN('(' + RTRIM(c.cid) + ':' + CAST(c.oid AS NVARCHAR(9)) + ')')),
      c.changed = 1
  FROM #CL_IDCache AS c
    JOIN #non_dups AS n ON n.cid = c.cid AND n.id = c.oid
  -- verify that the entry already has the cid and id appended to its value
  WHERE CHARINDEX(REVERSE(n.name + '(' + RTRIM(c.cid) + ':' + CAST(c.oid AS NVARCHAR(9)) + ')'), REVERSE(c.value)) = 1

  -- Defect 135247: Wrap in transaction logic when updating CL cache
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_update_CL_IDCache
  ELSE
    SAVE TRAN asisp_update_CL_IDCache

  -- Do a mass update of the CL_IDCache table
  UPDATE c
  SET c.value = t.value
  FROM CL_IDCache AS c
    JOIN #CL_IDCache AS t ON t.cid = c.cid AND t.oid = c.oid
  WHERE c.keyword = 'CL_InstID'
    AND t.changed = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: error updating CL_IDCache. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_update_CL_IDCache
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: update %d processed record(s) in CL_IDCache', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Do an insert of the new records
  INSERT INTO CL_IDCache (cid, oid, value, keyword)
  SELECT cid, oid, value, 'CL_InstID'
  FROM #CL_IDCache AS t
  WHERE t.new = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: error inserting into CL_IDCache. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_update_CL_IDCache
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: inserted %d processed record(s) into CL_IDCache', 0, 1, @now, @rc) WITH NOWAIT
  END

  DECLARE @instr_rowid INT
  SELECT @instr_rowid = rowid FROM CL_InstrumentationID WHERE instrID = 'TBSM_INTERNAL'
  IF @@ROWCOUNT = 0
  BEGIN
    INSERT INTO CL_InstrumentationID (instrID) VALUES ('TBSM_INTERNAL')
    SELECT @instr_rowid = rowid FROM CL_InstrumentationID WHERE instrID = 'TBSM_INTERNAL'
  END

  IF @instr_rowid > 0
  BEGIN
    INSERT INTO CL_ObjInstrMap (obj_rowid, instr_rowid)
      SELECT c.rowid, @instr_rowid FROM CL_IDCache c
        JOIN #CL_IDCache c1 ON c.cid = c1.cid AND c.oid = c1.oid AND c.value = c1.value
        WHERE c.keyword = 'CL_InstID' AND c1.new = 1

    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: error inserting into CL_ObjInstrMap. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_update_CL_IDCache
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_update_CL_IDCache: inserted %d processed record(s) into CL_ObjInstrMap', 0, 1, @now, @rc) WITH NOWAIT
    END      
  END
  ELSE BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_update_CL_IDCache: error getting instrID', 0, 1, @now) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_update_CL_IDCache
    RETURN -30000
  END

  -- Finish transaction
  IF @tcount = 0
    COMMIT TRAN asisp_update_CL_IDCache

GO
