-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2008 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/loadxrc_sessionpairs.sql, tbsm.sql, tbsm_base31e
-- SID: 1.7.1.6
-- Date Modified: 08/07/31 07:10:33
--
-- _loadxrc_sessionpairs_discoverybatch.sql
--
-- Module Description
--   Creates _LoadXRC_SessionPairs_DiscoveryBatch

USE Object

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_LoadXRC_SessionPairs_DiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _LoadXRC_SessionPairs_DiscoveryBatch
GO

PRINT 'creating procedure _LoadXRC_SessionPairs_DiscoveryBatch'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2008 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/loadxrc_sessionpairs.sql, tbsm.sql, tbsm_base31e
-- SID: 1.7.1.6
-- Date Modified: 08/07/31 07:10:33
CREATE PROCEDURE _LoadXRC_SessionPairs_DiscoveryBatch
  @cid          ClassID,
  @id           ObjID,
  @batchid      INT = NULL
AS
  DECLARE @formatType INT
  SELECT @formatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = 'XRC'

  DECLARE @installDir NVARCHAR(255)
  DECLARE @sql NVARCHAR(1024)
  DECLARE @err INT
  DECLARE @rc INT
  DECLARE @FilePath NVARCHAR(500)
  DECLARE @CodePage NVARCHAR(8)

  IF @batchid IS NOT NULL 
  BEGIN
    IF NOT EXISTS(SELECT 1 FROM DiscoveryBatch WHERE batchid = @batchid)
    BEGIN
      RAISERROR('batchid %d does not exist in DiscoveryBatch',3,1,@batchid) WITH NOWAIT
      RETURN -30000
    END
    IF NOT EXISTS(SELECT 1 FROM DiscoveryBatch WHERE batchid = @batchid AND FormatType = @formatType AND ActionType = 1)
    BEGIN
      RAISERROR('FormatType/ActionType for batchid %d is not %d',3,1,@batchid,@formatType) WITH NOWAIT
      RETURN -30000
    END
  END
  ELSE IF @batchid IS NULL
  BEGIN
    SELECT @batchid = MIN(batchid)
    FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
    WHERE b.FormatType = @formatType
      AND b.ActionType = 1
      AND v.name = 'ENQUEUED'

    IF @batchid IS NULL
    BEGIN
      RAISERROR('No XRC Session Pairs DiscoveryBatch IS IN a ENQUEUED state, nothing to do!',0,1) WITH NOWAIT
      RETURN 0
    END
  END


  SELECT * FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
   WHERE b.FormatType = @formatType
     AND b.ActionType = 1
     AND v.name = 'INPROGRESS'

  IF @@ROWCOUNT > 0
  BEGIN
    RAISERROR('There are XRC Session Pairs discovery batches IN states preventing a load from taking place.',3,1) WITH NOWAIT
    RETURN -30000
  END


  UPDATE d SET pstat = 2 --'INPROGRESS'
  FROM DiscoveryBatch d 
  INNER JOIN DiscoveryBatchPStatVal v ON v.pstat = d.pstat
  WHERE batchid = @batchid 
    AND FormatType = @formatType 
    AND ActionType = 1
    AND v.name = 'ENQUEUED' 

  IF @@ROWCOUNT <> 1
  BEGIN
    RAISERROR('The batch %d could not be moved into an ENQUEUED state.',3,1,@batchid) WITH NOWAIT
    RETURN -30000
  END


  UPDATE d SET pstat = -1 --'LOAD_ERROR'. Set error now, in case BULK INSERT aborts
  FROM DiscoveryBatch d 
  INNER JOIN DiscoveryBatchPStatVal v ON v.pstat = d.pstat
  WHERE batchid = @batchid 
    AND FormatType = @formatType 
    AND ActionType = 1
    AND v.name = 'INPROGRESS' 

  IF @@error <> 0
  BEGIN
    RAISERROR('The batch %d could not be moved into an LOAD_ERROR state.',3,1,@batchid) WITH NOWAIT
    RETURN -30000
  END

  EXEC master.dbo.xp_regread 'HKEY_LOCAL_MACHINE',
           'SOFTWARE\Accessible Software, Inc.\Access1\1.0',
           'InstallDirectory',
           @installDir OUTPUT

  -- Lookup file AND codepage of the batch
  SELECT @FilePath = FilePath, 
         @CodePage = CodePage 
  FROM DiscoveryBatch 
  WHERE batchid = @batchid

  IF @FilePath IS NULL OR @CodePage IS NULL
  BEGIN
    RAISERROR('Could not determine FilePath and CodePage for batchid %d',3,1,@batchid) WITH NOWAIT
    RETURN -30000
  END

  SET @FilePath = replace(@FilePath, '/', '\')

  -- Perform bulk load of data
  DELETE FROM XRC_SESSION_PAIRS

  SET @sql = "
  BULK INSERT XRC_SESSION_PAIRS FROM '" + @FilePath + "'
  WITH
  (
    BATCHSIZE = 1000
   ,FORMATFILE = '" + @installDir + "\data\XRC_SESSION_PAIRS.fmt'
  )
  "

  EXEC(@sql)

  SELECT @err = @@ERROR

  IF @err <> 0
  BEGIN
    RAISERROR('Error loading discovery batch file %s', 3, 1, @FilePath)

    UPDATE DiscoveryBatch SET pstat = v.pstat, error = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOAD_ERROR'

    RETURN -30000
  END
  ELSE BEGIN
    RAISERROR('Loaded file %s', 0, 1, @FilePath)

    UPDATE DiscoveryBatch SET pstat = v.pstat, numrows = @err
    FROM DiscoveryBatchPStatVal AS v
    WHERE batchid = @batchid AND v.name = 'LOADED'
  END

GO
