-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/processxrc_sessionpairs.sql, tbsm.sql.porting, tbsm_base31e
-- SID: 1.5.1.3
-- Date Modified: 08/08/15 08:09:12
--
-- _processxrc_sessionpairs_discoverybatch.sql
--
-- Module Description
--   Creates _ProcessXRC_SessionPairs_DiscoveryBatch

USE Object

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_ProcessXRC_SessionPairs_DiscoveryBatch' AND type = 'P')
  DROP PROCEDURE _ProcessXRC_SessionPairs_DiscoveryBatch
GO

PRINT 'creating procedure _ProcessXRC_SessionPairs_DiscoveryBatch'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/processxrc_sessionpairs.sql, tbsm.sql.porting, tbsm_base31e
-- SID: 1.5.1.3
-- Date Modified: 08/08/15 08:09:12
CREATE PROCEDURE _ProcessXRC_SessionPairs_DiscoveryBatch
  @verbose TINYINT=0,
  @timings TINYINT=0
AS

RAISERROR('_ProcessXRC_SessionPairs_DiscoveryBatch starting', 0, 1)

DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN _ProcessXRC_SessionPairs_TRAN
ELSE
  SAVE TRAN _ProcessXRC_SessionPairs_TRAN

DECLARE @now DATETIME
SET @now = CURRENT_TIMESTAMP

CREATE TABLE #xrcsList (
   id ObjID NOT NULL
)

DECLARE @formatType INT
SELECT @formatType = FORMAT_TYPE FROM FORMAT_TYPE WHERE NAME = 'XRC'

DECLARE @step INT
SET @step = 0

DECLARE @batchid INT
SELECT @batchid = batchid 
FROM DiscoveryBatch AS b JOIN DiscoveryBatchPStatVal AS v ON v.pstat = b.pstat
WHERE
  b.FormatType = @formatType
  AND b.ActionType = 1
  AND v.name = 'LOADED'
IF @batchid IS NULL
BEGIN
  PRINT 'There are no loaded XRC SESSION_PAIRS discovery batches waiting to be processed'
  ROLLBACK TRAN _ProcessXRC_SessionPairs_TRAN
  RETURN
END

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| fixing null MasterSessionNames in bulk data'

UPDATE XRC_SESSION_PAIRS
  SET MasterSessionName = '' WHERE MasterSessionName IS NULL

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| fixing null MasterSessionNames in bulk data'

DECLARE @objs INT
DECLARE @idBase ObjID

/* make a list of existing sessions for the operating system(s) matching the discovery data */

CREATE TABLE #sessions (
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  xrcs_id           ObjID NOT NULL
)

/* first work on sessions that are under the xrc folder(s) */

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #sessions (1)'

INSERT INTO #sessions (CPNetName, MasterSessionName, SessionName, xrcs_id)
  SELECT DISTINCT sp.CPNetName, xrcs._MasterSessionName, xrcs.name, xrcs.id
    FROM XRC_SESSION_PAIRS sp
    JOIN OS_V os ON os._SystemId = sp.CPNetName
    JOIN containment c1 ON c1.src_cid = 'LPAR' AND c1.dst_cid = 'OS' AND c1.dst_id = os.id AND c1.deleted = 0
    JOIN containment c2 ON c2.src_cid = 'MACH' AND c2.dst_cid = c1.src_cid AND c2.dst_id = c1.src_id AND c2.deleted = 0
    JOIN containment c3 ON c3.src_cid = 'COMP' AND c3.dst_cid = c2.src_cid AND c3.dst_id = c2.src_id AND c3.deleted = 0
    JOIN containment c4 ON c4.src_cid = 'ENT'  AND c4.dst_cid = c3.src_cid AND c4.dst_id = c3.src_id AND c4.deleted = 0
    JOIN containment c5 ON c5.dst_cid = 'XRCF' AND c5.src_cid = c4.src_cid AND c5.src_id = c4.src_id AND c5.deleted = 0
    JOIN containment c6 ON c6.dst_cid = 'XRCS' AND c6.src_cid = c5.dst_cid AND c6.src_id = c5.dst_id AND c6.deleted = 0
    JOIN XRCSession_V xrcs ON xrcs.id = c6.dst_id

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create #sessions (1)'

/* now work on sessions that are under master sessions */

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #sessions (2)'

INSERT INTO #sessions (CPNetName, MasterSessionName, SessionName, xrcs_id)
  SELECT DISTINCT sp.CPNetName, xrcs._MasterSessionName, xrcs.name, xrcs.id
    FROM XRC_SESSION_PAIRS sp
    JOIN OS_V os ON os._SystemId = sp.CPNetName
    JOIN containment c1 ON c1.src_cid = 'LPAR' AND c1.dst_cid = 'OS' AND c1.dst_id = os.id AND c1.deleted = 0
    JOIN containment c2 ON c2.src_cid = 'MACH' AND c2.dst_cid = c1.src_cid AND c2.dst_id = c1.src_id AND c2.deleted = 0
    JOIN containment c3 ON c3.src_cid = 'COMP' AND c3.dst_cid = c2.src_cid AND c3.dst_id = c2.src_id AND c3.deleted = 0
    JOIN containment c4 ON c4.src_cid = 'ENT'  AND c4.dst_cid = c3.src_cid AND c4.dst_id = c3.src_id AND c4.deleted = 0
    JOIN containment c5 ON c5.dst_cid = 'XRCF' AND c5.src_cid = c4.src_cid AND c5.src_id = c4.src_id AND c5.deleted = 0
    JOIN containment c6 ON c6.dst_cid = 'XRCM' AND c6.src_cid = c5.dst_cid AND c6.src_id = c5.dst_id AND c6.deleted = 0
    JOIN containment c7 ON c7.dst_cid = 'XRCS' AND c7.src_cid = c6.dst_cid AND c7.src_id = c6.dst_id AND c7.deleted = 0
    JOIN XRCSession_V xrcs ON xrcs.id = c7.dst_id

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create #sessions (2)'

IF @verbose <> 0
  SELECT '#sessions', * FROM #sessions

/* process storage control sessions */

CREATE TABLE #existing_xscs (
  id                ObjID NOT NULL,
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  name              NVARCHAR(2) NOT NULL,
  CONSTRAINT existing_xscs_PK PRIMARY KEY (id, CPNetName, MasterSessionName, SessionName, name)
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #existing_xscs'

INSERT INTO #existing_xscs
  SELECT DISTINCT xscs.id, xscs._CPNetName, xscs._MasterSessionName, xscs._SessionName, xscs.name
    FROM #sessions sessions
    JOIN containment c ON c.src_cid = 'XRCS' AND c.src_id = sessions.xrcs_id AND c.dst_cid = 'XSCS' AND c.deleted = 0
    JOIN XRCControlSession_V xscs ON xscs.id = c.dst_id

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end |' + CAST(@step as VARCHAR) + '| create #existing_xscs'

IF @verbose <> 0
  SELECT '#existing_xscs', * FROM #existing_xscs

CREATE TABLE #new_xscs (
  id                ObjID IDENTITY(1,1) NOT NULL,
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  name              NVARCHAR(2) NOT NULL,
  parent_id         ObjID NOT NULL
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #new_xscs'

INSERT INTO #new_xscs (CPNetName, MasterSessionName, SessionName, name, parent_id)
  SELECT DISTINCT sp.CPNetName, sp.MasterSessionName, sp.SessionName, sp.PrimarySCSN, sessions.xrcs_id
    FROM XRC_SESSION_PAIRS sp
    JOIN #sessions sessions ON sessions.CPNetName = sp.CPNetName AND sessions.MasterSessionName = sp.MasterSessionName
         AND sessions.SessionName = sp.SessionName
    LEFT OUTER JOIN #existing_xscs exscs ON exscs.CPNetName = sp.CPNetName AND exscs.MasterSessionName = sp.MasterSessionName
         AND exscs.SessionName = sp.SessionName
  WHERE exscs.CPNetName IS NULL

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create #new_xscs'

IF @verbose <> 0
  SELECT '#new_xscs', * FROM #new_xscs

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create new xscs'

SELECT @objs = COUNT(*) FROM #new_xscs
IF @objs > 0 BEGIN
  UPDATE XSCS_ID SET last_id = last_id + @objs
  SELECT @idBase = last_id - @objs FROM XSCS_ID

  INSERT INTO XRCControlSession_C (id, name, _CPNetName, _MasterSessionName, _SessionName, _Discovered)
    SELECT @idBase + id, name, CPNetName, MasterSessionName, SessionName, @now
      FROM #new_xscs

  INSERT INTO XRCS_XSCS_PHYC (src_id, dst_id)
    SELECT parent_id, @idBase + id
      FROM #new_xscs

  /* we need to have an up-to-date list of control sessions so we can process volume pairs and utility volumes */

  INSERT INTO #existing_xscs
     SELECT @idBase + id, CPNetName, MasterSessionName, SessionName, name FROM #new_xscs

  IF @verbose <> 0
    SELECT 'updated #existing_xscs', * FROM #existing_xscs
END

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create new xscs'

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| update xscs'

UPDATE o SET
    _HostDiscovery      = sp.DiscoveryTimestamp,
    _LastDiscovered     = @now,
    _SystemName         = sp.SystemName
  FROM XRCControlSession_C o
  JOIN containment c ON c.dst_cid = 'XSCS' AND c.dst_id = o.id AND c.deleted = 0
  JOIN XRC_SESSION_PAIRS sp ON o._CPNetName = sp.CPNetName AND o.name = sp.PrimarySCSN AND o._MasterSessionName = sp.MasterSessionName
                           AND o._SessionName = sp.SessionName

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end |' + CAST(@step as VARCHAR) + '| update xscs'

/* process volume pairs */

CREATE TABLE #existing_xrcp (
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  ControlSession    NVARCHAR(2) NOT NULL,
  name              ObjName NOT NULL,
  CONSTRAINT existing_xrcp_PK PRIMARY KEY (CPNetName, MasterSessionName, SessionName, ControlSession, name)
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #existing_xrcp'

INSERT INTO #existing_xrcp
  SELECT xrcp._CPNetName, xrcp._MasterSessionName, xrcp._SessionName, xrcp._PrimarySCSN, xrcp.name
    FROM #existing_xscs xscs
    JOIN containment c ON c.src_cid = 'XSCS' AND c.src_id = xscs.id AND c.dst_cid = 'XRCP' AND c.deleted = 0
    JOIN XRCVolumePair_V xrcp ON xrcp.id = c.dst_id

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create #existing_xrcp'

IF @verbose <> 0
  SELECT '#existing_xrcp', * FROM #existing_xrcp

CREATE TABLE #new_xrcp (
  id                ObjID IDENTITY(1,1) NOT NULL,
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  ControlSession    NVARCHAR(2) NOT NULL,
  SynchStatus       NVARCHAR(3),
  name              ObjName NOT NULL,
  parent_id         ObjID NOT NULL
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #new_xrcp'

INSERT INTO #new_xrcp (CPNetName, MasterSessionName, SessionName, ControlSession, SynchStatus, name, parent_id)
  SELECT sp.CPNetName, sp.MasterSessionName, sp.SessionName, sp.PrimarySCSN, sp.SynchStatus, 
         sp.PrimaryVolume + ',' + sp.SecondaryVolume, xscs.id
  FROM XRC_SESSION_PAIRS sp
  JOIN #existing_xscs xscs ON xscs.CPNetName = sp.CPNetName AND xscs.MasterSessionName = sp.MasterSessionName AND
                              xscs.SessionName = sp.SessionName AND xscs.name = sp.PrimarySCSN
  LEFT OUTER JOIN #existing_xrcp exrcp ON 
                                 exrcp.CPNetName          = sp.CPNetName AND 
                                 exrcp.MasterSessionName  = sp.MasterSessionName AND
                                 exrcp.SessionName        = sp.SessionName AND
                                 exrcp.ControlSession     = sp.PrimarySCSN AND
                                 exrcp.name               = sp.PrimaryVolume + ',' + sp.SecondaryVolume
  WHERE sp.SecondaryVolume <> 'XRCUTL' AND sp.SynchStatus <> 'UTL' AND exrcp.CPNetName IS NULL

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end |' + CAST(@step as VARCHAR) + '| create #new_xrcp'

IF @verbose <> 0
  SELECT '#new_xrcp', * FROM #new_xrcp

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create new xrcp'

SELECT @objs = COUNT(*) FROM #new_xrcp
IF @objs > 0 BEGIN
  UPDATE XRCP_ID SET last_id = last_id + @objs
  SELECT @idBase = last_id - @objs FROM XRCP_ID

  INSERT INTO XRCVolumePair_C (id, name, _CPNetName, _MasterSessionName, _SessionName, _PrimarySCSN, _SynchStatus, _Discovered)
    SELECT @idBase + id, name, CPNetName, MasterSessionName, SessionName, ControlSession, SynchStatus, @now
      FROM #new_xrcp

  INSERT INTO XSCS_XRCP_LSTV (src_id, dst_id)
    SELECT parent_id, @idBase + id
      FROM #new_xrcp

  /* recalculate xrc session counters */

  INSERT INTO #xrcsList
    SELECT DISTINCT c.src_id
      FROM #new_xrcp xrcp
      JOIN containment c ON c.dst_cid = 'XSCS' AND c.dst_id = xrcp.parent_id AND c.src_cid = 'XRCS' AND c.deleted = 0

  EXEC _UpdateXRCSPairCounts 'ROOT', 0, @verbose=@verbose, @timings=@timings
END

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create new xrcp'

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| update xrcp'

UPDATE o SET
    _HostDiscovery      = sp.DiscoveryTimestamp,
    _LastDiscovered     = @now,
    _PairErrorLevel     = sp.PairErrorLevel,
    _PrimaryCCA         = sp.PrimaryCCA,
    _PrimaryDeviceID    = sp.PrimaryDeviceID,
    _PrimarySCID        = sp.PrimarySCID,
    _PrimarySSID        = sp.PrimarySSID,
    _PrimaryVolume      = sp.PrimaryVolume,
    _SecondaryCCA       = sp.SecondaryCCA,
    _SecondaryDeviceID  = sp.SecondaryDeviceID,
    _SecondarySSID      = sp.SecondarySSID,
    _SecondaryVolume    = sp.SecondaryVolume,
    _SystemName         = sp.SystemName
  FROM XRCVolumePair_C o
  JOIN containment c ON c.dst_cid = 'XRCP' AND c.dst_id = o.id AND c.deleted = 0
  JOIN XRC_SESSION_PAIRS sp ON o._CPNetName = sp.CPNetName AND o._PrimarySCSN = sp.PrimarySCSN AND 
                               o._MasterSessionName = sp.MasterSessionName AND
                               o._SessionName = sp.SessionName AND o.name = sp.PrimaryVolume + ',' + sp.SecondaryVolume

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| update xrcp'

/* process utility volumes */

CREATE TABLE #existing_xrcu (
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  ControlSession    NVARCHAR(2) NOT NULL,
  name              ObjName NOT NULL
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #existing _xrcu'

INSERT INTO #existing_xrcu
  SELECT xrcu._CPNetName, xrcu._MasterSessionName, xrcu._SessionName, xrcu._PrimarySCSN, xrcu.name
    FROM #existing_xscs xscs
    JOIN containment c ON c.src_cid = 'XSCS' AND c.src_id = xscs.id AND c.dst_cid = 'XRCU' AND c.deleted = 0
    JOIN XRCUtilityVolume_V xrcu ON xrcu.id = c.dst_id

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end |' + CAST(@step as VARCHAR) + '| create #existing _xrcu'

IF @verbose <> 0
  SELECT '#existing_xrcu', * FROM #existing_xrcu

CREATE TABLE #new_xrcu (
  id                ObjID IDENTITY(1,1) NOT NULL,
  CPNetName         NVARCHAR(17) NOT NULL,
  MasterSessionName NVARCHAR(8),
  SessionName       NVARCHAR(8) NOT NULL,
  ControlSession    NVARCHAR(2) NOT NULL,
  name              ObjName NOT NULL,
  parent_id         ObjID NOT NULL
)

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create #new_xrcu'

INSERT INTO #new_xrcu (CPNetName, MasterSessionName, SessionName, ControlSession, name, parent_id)
  SELECT sp.CPNetName, sp.MasterSessionName, sp.SessionName, sp.PrimarySCSN, sp.PrimaryVolume, xscs.id
  FROM XRC_SESSION_PAIRS sp
  JOIN #existing_xscs xscs ON xscs.CPNetName = sp.CPNetName AND xscs.MasterSessionName = sp.MasterSessionName AND
                              xscs.SessionName = sp.SessionName AND xscs.name = sp.PrimarySCSN
  LEFT OUTER JOIN #existing_xrcu exrcu ON 
                                 exrcu.CPNetName          = sp.CPNetName AND 
                                 exrcu.MasterSessionName  = sp.MasterSessionName AND
                                 exrcu.SessionName        = sp.SessionName AND
                                 exrcu.ControlSession     = sp.PrimarySCSN AND
                                 exrcu.name               = sp.PrimaryVolume
  WHERE sp.SecondaryVolume = 'XRCUTL' AND sp.SynchStatus = 'UTL' AND exrcu.CPNetName IS NULL

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create #new_xrcu'

IF @verbose <> 0
  SELECT '#new_xrcu', * FROM #new_xrcu

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| create xrcu'

SELECT @objs = COUNT(*) FROM #new_xrcu
IF @objs > 0 BEGIN
  UPDATE XRCU_ID SET last_id = last_id + @objs
  SELECT @idBase = last_id - @objs FROM XRCU_ID

  INSERT INTO XRCUtilityVolume_C (id, name, _CPNetName, _MasterSessionName, _SessionName, _PrimarySCSN, _Discovered)
    SELECT @idBase + id, name, CPNetName, MasterSessionName, SessionName, ControlSession, @now
      FROM #new_xrcu

  INSERT INTO XSCS_XRCU_PHYC (src_id, dst_id)
    SELECT parent_id, @idBase + id
      FROM #new_xrcu

  /* recalculate xrc session counter */

  DELETE #xrcsList

  INSERT INTO #xrcsList
    SELECT DISTINCT c.src_id
      FROM #new_xrcu xrcu
      JOIN containment c ON c.dst_cid = 'XSCS' AND c.dst_id = xrcu.parent_id AND c.src_cid = 'XRCS' AND c.deleted = 0

  EXEC _UpdateXRCSUtilityCounts 'ROOT', 0
END

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| create xrcu'

SET @step = @step + 1
IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|begin|' + CAST(@step as VARCHAR) + '| update xrcu'

UPDATE o SET
    _HostDiscovery      = sp.DiscoveryTimestamp,
    _LastDiscovered     = @now,
    _PrimaryCCA         = sp.PrimaryCCA,
    _PrimaryDeviceID    = sp.PrimaryDeviceID,
    _PrimarySSID        = sp.PrimarySSID,
    _PrimarySCID        = sp.PrimarySCID,
    _PrimaryVolume      = sp.PrimaryVolume,
    _SystemName         = sp.SystemName
  FROM XRCUtilityVolume_C o
  JOIN containment c ON c.dst_cid = 'XRCU' AND c.dst_id = o.id AND c.deleted = 0
  JOIN XRC_SESSION_PAIRS sp ON o._CPNetName = sp.CPNetName AND o._PrimarySCSN = sp.PrimarySCSN AND 
                               o._MasterSessionName = sp.MasterSessionName AND
                               o._SessionName = sp.SessionName AND o.name = sp.PrimaryVolume

IF @timings = 1
  PRINT CONVERT(NVARCHAR, GETDATE(), 114) + '|_ProcessXRC_SessionPairs_DiscoveryBatch|end  |' + CAST(@step as VARCHAR) + '| update xrcu'

DROP TABLE #xrcsList

/* ----- we're done! ----- */

UPDATE DiscoveryBatch SET pstat = v.pstat, numrows = 0
FROM DiscoveryBatchPStatVal AS v
WHERE batchid = @batchid AND v.name = 'COMPLETED'

IF @tcount = 0
  COMMIT TRAN _ProcessXRC_SessionPairs_TRAN
GO

PRINT 'Done'
GO
