-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 1999,2007 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/notehistorytable.sql, tbsm_base, tbsm_base31e
-- SID: 1.6
-- Date Modified: 08/02/06 08:35:01
--
-- Module Description:
--   Creates the NoteHistory table and the NoteHistoryDetail table
--

USE Object
GO

  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'NoteHistory')
  BEGIN
    PRINT 'Creating table NoteHistory'
    CREATE TABLE NoteHistory
    (
      noteid            INT            NOT NULL,
      Action            NVARCHAR(255)  NOT NULL,
      Executer          NVARCHAR(255)      NULL,
      TimeExecuted      DATETIME       NOT NULL DEFAULT (GETDATE()),       
      ident             INT            NOT NULL IDENTITY(1,1),
     
      CONSTRAINT        NoteHistory_pk PRIMARY KEY (ident),
      CONSTRAINT        NoteHistory_fk FOREIGN KEY (noteid) REFERENCES Note_C(id) ON DELETE CASCADE
    )
    IF @@ERROR <> 0
    BEGIN
      PRINT 'ERROR creating table NoteHistory'
      RETURN
    END
  END 
  ELSE BEGIN 
    -- Ensure that NoteHistory_fk foreign key constraint has "ON DELETE CASCADE" attribute.  This will
    -- cause SQL SERVER to automatically delete NoteHistory rows when rows are deleted from Note_C.
    DECLARE @id INT
    SELECT @id = id FROM sysobjects WHERE name = 'NoteHistory_fk' AND type = 'F'
    IF @id IS NOT NULL AND ObjectProperty(@id, 'CnstIsDeleteCascade') <> 1 BEGIN
      PRINT 'Dropping NoteHistory_fk constraint because it does not have ON DELETE CASCADE attribute'
      ALTER TABLE NoteHistory DROP CONSTRAINT NoteHistory_fk
    END
    IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'NoteHistory_fk' AND type = 'F') BEGIN
      PRINT 'Adding NoteHistory_fk constraint with ON DELETE CASCADE attribute'
      ALTER TABLE NoteHistory ADD  CONSTRAINT NoteHistory_fk FOREIGN KEY (noteid) REFERENCES Note_C(id) ON DELETE CASCADE
    END
  END

  IF NOT EXISTS (SELECT * FROM sysindexes WHERE name = 'NoteHistory_NoteId')
  BEGIN
    CREATE INDEX NoteHistory_NoteId ON NoteHistory ( noteid, ident )
  END

  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'NoteHistoryDetail')
  BEGIN
    PRINT 'Creating table NoteHistoryDetail'
    CREATE TABLE NoteHistoryDetail
    (
      NoteHistory_id  INT            NOT NULL,
      Subject         NVARCHAR(255)      NULL,
      Body            NTEXT              NULL,
      Resolution      NTEXT              NULL,
      UserID          NVARCHAR(255)      NULL,
      ProblemID       NVARCHAR(255)      NULL,
      Remarks         NVARCHAR(2024)     NULL,
      ident           INT            NOT NULL IDENTITY(1,1),
     
      CONSTRAINT        NoteHistoryDetail_pk PRIMARY KEY (ident),
      CONSTRAINT        NoteHistoryDetail_fk FOREIGN KEY (NoteHistory_id) REFERENCES NoteHistory(ident) ON DELETE CASCADE
    )
    IF @@ERROR <> 0
    BEGIN
      PRINT 'ERROR creating table NoteHistoryDetail'
      RETURN
    END
  END 

GO

