-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
--  Copyright IBM Corp. 2006 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tracks/oa26022.sql, tbsm, tbsm_base31e
-- SID: 1.3
-- Date Modified: 08/08/20 19:02:26
--
-- Module Description
--  Part 1
--  Fix the processing_sql for XRC Generic Traps in GenericEventTypeMap
--  Part 2
--  Swop the PrimarySCID and PrimarySCSN for XRC Volume Pairs and Utility Volumes and
--  update the name column of XRC Storage Control Sessions.    
-----------------------------------------------------------------------------------------

USE Object
GO
SET NOCOUNT ON
GO
BEGIN TRAN
GO
DECLARE @processing_sql NVARCHAR(300)	
DECLARE @description NVARCHAR(255)	--Primary Key
DECLARE @stringPos INT
DECLARE @rc INT
DECLARE @err INT
DECLARE @msg NVARCHAR(1024)
DECLARE @continue INT

SET @continue=1

SELECT description, processing_sql FROM GenericEventTypeMap WHERE processing_sql like '%asisp_process_XRC_Event%@_attrSet=%=%'

WHILE EXISTS ( SELECT * FROM GenericEventTypeMap WHERE processing_sql like '%asisp_process_XRC_Event%@_attrSet=''xrcp.%' ) AND @continue=1
BEGIN
  SELECT @description = description, @processing_sql = processing_sql
  FROM GenericEventTypeMap
  WHERE processing_sql like '%asisp_process_XRC_Event%@_attrSet=''xrcp.%'

  SELECT @stringPos = CHARINDEX('@_attrSet=''xrcp.',@processing_sql,0)
  IF @stringPos > 0
  BEGIN
    UPDATE GenericEventTypeMap
    SET processing_sql = SUBSTRING(@processing_sql,1,@stringPos+10) + SUBSTRING(@processing_sql,@stringPos+16,LEN(@processing_sql))
    WHERE @description = description
    SELECT @rc = @@rowcount, @err = @@error

    IF @err = 0
    BEGIN
      SELECT @msg = 'Updated ' + convert(nvarchar(10),@rc) + ' row(s) in GenericEventTypeMap.'
      RAISERROR('%s',0,1,@msg) WITH NOWAIT
    END
    ELSE
    BEGIN
      SET @continue=0
      RAISERROR('An error occured updating the GenericEventTypeMap table. No rows updated.',3,1,@msg) WITH NOWAIT
   END
  END
  ELSE
  BEGIN
    SET @continue=0
    RAISERROR('An unexpected error occured when searching for update string in the GenericEventTypeMap table.',3,1,@msg) WITH NOWAIT
  END
END

SELECT description, processing_sql FROM GenericEventTypeMap WHERE processing_sql like '%asisp_process_XRC_Event%@_attrSet=%=%'

IF @continue=0
  ROLLBACK TRAN
ELSE
  COMMIT TRAN
GO


--  Part 2
--  Swop the PrimarySCID and PrimarySCSN for XRC Volume Pairs and Utility Volumes and
--  update the name column of XRC Storage Control Sessions.    

USE Object
GO

SET NOCOUNT ON
GO

DECLARE @err INT
DECLARE @msg NVARCHAR(1024)
DECLARE @continue INT
DECLARE @primarySCID NVARCHAR(2)
DECLARE @primarySCSN NVARCHAR(2)

CREATE TABLE #IDsToSwop (PrimarySCID NVARCHAR(2), PrimarySCSN NVARCHAR(2))

INSERT INTO #IDsToSwop 
  SELECT _PrimarySCID, _PrimarySCSN 
  FROM XRCVolumePair_C
  WHERE deleted = 0
  AND _PrimarySCID IS NOT NULL
  AND _PrimarySCSN IS NOT NULL
UNION
  SELECT _PrimarySCID, _PrimarySCSN 
  FROM XRCUtilityVolume_C
  WHERE deleted = 0
  AND _PrimarySCID IS NOT NULL
  AND _PrimarySCSN IS NOT NULL

SET @continue=1

WHILE EXISTS ( SELECT * FROM #IDsToSwop ) AND @continue=1
BEGIN
  SET ROWCOUNT 1
  SELECT @primarySCID = PrimarySCID, @primarySCSN = PrimarySCSN
  FROM #IDsToSwop 
  SET ROWCOUNT 0

  SET @msg = 'Swopping PrimarySCID = ' +  @primarySCID + ' and PrimarySCSN ' +  @primarySCSN + '. Correct values are PrimarySCID = ' +  @primarySCSN + ' and PrimarySCSN ' +  @primarySCID + '.'
  RAISERROR (@msg,0,1) WITH NOWAIT

  BEGIN TRAN

  UPDATE XRCVolumePair_C
  SET _PrimarySCSN = @primarySCID, _PrimarySCID = @primarySCSN
  WHERE deleted = 0
  AND _PrimarySCID = @primarySCID AND _PrimarySCSN = @primarySCSN

  IF @@error <> 0
  BEGIN
    SET @continue = 0
    RAISERROR ('An error occured updating XRCVolumePair_C. Aborting script oa26022.sql',3,1) WITH NOWAIT
    ROLLBACK TRAN
  END
  ELSE
  BEGIN
    UPDATE XRCUtilityVolume_C
    SET _PrimarySCSN = @primarySCID, _PrimarySCID = @primarySCSN
    WHERE deleted = 0
    AND _PrimarySCID = @primarySCID AND _PrimarySCSN = @primarySCSN

    IF @@error <> 0
    BEGIN
      SET @continue = 0
      RAISERROR ('An error occured updating XRCUtilityVolume_C. Aborting script oa26022.sql',3,1) WITH NOWAIT
      ROLLBACK TRAN
    END
    ELSE
    BEGIN
      UPDATE XRCControlSession_C
      SET name = @primarySCID
      WHERE deleted = 0
      AND name = @primarySCSN

      IF @@error <> 0
      BEGIN
        SET @continue = 0
        RAISERROR ('An error occured updating XRCControlSession_C. Aborting script oa26022.sql',3,1) WITH NOWAIT
        ROLLBACK TRAN
      END
    END
  END
  
  IF @continue = 1
    COMMIT TRAN

  DELETE FROM #IDsToSwop WHERE @primarySCID = PrimarySCID AND @primarySCSN = PrimarySCSN
END
GO
